/* Lirc Infra Red Controller Plugin
        Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
        code from gtuner lirc plugin
        cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "lirc.h"
#include "lirc_image.h"

static gboolean configured = FALSE;

void load_config()
{
	GList *node;
	gchar *filename;
	ConfigFile *cfgfile;
	int i;

	filename=g_strconcat(g_get_home_dir(),"/.gtunerrc",NULL);

	if (device) free (device);

        node=lirc_codes;
        while(node)
        {
	    if (node->data)
                g_free(node->data);
            node=node->next;
        }
        g_list_free(lirc_codes);

	for (i=0; i<num_codes; i++)
	{
	    struct lirc_code *tc;
	    tc = malloc(sizeof(struct lirc_code));
	    if (tc==NULL)
	    {
		printf("LIRC: Memory allocation error\n");
		continue;
	    }

	    tc->function = g_strdup(code_list[i]);
	    tc->action = i;
	    tc->code = 0;
	    tc->remote = 0;
	    lirc_codes = g_list_append(lirc_codes, tc);
	}

	if((cfgfile=gtuner_cfg_open_file(filename)))
	{
	    gtuner_cfg_read_string(cfgfile,"LIRC","device", &device);
	    for (i=0; i<num_codes; i++)
	    {
		struct lirc_code *tc;
		tc = g_list_nth(lirc_codes, i)->data;
		if (!tc)
		{   
		    printf("Lirc Infra Red Controller: Fatal error\n");
		    exit(1);
		}

		gtuner_cfg_read_string(cfgfile,"LIRC_CODE",code_list[i],&tc->code);
		gtuner_cfg_read_string(cfgfile,"LIRC_REMOTE",code_list[i],&tc->remote);
	    }
	    gtuner_cfg_free(cfgfile);
	}
	g_free(filename);
	if(!device)
	    device=g_strdup("/dev/lircd");

	for (i=0; i<num_codes; i++)
	{
	    struct lirc_code *tc;

	    tc = g_list_nth(lirc_codes, i)->data;
	    if (!tc)
	    {
		printf("Lirc Infra Red Controller: Fatal error\n");
		exit(1);
	    }

	    if (!tc->code) tc->code = g_strdup("none");
	    if (!tc->remote) tc->remote = g_strdup("none");
	}
    configured = TRUE;
}

static GtkWidget *configure_win=0,*vbox;
static GtkWidget *dev_frame, *dev_entry;
static GtkWidget *codes_frame, *codes_list, *scrolled;
static GtkWidget *codes_vbox, *codes_hbox;
static GtkWidget *codes_manual, *codes_probe, *codes_clear;
static GtkWidget *bbox,*ok,*cancel;
void insert_rows();
void probe_remote(GtkWidget *w,gpointer data);
void save_settings(GtkWidget *w,gpointer data);
void choose_codes(GtkWidget *w,gpointer data);
void clear_codes(GtkWidget *w,gpointer data);


void configure()
{
    if (!configured)
    {
	GtkWidget *dialog,*button,*label;
	dialog=gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog),
		"Error");

	gtk_widget_realize(dialog);
	label=gtk_label_new("You must enable the plugin so that\n\
the default settings can be loaded");
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),label,TRUE,TRUE,5);
	gtk_widget_show(label);

	button=gtk_button_new_with_label(" Close ");
	gtk_signal_connect_object(GTK_OBJECT(button),"clicked",
		GTK_SIGNAL_FUNC(gtk_widget_destroy),GTK_OBJECT(dialog));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
						button,FALSE,FALSE,0);
	gtk_widget_show (button);

	gtk_widget_show(dialog);
	gtk_widget_grab_focus(button);
    }
    else
    if (!configure_win)
    {
	GtkWidget *pixmapwid;
	GdkPixmap *pixmap;
	GdkBitmap *mask;
	GtkStyle *style;

	configure_win = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_container_border_width(GTK_CONTAINER(configure_win),5);
	gtk_signal_connect(GTK_OBJECT(configure_win),"destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroyed),&configure_win);
	gtk_window_set_title(GTK_WINDOW(configure_win),
			"LIRC Infra Red Controller Configuration");

	vbox = gtk_vbox_new(FALSE, 5);

        gtk_container_add(GTK_CONTAINER(configure_win), vbox);
	gtk_widget_realize(configure_win);

	pixmap = gdk_pixmap_create_from_xpm_d (configure_win->window,&mask,
					     &style->bg[GTK_STATE_NORMAL],
					     (gchar **) lirc_image);
	pixmapwid = gtk_pixmap_new (pixmap, mask);
	gtk_box_pack_start (GTK_BOX(vbox), pixmapwid, FALSE, FALSE, 0);

	dev_frame=gtk_frame_new("Lirc Device:");
	gtk_container_add(GTK_CONTAINER(vbox),dev_frame);

	dev_entry=gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(dev_entry), device);
	gtk_container_add(GTK_CONTAINER(dev_frame),dev_entry);

	codes_vbox = gtk_vbox_new(FALSE, 5);
	codes_hbox = gtk_hbox_new(FALSE, 5);
	gtk_container_border_width(GTK_CONTAINER(codes_vbox),5);

	codes_frame=gtk_frame_new("Remote Codes:");
	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_set_usize(scrolled, -1, 200);
	gtk_container_add(GTK_CONTAINER(vbox),codes_frame);
	gtk_container_add(GTK_CONTAINER(codes_frame),codes_vbox);
	gtk_box_pack_end(GTK_BOX(codes_vbox), codes_hbox, FALSE, FALSE, 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
					GTK_POLICY_AUTOMATIC,GTK_POLICY_ALWAYS);

	codes_list=gtk_clist_new(3);
	gtk_clist_set_column_title(GTK_CLIST(codes_list), 0, "Command");
	gtk_clist_set_column_title(GTK_CLIST(codes_list), 1, "Code");
	gtk_clist_set_column_title(GTK_CLIST(codes_list), 2, "Remote");
	gtk_clist_column_titles_show(GTK_CLIST(codes_list));
	gtk_clist_set_column_width (GTK_CLIST(codes_list), 0, 75);
	gtk_clist_set_column_width (GTK_CLIST(codes_list), 1, 75);
	gtk_clist_set_column_width (GTK_CLIST(codes_list), 2, 75);
	gtk_clist_column_titles_passive (GTK_CLIST(codes_list));

	insert_rows();

	gtk_container_add(GTK_CONTAINER(scrolled), codes_list);
	gtk_container_add(GTK_CONTAINER(codes_vbox), scrolled);

	codes_manual = gtk_button_new_with_label("Choose Code");
	codes_probe = gtk_button_new_with_label("Capture Code");
	codes_clear = gtk_button_new_with_label("Clear Code");

	gtk_container_add(GTK_CONTAINER(codes_hbox), codes_manual);
	gtk_container_add(GTK_CONTAINER(codes_hbox), codes_probe);
	gtk_container_add(GTK_CONTAINER(codes_hbox), codes_clear);
	gtk_signal_connect(GTK_OBJECT(codes_probe),"clicked",
				GTK_SIGNAL_FUNC(probe_remote),NULL);
	gtk_signal_connect(GTK_OBJECT(codes_manual),"clicked",
				GTK_SIGNAL_FUNC(choose_codes),NULL);
	gtk_signal_connect(GTK_OBJECT(codes_clear),"clicked",
				GTK_SIGNAL_FUNC(clear_codes),NULL);

	bbox=gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox),GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox),5);
	gtk_box_pack_end(GTK_BOX(vbox),bbox,FALSE,FALSE,0);

	ok=gtk_button_new_with_label("Ok");
	gtk_signal_connect(GTK_OBJECT(ok),"clicked",
				GTK_SIGNAL_FUNC(save_settings), NULL);

	GTK_WIDGET_SET_FLAGS(ok,GTK_CAN_DEFAULT);
	gtk_box_pack_start(GTK_BOX(bbox),ok,TRUE,TRUE,0);
	gtk_widget_show(ok);
	gtk_widget_grab_default(ok);

	cancel=gtk_button_new_with_label("Cancel");
	gtk_signal_connect_object(GTK_OBJECT(cancel),"clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy),GTK_OBJECT(configure_win));
	GTK_WIDGET_SET_FLAGS(cancel,GTK_CAN_DEFAULT);
	gtk_box_pack_start(GTK_BOX(bbox),cancel,TRUE,TRUE,0);
	gtk_widget_show(cancel);

	gtk_widget_show_all(configure_win);
    }
}

char *rm_under_score(char *text)
{
    char *new, *temp;

    new = g_strdup(text);
    temp = strchr(new, '_');

    while (temp != NULL)
    {
	if (temp) temp[0] = ' ';
	temp = strchr(new, '_');
    }
    return new;
}

void insert_rows()
{
    int i;
    gchar *row[3];

    for (i=0; i<num_codes; i++)
    {
	struct lirc_code *tc;

	tc = g_list_nth(lirc_codes, i)->data;
	if (!tc)
	{
	    printf("Lirc Infra Red Controller: Fatal error\n");
	    exit(1);
	}

	row[0] = rm_under_score(tc->function);
	row[1] = rm_under_score(tc->code);
	row[2] = rm_under_score(tc->remote);
	gtk_clist_insert(GTK_CLIST(codes_list), i, row);
	g_free(row[0]);
	g_free(row[1]);
	g_free(row[2]);
    }
}

extern int lirc_fd;
gchar *description;

void probe_remote(GtkWidget *w,gpointer data)
{
    if(GTK_CLIST(codes_list)->selection)
    {
        capture = 1;
	gtk_widget_set_sensitive(vbox, FALSE);
    }
}

void got_remote(char *new_code, char *new_remote)
{
    if(GTK_CLIST(codes_list)->selection)
    {
	gint row;
	gchar *text;
	gchar *new[3];

	capture = 0;
	row= (gint)GTK_CLIST(codes_list)->selection->data;

	if (!new_code)
	    return;
	gtk_widget_set_sensitive(vbox, TRUE);

	gtk_clist_get_text(GTK_CLIST(codes_list), row, 0, &text);
	new[0] = g_strdup(text);
	new[1] = g_strdup(new_code);
	new[2] = g_strdup(new_remote);
        gtk_clist_remove(GTK_CLIST(codes_list), row);
	gtk_clist_insert(GTK_CLIST(codes_list), row, new);
    }
}

static GtkWidget *remote_tree = 0;

void send_command (char *msg)
{
	int todo, done;
	const char *buffer;

	if ((msg==NULL) || (lirc_fd==-1))
	    return;

        todo=strlen(msg);
	buffer=(const char *) msg;

        while(todo>0)
        {
                done=write(lirc_fd,(void *) buffer,todo);
                if(done<0)
                {
                        return;
                }
                buffer+=done;
                todo-=done;
        }
	usleep(1000);
}

GtkWidget *tree_dialog = 0;

void tree_cancel(GtkWidget *widget, GtkWidget **to_delete)
{
    gtk_widget_destroy(*to_delete);
    gtk_widget_destroyed(widget, to_delete);
    gtk_widget_set_sensitive(vbox, TRUE);
    reset();
}

static void tree_ok (GtkWidget *widget, GtkWidget **to_delete)
{
    gchar *name, *text;
    GtkLabel *label;
    int row;
    char *new[3];
    GtkWidget *item;

    if (!(GTK_TREE(remote_tree)->selection))
	return;
    item = GTK_TREE_SELECTION(remote_tree)->data;
    if (!item) return;

    label = GTK_LABEL (GTK_BIN (item)->child);
    gtk_label_get (label, &name);

    row= (gint)GTK_CLIST(codes_list)->selection->data;

    gtk_clist_get_text(GTK_CLIST(codes_list), row, 0, &text);
    new[0] = g_strdup(text);
    new[1] = g_strdup(name);

    item = GTK_TREE(item->parent)->tree_owner;
    label = GTK_LABEL (GTK_BIN (item)->child);
    gtk_label_get (label, &name);

    new[2] = g_strdup(name);

    gtk_clist_remove(GTK_CLIST(codes_list), row);
    gtk_clist_insert(GTK_CLIST(codes_list), row, new);

    tree_cancel(widget, to_delete);
}

void draw_tree()
{
    GtkWidget *t_scrolled, *t_cancel, *t_ok, *t_bbox;

    if (!tree_dialog)
    {
	tree_dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(tree_dialog), "Select a Code");
	gtk_signal_connect(GTK_OBJECT(tree_dialog),"destroy",
			GTK_SIGNAL_FUNC(tree_cancel),&tree_dialog);

        t_scrolled = gtk_scrolled_window_new(NULL, NULL);
        gtk_widget_set_usize(t_scrolled, 200, 300);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(t_scrolled),
						remote_tree);
	gtk_tree_set_selection_mode (GTK_TREE(remote_tree),
                                   GTK_SELECTION_SINGLE);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(tree_dialog)->vbox),
			t_scrolled);

	t_bbox=gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(t_bbox),GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing(GTK_BUTTON_BOX(t_bbox),5);
	gtk_box_pack_end(GTK_BOX(GTK_DIALOG(tree_dialog)->action_area),
			t_bbox,FALSE,FALSE,0);

	t_cancel = gtk_button_new_with_label("Cancel");
	t_ok = gtk_button_new_with_label("OK");
	gtk_signal_connect(GTK_OBJECT(t_cancel),"clicked",
		GTK_SIGNAL_FUNC(tree_cancel),&tree_dialog);
	gtk_signal_connect(GTK_OBJECT(t_ok),"clicked",
		GTK_SIGNAL_FUNC(tree_ok),&tree_dialog);

	gtk_box_pack_start(GTK_BOX(t_bbox), t_ok, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(t_bbox), t_cancel, FALSE, FALSE, 0);
	gtk_widget_show_all(tree_dialog);
    }
}

void choose_codes(GtkWidget *w,gpointer data)
{
    if (!tree_dialog)
    if (lirc_fd)
    if(GTK_CLIST(codes_list)->selection)
    {
        gint i, j, count1, count2;
	gchar *buf;
	GtkWidget *tree_item;
	struct lirc_packet *pkt;
	GList *remotes=0, *test;

//	if (remote_tree) gtk_widget_destroy(remote_tree);
	remote_tree = gtk_tree_new();
	gtk_widget_set_sensitive(vbox, FALSE);
        draw_tree();

	reset_packets();
	send_command("LIST\n");

	pkt = get_next_packet();
	if (!pkt)
	{
	    printf("LIRC: Daemon timed out\n");
	    return;
	}

	if (pkt->status == R_ERROR)
	{
	    printf("LIRC: Daemon returned an error:\n");
	    test = pkt->data;
	    while (test != NULL)
	    {
		if (test->data)
		    printf("-> %s\n", (char*)test->data);
		test = test->next;
	    }
	    return;
	}

	if (pkt->packet_type != R_DATA)
	{
	    printf("LIRC: Error reading config\n");
	    return;
	}
	if (strcasecmp(pkt->message, "LIST")!=0)
	{
	    printf("LIRC: Wrong packet recieved ... confused\n");
	    return;
	}

	test = pkt->data;
	count1 = 0;
	while (test != NULL)
	{
	    if (test->data)
		remotes = g_list_append(remotes, g_strdup(test->data));
	    test = test->next;
	    count1 ++;
	}

	free(pkt);
	done_with_packet();

	if (count1 > 0)
	{
	    for (i = 0; i<count1; i++)
	    {
		buf = g_list_nth(remotes, i)->data;
		if (buf)
		{
		    tree_item = gtk_tree_item_new_with_label(buf);
		    gtk_tree_append(GTK_TREE(remote_tree), tree_item);
		    gtk_widget_show (tree_item);
		}
	    }
	    for (i = 0; i<count1; i++)
	    {
		GList *codes, *temp;
		gchar *name;            
		GtkLabel *label;            
		GtkWidget *sub_tree;
		char msg[256];

		temp = GTK_TREE(remote_tree)->children;
		tree_item = g_list_nth(temp, i)->data;
		sub_tree = gtk_tree_new();

		/* Get a GtkWidget pointer from the list node */
		label = GTK_LABEL (GTK_BIN (tree_item)->child);
		gtk_label_get (label, &name);

		sprintf(msg, "LIST %s\n", name);
		send_command(msg);
		sprintf(msg, "LIST %s", name);

		pkt = get_next_packet();
		if (!pkt)
		{
		    printf("LIRC: Daemon timed out\n");
		    break;
		}

		if (pkt->status == R_ERROR)
		{
		    printf("LIRC: Daemon returned an error:\n");
		    test = pkt->data;
		    while (test != NULL)
		    {
			if (test->data)
			    printf("-> %s\n", (char*)test->data);
			test = test->next;
		    }
		    break;
		}

		if (pkt->packet_type != R_DATA)
		{
		    printf("LIRC: Error reading config\n");
		    break;
		}

		if (strcasecmp(pkt->message, msg)!=0)
		{
		printf("code = %s, msg = %s\n", msg, pkt->message);
		    printf("LIRC: Wrong packet recieved ... confused\n");
		    break;
		}

		test = pkt->data;
		count2 = 0;
		codes = 0;

		while (test != NULL)
		{
		    if (test->data)
			codes = g_list_append(codes, g_strdup(test->data));
		    test = test->next;
		    count2 ++;
		}

		free(pkt);
		done_with_packet();

		for (j=0; j<count2; j++)
		{
		    int k;
		    GtkWidget *subtree_item;
		    unsigned long long num;
		    char code[64];

		    buf = g_list_nth(codes, j)->data;
		    if (buf)
		    {
			k = sscanf (buf, "%Lx %s", &num, code);
			if (k!=2)
			{
			    printf("LIRC: Invalid remote data recieved\n");
			    continue;
			}

			subtree_item = gtk_tree_item_new_with_label(code);
			gtk_tree_append(GTK_TREE(sub_tree), subtree_item);
				gtk_widget_show (subtree_item);
		    }
		}

		test = codes;
		while (test != NULL)
		{
		    if (test->data)
			g_free(test->data);
		    test = test->next;
		}
		g_list_free(codes);

		gtk_tree_item_set_subtree(GTK_TREE_ITEM(tree_item), sub_tree);
	    }
	}

	test = remotes;
	while (test != NULL)
	{
	    if (test->data)
		g_free(test->data);
	    test = test->next;
	}  
	g_list_free(remotes);
    }
}

void clear_codes(GtkWidget *w,gpointer data)
{
    if(GTK_CLIST(codes_list)->selection)
    {
	gint row;
	gchar *text;
	gchar *new[3];
	
	row= (gint)GTK_CLIST(codes_list)->selection->data;
	
	gtk_clist_get_text(GTK_CLIST(codes_list), row, 0, &text);
	new[0] = g_strdup(text);
	new[1] = g_strdup("none");
	new[2] = g_strdup("none");

	gtk_clist_remove(GTK_CLIST(codes_list), row);
	gtk_clist_insert(GTK_CLIST(codes_list), row, new);
    }
}

struct lirc_code *find_code(char *function)
{
    GList *tl = lirc_codes;
    struct lirc_code *tc;

    if (function)
    while (tl != NULL)
    {
        tc = tl->data;
        if (tc)
        if (strcasecmp(tc->function, function)==0)
            return tc;
        tl = g_list_next(tl);
    }

    return NULL;
}

char *ins_under_score(char *text)
{
    char *new, *temp;

    new = g_strdup(text);
    temp = strchr(new, ' ');

    while (temp != NULL)
    {
	if (temp) temp[0] = '_';
	temp = strchr(new, ' ');
    }
    return new;
}

void save_settings(GtkWidget *w,gpointer data)
{
	gchar *filename, *tn, *tc, *tr;
	ConfigFile *cfgfile;
	int i;

	filename=g_strconcat(g_get_home_dir(),"/.gtunerrc",NULL);

	if (device)
	    g_free(device);

	device = g_strdup(gtk_entry_get_text(GTK_ENTRY(dev_entry)));

	for (i=0; i<num_codes; i++)
	{
                struct lirc_code *tcode;

		gtk_clist_get_text(GTK_CLIST(codes_list), i, 0, &tn);
		gtk_clist_get_text(GTK_CLIST(codes_list), i, 1, &tc);
		gtk_clist_get_text(GTK_CLIST(codes_list), i, 2, &tr);

		tn = ins_under_score(tn);
		tcode = find_code(tn);
		if (!tcode)
		{
		    printf("LIRC: Saving error: %s\n", tn);
		    continue;
		}

		g_free(tn);
		g_free(tcode->code);
		g_free(tcode->remote);
		tcode->code = g_strdup(tc);
		tcode->remote = g_strdup(tr);
	}

	if((cfgfile=gtuner_cfg_open_file(filename)))
	{
		gtuner_cfg_write_string(cfgfile,"LIRC","device",device);
		for (i=0; i<num_codes; i++)
		{
		    struct lirc_code *tc;
		    tc = g_list_nth(lirc_codes, i)->data;
		    if (!tc)
		    {
			printf("Lirc Infra Red Controller: Fatal error\n");
			exit(1);
		    }

		    gtuner_cfg_write_string(cfgfile,"LIRC_CODE",
						code_list[i],tc->code);
		    gtuner_cfg_write_string(cfgfile,"LIRC_REMOTE",
						code_list[i],tc->remote);
		}
		gtuner_cfg_write_file(cfgfile, filename);
		gtuner_cfg_free(cfgfile);
	}

	gtk_widget_destroy(configure_win);
	g_free(filename);
	printf("LIRC Resetting\n");
	cleanup();
	reset();
}

