/* Lirc Infra Red Controller Plugin
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	code from gtuner lirc plugin
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "lirc.h"
#include <fcntl.h>
#include <signal.h>

void init();

GenericPlugin lirc_plugin = {
    NULL,
    NULL,
    "LIRC Infra Red Controller Plugin "VERSION,
    init,
    about,
    configure,
    cleanup,
};

GenericPlugin *get_gplugin_info(void)
{
        return &lirc_plugin;
}


int lirc_fd = -1;
static struct sockaddr_un lirc_addr;
GList *lirc_codes = 0;
gchar *device = 0;

static char *buffer;
static char message[128] = "";
static gint ptr = 0, num_lines = 0, packet_ready = 0, packet_busy = 0;
static gint decoding = 0;
static enum reciver_state state = R_NONE;
static enum reciver_state packet = R_NONE;
static enum reciver_state status = R_NONE;
GList *msg_data = 0;

char *protocol_string[] =
{
        "BEGIN",
        "DATA",
        "END",
        "ERROR",
        "SUCCESS",
        "SIGHUP"
};

void init()
{
    struct lirc_packet *pkt;
    GList *test;

    load_config();

    printf("LIRC Infra Red Controller %s\n", VERSION);
    reset();
    if (lirc_fd != -1)
    {
	reset_packets();
	send_command("VERSION\n");

	pkt = get_next_packet();
	if (!pkt)
	{
	    printf("LIRC: Daemon timed out\n");
	    return;
	}

	if (pkt->status == R_ERROR)
	{
	    printf("LIRC: Daemon returned an error:\n");
            test = pkt->data;
            while (test != NULL)
	    {   
		if (test->data)
		    printf("-> %s\n", (char*)test->data);
		test = test->next;
	    }
	    return;
	}

	if (strcasecmp(pkt->message, "VERSION")!=0)
	{
	    printf("LIRC: Wrong packet recieved ... confused\n");
	    return;
	}

        test = pkt->data;
        if (test != NULL)
            if (test->data)
                printf("LIRC version: %s\n", (char*)test->data);

	free(pkt);
	done_with_packet();
    }
}

void lirc_input_callback();

void sigpoll (int sig)
{
    lirc_input_callback();
}

void lirc_file_callback( gpointer data, gint source,
                                GdkInputCondition condition )
{
    lirc_input_callback();
}

void reset()
{
    gint flags;
    struct sigaction new_sig;
    sigset_t set;
    capture = 0;

    strcpy(lirc_addr.sun_path, device);
    lirc_addr.sun_family=AF_UNIX;

    lirc_fd=socket(AF_UNIX,SOCK_STREAM,0);
    if(lirc_fd==-1)  {
	perror ("lirc_plugin");
	printf("lirc plugin: Cannot open %s\n", device);
	return;
    }
    if(connect(lirc_fd,(struct sockaddr *)&lirc_addr,sizeof(lirc_addr))==-1)  {
	perror("lirc_connect");
	printf("lirc plugin: Cannot connect to %s\n", device);
	return;
    }

    fcntl(lirc_fd,F_SETOWN,getpid());

    flags=fcntl(lirc_fd,F_GETFL,0);
    if(flags!=-1)
	fcntl(lirc_fd,F_SETFL,flags|FASYNC|O_NONBLOCK);

    sigemptyset(&set);
    new_sig.sa_handler = sigpoll;
    new_sig.sa_mask = set;
    new_sig.sa_flags = SA_RESTART; /* BSD Like signals */
    sigaction(SIGPOLL, &new_sig, NULL);
}

void cleanup()
{
    signal(SIGPOLL, SIG_DFL);
    if(lirc_fd!=-1)
	close(lirc_fd);
}

void lirc_input_callback()
{
    int rep, i=128, n=0, pos = 0;
    char buf[128];
    char *temp1, *temp2;
    unsigned long long code;
    char name[64], remote[64];

    while (decoding)
        usleep(100);
    usleep(100);

    decoding = 1;

    ptr = 0;
    temp1 = malloc(128);
    if (temp1==NULL)
    {
	printf("LIRC: Memory allocation error\n");
	return;
    }
    temp2 = 0;
    while (i == 128)
    {
	i = read (lirc_fd, temp1, 128);
	buffer = malloc(ptr+i);
	if (buffer==NULL)
	{
	    printf("LIRC: Memory allocation error\n");
	    free(temp1);
	    if (temp2) free(temp2);
	    return;
	}
	memmove (buffer+ptr, temp1, i);
	if (temp2)
	{
	    memmove(buffer, temp2, ptr);
	    free(temp2);
	}
	ptr += i;
	temp2 = buffer;
    }
    free(temp1);

    for (i=0; i < ptr; i++)
    {
	if (buffer[i] == '\n')
	    buffer[i] = 0;
    }

    i = strlen(buffer);

    while ((i+pos) < ptr)
    {
	strncpy(buf, buffer+pos, i+1);
	pos += i+1;
	i = strlen(buffer+pos);

	switch(state)
	{
	case R_NONE:
		if (strcasecmp(buf, protocol_string[R_BEGIN])==0)
		{
		    state = R_BEGIN;
		    packet = R_NONE;
		    while (packet_busy)
			usleep(100);
		    packet_ready = 0;
		}
		else if (strlen(buf) > 16)
		{
		    int k;
		    k = sscanf (buf, "%Lx %x %s %s", &code, &rep, name, remote);
		    if (k==4)
			lirc_handle_code(rep, name, remote);
		    else
			printf("LIRC: Invalid code recieved\n");
		}
		break;
	case R_BEGIN:
		if (strcasecmp(buf, protocol_string[R_SIGHUP])==0)
		{
		    printf("LIRC: Recived SIGHUP from daemon\n");
		    state = R_END;
		    break;
		}
		strncpy(message, buf, 128);
		state = R_STATUS;
		break;
	case R_STATUS:
		if (strcasecmp(buf, protocol_string[R_DATA])==0)
		{
		    state = R_N;
		    packet = R_DATA;
		    status = R_NONE;
		    break;
		}
		if (strcasecmp(buf, protocol_string[R_END])==0)
		{
		    state = R_NONE;
		    break;
		}
		if (strcasecmp(buf, protocol_string[R_SUCCESS])==0)
		{
		    break;
		    state = R_NEXT;
		    packet = R_STATUS;
		    status = R_SUCCESS;
		}
		if (strcasecmp(buf, protocol_string[R_ERROR])==0)
		{
		    break;
		    state = R_NEXT;
		    packet = R_STATUS;
		    status = R_SUCCESS;
		}
		printf("LIRC: Invalid packet from daemon\n");
		state = R_NONE;
		break;
	case R_NEXT:
		if (strcasecmp(buf, protocol_string[R_DATA])==0)
		{
		    state = R_N;
		    packet = R_DATA;
		    break;
		}
		if (strcasecmp(buf, protocol_string[R_END])==0)
		{
		    state = R_NONE;
		    break;
		}
		printf("LIRC: Invalid packet from daemon\n");
		state = R_NONE;
		break;
	case R_N:
		num_lines = atoi(buf);
		if (!num_lines)
		{
		    state = R_END;
		    break;
		}
		if (msg_data)
		{
		    GList *node;
		    node=msg_data;
		    while(node)
		    {
			g_free(node->data);
			node=node->next;
		    }
		    g_list_free(msg_data);
		    msg_data = NULL;
		}
		n = 0;
		state = R_DATA;
		break;
	case R_DATA:
	      {
		gchar *line;

		line = malloc(strlen(buf)+1);
		if (line==NULL)
		{
		    printf("LIRC: Memory allocation error\n");
		    state = R_NONE;
		    break;
		}
		strcpy(line, buf);
		n++;
		msg_data = g_list_append(msg_data, line);
		if (n == num_lines)
		    state = R_END;
		break;
	      }
	case R_END:
		state = R_NONE;
		if (strcasecmp(buf, protocol_string[R_END])!=0)
		{
		    printf("LIRC: Invalid packet from daemon\n");
		    break;
		}
		switch(packet)
		{
		case R_DATA:
		case R_STATUS:
		case R_ERROR:
			packet_ready = 1;
			break;
		default:
			printf("LIRC: Invalid packet type\n");
		}
		break;
	case R_ERROR:
		break;
	case R_SUCCESS:
		break;
	case R_SIGHUP:
		break;
	default:
		printf("LIRC: Undefined error\n");
	}
    }

    free(buffer);
    decoding = 0;
}

struct lirc_packet *get_next_packet()
{
    struct lirc_packet *pkt;
    int i;

    i = 0;
    while ((!packet_ready) && (i<100))
    {
	usleep(100);
	i++;
    }
    if (i==100)
	return NULL;

    packet_busy = 1;

    pkt = malloc (sizeof(struct lirc_packet));
    if (pkt==NULL)
    {
	printf("LIRC: Memory allocation error\n");
	return NULL;
    }
    pkt->packet_type = packet;
    pkt->status = status;
    pkt->message = message;
    pkt->data = msg_data;

    return pkt;
}

void done_with_packet()
{
    packet_busy = 0;
}

void reset_packets()
{
    packet_ready = 0;
    state = R_NONE;
}
