/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "kernel.h"
#include <unistd.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/videodev.h>
#include <fcntl.h>

#define FREQ_STEP 0.01
#define MAX_VOL 65536
int v4l_device = -1;
char* v4l_card_name = 0;
static double v4l_frequency;
static int min_freq, max_freq;
struct video_audio va;
static double freq_fact;
static int v4l_muted;
static int v4l_volume;

void configure()
{
}

void cleanup()
{
    close(v4l_device);
    if (v4l_card_name) free(v4l_card_name);
    v4l_device = -1;
}

CardPlugin kernel_plugin = {
    NULL,
    NULL,
    "Video 4 Linux card",
    kernel_init,
    about,
    configure,
    cleanup,
    kernel_on,
    kernel_off,
    kernel_mute,
    kernel_unmute,
    kernel_inc_volume,
    kernel_dec_volume,
    kernel_set_volume,
    kernel_get_volume,
    kernel_set_frequency,
    kernel_get_frequency,
    kernel_tune_up,
    kernel_tune_down,
    kernel_set_stereo,
    kernel_is_tuned,
    kernel_is_stereo,
    kernel_get_frequency_step,
    kernel_get_max_frequency,
    kernel_get_min_frequency
};

CardPlugin *get_cplugin_info(void)
{
        return &kernel_plugin;
}

radioError kernel_init(int port)
{
    struct video_capability vc;
    struct video_tuner vt;
    unsigned long f;

    v4l_device = open ("/dev/radio", O_RDONLY);
    if (v4l_device == -1)
	return DEVICE_NOT_PRESENT;

    vt.tuner = 0;
    if (ioctl(v4l_device, VIDIOCGCAP, &vc) == -1)
	return DEVICE_INITIALIZATION_FAIL;
    v4l_card_name = g_strdup(vc.name);
    if (ioctl(v4l_device, VIDIOCGTUNER, &vt) == -1)
        return DEVICE_INITIALIZATION_FAIL;

    min_freq  = (vt.rangelow/100.) /
			get_freq_fact(v4l_device);
    max_freq  = (vt.rangehigh/100.) /
			get_freq_fact(v4l_device);

    ioctl(v4l_device, VIDIOCGFREQ, &f);
    v4l_frequency = (f/100.) / get_freq_fact(v4l_device);

    v4l_muted = FALSE;
    v4l_volume = 8192;

    va.audio = 0;
    va.flags = 0;
    va.mode = 0;
    freq_fact = get_freq_fact(v4l_device);

    return NO_ERROR;
}

void kernel_unmute();

void kernel_set_volume(int volume)
{
    if (v4l_muted)
       kernel_unmute();

    if (volume >= 0)
    if (volume < GLOBAL_MAX_VOL)
    {
        va.flags = VIDEO_AUDIO_VOLUME;
        va.volume = volume * MAX_VOL / GLOBAL_MAX_VOL;
	va.mode = 0;
        ioctl(v4l_device, VIDIOCSAUDIO, &va);
	v4l_volume = volume;
    }
}

void kernel_on()
{
    kernel_set_volume (v4l_volume);
}

void kernel_off()
{
    va.flags = VIDEO_AUDIO_MUTE;
    va.mode = 0;
    ioctl(v4l_device, VIDIOCSAUDIO, &va);
}

void kernel_mute()
{
    v4l_muted = TRUE;
    va.flags = VIDEO_AUDIO_MUTE;
    va.mode = 0;
    ioctl(v4l_device, VIDIOCSAUDIO, &va);
}

void kernel_unmute()
{
    v4l_muted = FALSE;
    va.flags = 0;
    kernel_on();
}

void kernel_inc_volume()
{
    if (v4l_volume < GLOBAL_MAX_VOL-1)
    {
       v4l_volume++;
       kernel_set_volume(v4l_volume);
    }
}

void kernel_dec_volume()
{
    if (v4l_volume > 0)
    {
       v4l_volume--;
       kernel_set_volume(v4l_volume);
    }
}

int kernel_get_volume()
{
    return v4l_volume;
}

/* Determine and return the appropriate frequency multiplier for
   the first tuner on the open video device with handle FD. */
static double get_freq_fact (int fd)
{
       struct video_tuner tuner;
       tuner.tuner = 0;
       if (-1 == ioctl (fd, VIDIOCGTUNER, &tuner)
	   || (tuner.flags & VIDEO_TUNER_LOW) == 0)
		return .16;
       return 160.;
}

void kernel_set_frequency(double freq)
{
    unsigned long f = (unsigned long)((freq*100)*freq_fact);
    ioctl(v4l_device, VIDIOCSFREQ, &f);
    v4l_frequency = freq;
}

double kernel_get_frequency()
{
    return v4l_frequency;
}

void kernel_tune_up()
{
    v4l_frequency += FREQ_STEP;
    if (v4l_frequency > max_freq)
	v4l_frequency = min_freq;
    kernel_set_frequency(v4l_frequency);
}

void kernel_tune_down()
{
    v4l_frequency -= FREQ_STEP;
    if (v4l_frequency < min_freq)
        v4l_frequency = max_freq;
    kernel_set_frequency(v4l_frequency);
}

gboolean kernel_is_tuned()
{
    struct video_tuner vt;
    vt.tuner = 0;
    ioctl(v4l_device, VIDIOCGTUNER, &vt);
    return (vt.signal > 32678);
}

gboolean kernel_is_stereo()
{
    struct video_tuner vt;
    vt.tuner = 0;
    ioctl(v4l_device, VIDIOCGTUNER, &vt);
    return (vt.flags & VIDEO_TUNER_STEREO_ON);
}

void kernel_set_stereo(gboolean state)
{
    if (state)
	va.mode = VIDEO_SOUND_STEREO;
    else
	va.mode = VIDEO_SOUND_MONO;

    va.volume = v4l_volume*MAX_VOL/GLOBAL_MAX_VOL;
    ioctl(v4l_device, VIDIOCSAUDIO, &va);
}

double kernel_get_frequency_step ()
{
    return FREQ_STEP;
}

double kernel_get_max_frequency ()
{
    return max_freq;
}

double kernel_get_min_frequency ()
{
    return min_freq;
}
