/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "zoltrix.h"
#include <time.h>
#include <sys/io.h>
#include <sys/types.h>

void test()
{
    time_t t;
    time(&t);
}

void about()
{
}

void configure()
{
}

void cleanup()
{
    int zoltrix_port = -1;
    int zoltrix_stereo = 0;
    int zoltrix_volume = 0;
    double zoltrix_frequency = 0;
    gboolean zoltrix_muted = FALSE;
}

CardPlugin zoltrix_plugin = {
    NULL,
    NULL,
    "Zoltrix Radio Plus card",
    zol_init,
    about,
    configure,
    cleanup,
    zol_on,
    zol_off,
    zol_mute,
    zol_unmute,
    zol_inc_volume,
    zol_dec_volume,
    zol_set_volume,
    zol_get_volume,
    zol_set_frequency,
    zol_get_frequency,
    zol_tune_up,
    zol_tune_down,
    zol_set_stereo,
    zol_is_tuned,
    zol_is_stereo,
    zol_get_frequency_step,
    zol_get_max_frequency,
    zol_get_min_frequency
};

CardPlugin *get_cplugin_info(void)
{
        return &zoltrix_plugin;
}

#define MIN_FREQ 88.0
#define MAX_FREQ 108.0
#define FREQ_STEP 0.01
#define MAX_VOLU 16

static int zoltrix_port = -1;
static int zoltrix_stereo = 0;
static int zoltrix_volume = 0;
static double zoltrix_frequency = 0;
static gboolean zoltrix_muted = FALSE;

#define READ(x) inb(zoltrix_port+x);
#ifdef FREEBSD
#define WRITE(x) usleep(0); outb(zoltrix_port, x);
#else
#define WRITE(x) test(); outb(x, zoltrix_port);
#endif 

int get_io_privs(int port)
{
#if (defined FREEBSD)
    if ((fpiopl = open( "/dev/io", O_RDONLY ) < 0) )
    {
        return fpiopl;
    }
    return 0;
#else
   unsigned long basePort = port;
   /* get ioperm for RadioTrack range */
   int err;
   err=ioperm(basePort,4,0xFFFF);  /* i could not deduce from
                                    * man page what turn_on must
                                    * be */
   return err;
#endif
}

void give_up_root()
{
    /* get the real uid and give up root */
    uid_t uid;
    int err;

    uid=getuid();
    err=seteuid(uid);
}

radioError zol_init(int port)
{
    zoltrix_port = -1;
    if ((port != 0x20c) && (port != 0x30c))
	return DEVICE_NOT_PRESENT;
    if (get_io_privs(port) != 0)
	return PORT_ACCESS_DENIED;

    zoltrix_port = port;

   // give_up_root();

    zoltrix_stereo  = 1;
    zoltrix_volume  = 0;
    zoltrix_muted   = FALSE;
    zoltrix_frequency = MIN_FREQ;

    return NO_ERROR;
}

void zol_on()
{
    WRITE(zoltrix_volume);
    usleep(10000);
    READ(2);
}

void zol_off()
{
    WRITE(0);
    WRITE(0);
    READ(3);
}

void zol_mute()
{
    zoltrix_muted = TRUE;
    zol_off();
}

void zol_unmute()
{
    zoltrix_muted = FALSE;
    zol_on();
}

void zol_set_volume(int volume)
{
    volume = volume * MAX_VOLU / GLOBAL_MAX_VOL;

    if (zoltrix_muted)
       zol_mute();
    else
    if (volume < MAX_VOLU)
    {
	WRITE(volume);
	usleep(10000);
	READ(2);
	zoltrix_volume = volume;
    }
}

void zol_inc_volume()
{
    if (zoltrix_volume < MAX_VOLU-1)
    {
       zoltrix_volume++;
       zol_set_volume(zoltrix_volume);
    }
}

void zol_dec_volume()
{
    if (zoltrix_volume > 0)
    {
       zoltrix_volume--;
       zol_set_volume(zoltrix_volume);
    }
}

int zol_get_volume()
{
    return zoltrix_volume * GLOBAL_MAX_VOL / MAX_VOLU;
}

void zol_set_frequency(double freq)
{
    /* tunes the radio to the desired frequency */
    unsigned long long bitmask, f;
    unsigned int stereo = zoltrix_stereo;
    int i;

    if (freq > MAX_FREQ) freq = MAX_FREQ;
    else
	if (freq < MIN_FREQ) freq = MIN_FREQ;

    zoltrix_frequency = freq;

    f = (unsigned long long)(((float)(freq)-88.00)*200.0)+0x4d1c;
    bitmask = 0xc480402c10080000ull;
    i = 45;
    WRITE (0x00);
    WRITE (0x00);
    READ (3);

    WRITE (0x40);
    WRITE (0xc0);
    bitmask = (bitmask ^ ((f & 0xff) << 47) ^ ((f & 0xff00) << 30)
		^ (stereo << 31));
    while (i--)
      {
        if ((bitmask & 0x8000000000000000ull) != 0)
          {
            WRITE (0x80);
            WRITE (0x00);
            WRITE (0x80);
          }
        else
          {
            WRITE (0xc0);
            WRITE (0x40);
            WRITE (0xc0);
          }
        bitmask *= 2;
      }
    /* termination sequence */
    WRITE (0x80);
    WRITE (0xc0);
    WRITE (0x40);
    usleep (20000);
    if (!zoltrix_muted)
	{ WRITE (zoltrix_volume); }
    usleep (10000);
    READ (2);
    if (zoltrix_muted)
	zol_off();
}

double zol_get_frequency()
{
    return zoltrix_frequency;
}

void zol_tune_up()
{
    zoltrix_frequency += FREQ_STEP;
    if (zoltrix_frequency > MAX_FREQ)
	zoltrix_frequency = MIN_FREQ;
    zol_set_frequency(zoltrix_frequency);
}

void zol_tune_down()
{
    zoltrix_frequency -= FREQ_STEP;
    if (zoltrix_frequency < MIN_FREQ)
        zoltrix_frequency = MAX_FREQ;
    zol_set_frequency(zoltrix_frequency);
}

gboolean zol_is_tuned()
{
    int x1,x2;

    WRITE(0);
    WRITE(zoltrix_volume);
    usleep(10000);

    x1 = READ(0);
    usleep(1000);
    x2 = READ(0);

    if ((x1 == x2) && (x1 == 0xcf))
       return TRUE;
    if ((x1 == x2) && (x1 == 0xdf))
       return TRUE;
    if ((x1 == x2) && (x1 == 0xef))
       return TRUE;

    return FALSE;
}

gboolean zol_is_stereo()
{
    int x1,x2;

    WRITE(0);
    WRITE(zoltrix_volume);
    usleep(10000);

    x1 = READ(0);
    usleep(1000);
    x2 = READ(0);

    if ((x1 == x2) && (x1 == 0xcf))
       return TRUE;

    else return FALSE;
}

void zol_set_stereo(gboolean state)
{
    if (state) zoltrix_stereo = TRUE;
    else zoltrix_stereo = FALSE;

    zol_set_frequency(zoltrix_frequency);
}

double zol_get_frequency_step ()
{
    return FREQ_STEP;
}

double zol_get_max_frequency ()
{
    return MAX_FREQ;
}

double zol_get_min_frequency ()
{
    return MIN_FREQ;
}

