/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "alarm.h"
#include "conf.h"
#include "callbacks.h"
#include "gui/gui.h"
#include "gui/popup_menu.h"
#include <gtk/gtkmain.h>
#include <time.h>

gboolean alarm_interupt, alarm_ringing;
static int last_day;

static int timeout_alarm_timer(struct radio_config *conf)
{
    time_t t;
    int i;
    struct tm *tm;
    int today, hour, minute;

    time(&t);
    tm = localtime(&t);
    today = tm->tm_wday;
    hour = tm->tm_hour;
    minute = tm->tm_min;

    if (today != last_day)
    for (i=1; i<g_slist_length(display_list); i++)
    {
	GSList *entry = g_slist_nth(display_list, i);
	if (entry->data == NULL) continue;

	DISP_SET_ALARM(entry, TRUE, conf->alarm.hour[today],
						conf->alarm.minute[today]);
    }
    last_day = today;

    if (conf->alarm.day_bitmask & (1 << today))
    {
	if ((conf->alarm.hour[today] == hour) &&
	    (conf->alarm.minute[today] == minute))
	{
	    if (!alarm_ringing)
	    {
		struct channel *chan;
		alarm_ringing = 1;
		callback_unmute(NULL, NULL);

		if ((conf->alarm.station[today] >= 0) &&
			(conf->alarm.station[today] <= conf->num_channels))
		{
		    chan = get_channel(conf->first_channel,
					conf->alarm.station[today]);
		    callback_set_channel(NULL, chan);
		}
	    }
	}
	else
	    alarm_ringing = 0;
    }
    else
	alarm_ringing = 0;

    if (conf->alarm.active)
	return TRUE;

    alarm_interupt = FALSE;
    return FALSE;
}

void initialize_alarm(struct radio_config *conf)
{
    alarm_interupt = 0;
    alarm_ringing = 0;

    if (!conf->alarm_on)
	return;

    conf->alarm.active = TRUE;
    alarm_interupt = gtk_timeout_add(1000,
                    (GtkFunction)timeout_alarm_timer, conf);
}

void activate_alarm(struct radio_config *conf)
{
    int i;
    time_t t;
    struct tm *tm;
    int today;

    time(&t);
    tm = localtime(&t);
    today = tm->tm_wday;

    alarm_interupt = 0;
    alarm_ringing = 0;

    if (conf->alarm.active)
	return;

    conf->alarm.active = TRUE;
    alarm_interupt = gtk_timeout_add(1000,
				(GtkFunction)timeout_alarm_timer, conf);

#ifdef USE_GNOME
    gtk_check_menu_item_set_active(alarm_item, TRUE);
#else
    GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(
                        popup_menu,"/Alarm Clock"))->active = TRUE;
#endif
    for (i=1; i<g_slist_length(display_list); i++)
    {
	GSList *entry = g_slist_nth(display_list, i);
	if (entry->data == NULL) continue;

	DISP_SET_ALARM(entry, TRUE, conf->alarm.hour[today],
						conf->alarm.minute[today]);
    }
}

void deactivate_alarm(struct radio_config *conf)
{
    int i;

    conf->alarm.active = FALSE;
    if (alarm_interupt)
	gtk_timeout_remove(alarm_interupt);
    alarm_interupt = 0;

#ifdef USE_GNOME
    gtk_check_menu_item_set_active(alarm_item, FALSE);
#else
    GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(
			popup_menu,"/Alarm Clock"))->active = FALSE;
#endif
    for (i=1; i<g_slist_length(display_list); i++)
    {
        GSList *entry = g_slist_nth(display_list, i);
        if (entry->data == NULL) continue;

        DISP_SET_ALARM(entry, FALSE, conf->alarm.hour[0],
                                                conf->alarm.minute[0]);
    }
}
