/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "callbacks.h"
#include "hardware.h"
#include "gui/gui.h"
#include "gui/popup_menu.h"
#include "conf.h"
#include <malloc.h>

extern struct radio_config *config;
static int scanning = 0;
extern struct CardPluginData *card_data;

void callback_set_channel(GtkWidget *win, struct channel* chan)
{
    int i, test=0;

    card_set_channel(chan);
    for (i=1; i<g_slist_length(display_list); i++)
    {
	GSList *entry = g_slist_nth(display_list, i);
	if (entry->data == NULL) continue;

	DISP_SET_FREQ(entry, chan->frequency);
	DISP_SET_NAME(entry, chan->name);
	DISP_SET_CHAN(entry, chan->number);
    }
    test = card_is_tuned ();
    for (i=1; i<g_slist_length(display_list); i++)
    {
	GSList *entry = g_slist_nth(display_list, i);
	if (entry->data == NULL) continue;

	DISP_SET_TUNED(entry, test);
    }
    for (i=1; i<g_slist_length(freq_bar_list); i++)
    {
	GSList *entry = g_slist_nth(freq_bar_list, i);
	if (entry->data == NULL) continue;

	FREQ_BAR_SET(entry, chan->frequency);
    }
}

int callback_timer_check_radio(void)
{
    static int tuned;
    static int stereo;
 
    int i, test = 0;

    if (card_data->card_on)
    {
	test = card_is_tuned ();

	if (test != tuned)
	for (i=1; i<g_slist_length(display_list); i++)
	{
	    GSList *entry = g_slist_nth(display_list, i);
	    if (entry->data == NULL) continue;

	    DISP_SET_TUNED(entry, test);
	    tuned = test;
	}
	test = 0;
	test = card_is_stereo ();

	if (test != stereo)
	for (i=1; i<g_slist_length(display_list); i++)
	{
	    GSList *entry = g_slist_nth(display_list, i);
	    if (entry->data == NULL) continue;

	    DISP_SET_STEREO(entry, test);
	    stereo = test;
	}
    }
    return TRUE;
}

void callback_set_volume (GtkWidget *window, struct volume_changer *vc)
{
    int i;

    card_set_volume (vc->vol);

    for (i=1; i<g_slist_length(display_list); i++)
    {
	GSList *entry = g_slist_nth(display_list, i);
	if (entry->data == NULL) continue;

	DISP_SET_VOLUME(entry, vc->vol);
    }

    for (i=1; i<g_slist_length(volume_list); i++)
    {
	GSList *entry = g_slist_nth(volume_list, i);
	if (entry->data == NULL) continue;

        VOL_BAR_SET(entry, vc->vol);
    }
}

void callback_set_freq (GtkWidget *window, double freq)
{
    int i, test=0;
    char *name = NULL;
    int number = -1;
    struct channel *chan=NULL;
    double temp;

    for (i=0; i<config->num_channels; i++)
    {
	chan = get_channel(config->first_channel, i);
        if (!chan) continue;
	temp = freq - chan->frequency;

	if (temp < 0) temp = -temp;
	if (temp > (card_get_frequency_step ()/2)) continue;

	name = chan->name;
	number = chan->number;
	break;
    }
    if (!name) { name = malloc(1); strcpy(name, ""); }

    card_set_frequency  (freq);
    card_data->channel = number;

    for (i=1; i<g_slist_length(display_list); i++)
    {
	GSList *entry = g_slist_nth(display_list, i);
	if (entry->data == NULL) continue;

	DISP_SET_FREQ(entry, freq);
	DISP_SET_NAME(entry, name);
	DISP_SET_CHAN(entry, number);
    }
    test = card_is_tuned ();
    for (i=1; i<g_slist_length(display_list); i++)
    {
	GSList *entry = g_slist_nth(display_list, i);
	if (entry->data == NULL) continue;

	DISP_SET_TUNED(entry, test);
    }
    for (i=1; i<g_slist_length(freq_bar_list); i++)
    {
	GSList *entry = g_slist_nth(freq_bar_list, i);
	if (entry->data == NULL) continue;

        FREQ_BAR_SET(entry, freq);
    }
}

static int timeout_callback_scan(int step)
{
    double freq = card_get_frequency ();
    int test = 0;

    freq += (double)(step)/1000;
    if (freq < card_get_min_frequency ())
	freq = card_get_max_frequency ();
    if (freq > card_get_max_frequency ())
	freq = card_get_min_frequency ();

    callback_set_freq (NULL, freq);
    test = card_is_tuned ();

    if (test)
    {
	scanning = 0;
	return FALSE;
    }
    return TRUE;
}

void callback_scan(double step)
{
    if (scanning)
    {
	gtk_timeout_remove(scanning);
	scanning = 0;
    }
    else
	scanning = gtk_timeout_add(50,
			(GtkFunction)timeout_callback_scan, (char*)(int)(step*1000));
}

void callback_mute(GtkWidget *window, char *ignore)
{
    int i;

    card_mute ();
#ifdef USE_GNOME
    gtk_check_menu_item_set_active(mute_item, TRUE);
#else
    GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(
			popup_menu,"/Mute"))->active = TRUE;
#endif
    for (i=1; i<g_slist_length(display_list); i++)
    {
        DISP_SET_MUTE(g_slist_nth(display_list, i), card_data->muted);
    }

    for (i=1; i<g_slist_length(mute_list); i++)
    {
	GSList *entry = g_slist_nth(mute_list, i);
	if (entry->data == NULL) continue;

	MUTE_BUT_SET(entry, card_data->muted);
    }
}

void callback_unmute(GtkWidget *window, char *ignore)
{
    int i;

    card_unmute ();
#ifdef USE_GNOME
    gtk_check_menu_item_set_active(mute_item, FALSE);
#else
    GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(
			popup_menu,"/Mute"))->active = FALSE;
#endif
    for (i=1; i<g_slist_length(display_list); i++)
    {
        DISP_SET_MUTE(g_slist_nth(display_list, i), card_data->muted);
    }

    for (i=1; i<g_slist_length(mute_list); i++)
    {
	GSList *entry = g_slist_nth(mute_list, i);
	if (entry->data == NULL) continue;

	MUTE_BUT_SET(entry, card_data->muted);
    }
}


void callback_set_stereo(GtkWidget *window, gboolean state)
{
    int i;

    card_set_stereo (state);
#ifdef USE_GNOME
    gtk_check_menu_item_set_active(stereo_item, state);
#else
    GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(
			popup_menu,"/Stereo"))->active = state;
#endif
    for (i=1; i<g_slist_length(stereo_list); i++)
    {
	GSList *entry = g_slist_nth(stereo_list, i);
	if (entry->data == NULL) continue;

	STEREO_BUT_SET(entry, state);
    }
}

