/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* config.c */
#include "conf.h"
#include "hardware.h"
#include "configfile.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/stat.h>
#include <unistd.h>
#include <gdk_imlib.h>

/* Macro for parsing pixmaps */
#define DO_PIXMAP(x,y,z)	\
    text = str = 0;	\
    gtuner_cfg_read_string(cf, "pixmaps", z, &text); \
    if (text)	    \
    {	\
	str = find_in_path(text, conf->pix.path);   \
	if (!str) printf("gtuner: not found: %s\n", text);    \
    }	\
    if (str)	\
    {		\
	GdkImlibImage *im = NULL;	\
	im = gdk_imlib_load_image (str);\
	if (im)	    \
	{	    \
	    gdk_imlib_render(im,im->rgb_width,im->rgb_height); \
	    conf->pix.x = gdk_imlib_copy_image (im);	\
	    conf->pix.y = gdk_imlib_copy_mask (im);	\
	    gdk_imlib_destroy_image (im);	\
	}	    \
	free(str);  \
    }

void display_options(struct radio_config *);

char *find_in_path(char *name, GPtrArray *path)
{
    char *file_name = NULL;
    int i=0;
    struct stat fstat;

    if (path->pdata)
	while (path->pdata[i] != NULL)
	{
	    if (file_name) free(file_name);
	    file_name = g_strconcat((char*)path->pdata[i], "/", name, NULL);
	    if (!stat(file_name, &fstat))
		return file_name;
	    i++;
	}
    else
	{
	    if (file_name) free(file_name);
	    file_name = g_strconcat("./", name);
	    if (!stat(file_name, &fstat))
		return file_name;
	}

    return 0;
}

void process_file(char *file_name, struct radio_config *conf)
{
    ConfigFile *cf;
    char *text, *str;
    struct channel *ch;
    int i;

    cf = gtuner_cfg_open_file(file_name);
    if (!cf) return;
    if (conf->file_name) g_free(conf->file_name);
    conf->file_name = g_strdup(file_name);

    gtuner_cfg_read_string(cf, "gtuner", "card_plugin", &conf->card_name);
    gtuner_cfg_read_int(cf, "gtuner", "card_port", &conf->card_port);
    gtuner_cfg_read_string(cf, "gtuner", "layout", &conf->layout_file);
    gtuner_cfg_read_int(cf, "gtuner", "def_channel", &conf->def_channel);
    gtuner_cfg_read_int(cf, "gtuner", "def_volume", &conf->def_volume);
    gtuner_cfg_read_boolean(cf, "gtuner", "set_volume", &conf->set_volume);
    gtuner_cfg_read_boolean(cf, "gtuner", "set_channel", &conf->set_channel);
    gtuner_cfg_read_boolean(cf, "gtuner", "alarm_on", &conf->alarm_on);
    gtuner_cfg_read_boolean(cf, "gtuner", "timer_on", &conf->timer_on);
    gtuner_cfg_read_boolean(cf, "gtuner", "verbose_messages",
		    &conf->verbose_messages);
#ifdef USE_PANEL
    gtuner_cfg_read_boolean(cf, "gtuner", "start_hidden",
		    &conf->start_hidden);
#endif

    gtuner_cfg_read_boolean(cf, "gtuner", "save_on_exit", &conf->save_on_exit);
    gtuner_cfg_read_int(cf, "gtuner", "sleep_time", &conf->sleep.sleep_time);
    gtuner_cfg_read_int(cf, "gtuner", "fft_style", &conf->fft_style);
 
    text = 0;
    gtuner_cfg_read_string(cf, "gtuner", "quit_mute", &text);
    if (text)
    {
	if (strcasecmp(text, "no")==0) conf->quit_mute = 0;
	else if (strcasecmp(text, "yes")==0) conf->quit_mute = 1;
	else if (strcasecmp(text, "ask")==0) conf->quit_mute = 2;
	g_free(text);
    }
    gtuner_cfg_read_boolean(cf, "plugins", "load_plugins", &conf->load_plugins);
    gtuner_cfg_read_string(cf, "plugins", "load", &text);
    if (text)
    {
	gchar *list, *temp;
	list = text;
	do
	{
	    temp = strchr(list,':');
	    if (temp)
	    {
		*temp = '\0';
		temp++;
	    }
	    conf->plugin_load_list =
		    g_list_append (conf->plugin_load_list, g_strdup(list));
	    list = temp;
	} while (list);
	g_free(text);
    }

    gtuner_cfg_read_int(cf, "channels", "num", &conf->num_channels);
    ch = NULL;
    if (conf->first_channel)
    {
	while (conf->first_channel->next)
	    remove_channel(conf->first_channel->next);
	remove_channel(conf->first_channel);
    }
	    
    for (i=0; i<conf->num_channels; i++)
    {
	char *num = g_strdup_printf("%i", i);
	struct channel *new;

	gtuner_cfg_read_string(cf, "channels", num, &text);
	g_free(num);
	if (text)
	{
	    float freq;
	    char *name = malloc(256);
	    if (sscanf(text, "%f \"%[^\"]", &freq, name)==2)
		new = new_channel(freq, name, i);
	    else new = new_channel(0, "--", i);
	    g_free(name);
	    g_free(text);
	}
	else new = new_channel(0, "--", i);
	if (!ch) {ch = new; conf->first_channel=ch;}
	else
	{
	    insert_channel(ch, new);
	    ch = ch->next;
	}
    }
    text = 0;
    gtuner_cfg_read_string(cf, "pixmaps", "XpmPixmapPath", &text);
    if (text)
    {
/*
	char *token = strtok(text, ":");
	struct stat fstat;
	while (token)
	{
	    if (stat(text, &fstat))
		printf("gtuner: invalid directory: %s\n",token);
	    else
		g_ptr_array_add(conf->pix.path, g_strdup(token));
	    token = strtok(NULL, ":");
	}
	g_free(text);
 */
	gchar *list, *temp;
	struct stat fstat;

	list = text;
	do
	{
	    temp = strchr(list,':');
	    if (temp)
	    {
		*temp = '\0';
		temp++;
	    }
	    if (stat(list, &fstat))
		printf("gtuner: invalid directory: %s\n",list);
	    else
		g_ptr_array_add(conf->pix.path, g_strdup(list));
	    list = temp;
	} while (list);
	g_free(text);
    }
    DO_PIXMAP(icon_xpm, icon_mask, "Icon")
    DO_PIXMAP(up_arrow_xpm, up_arrow_mask, "UpArrow")
    DO_PIXMAP(down_arrow_xpm, down_arrow_mask, "DownArrow")
    DO_PIXMAP(left_arrow_xpm, left_arrow_mask, "LeftArrow")
    DO_PIXMAP(right_arrow_xpm, right_arrow_mask, "RightArrow")
    DO_PIXMAP(scan_left_xpm, scan_left_mask, "ScanLeftXpm")
    DO_PIXMAP(scan_right_xpm, scan_right_mask, "ScanRightXpm")
    DO_PIXMAP(plus_xpm, plus_mask, "PlusXpm")
    DO_PIXMAP(minus_xpm, minus_mask, "MinusXpm")
    DO_PIXMAP(power_xpm, power_mask, "PowerXpm")
    DO_PIXMAP(stereo_xpm, stereo_mask, "StereoXpm")

    for (i=0; i<7; i++)
    {
	char *day = malloc(4);
	char *temp;
	switch(i)
	{
        case 0: snprintf(day, 4, "sun"); break;
        case 1: snprintf(day, 4, "mon"); break;
        case 2: snprintf(day, 4, "tue"); break;
        case 3: snprintf(day, 4, "wed"); break;
        case 4: snprintf(day, 4, "thr"); break;
        case 5: snprintf(day, 4, "fri"); break;
        case 6: snprintf(day, 4, "sat"); break;
	}
	temp = g_strconcat(day, "_state", NULL);
	text = 0;
	gtuner_cfg_read_string(cf, "alarm", temp, &text);
	if (text)
	{
	    if (strcasecmp(text, "off")==0)
		conf->alarm.day_bitmask |= 1<<i;
	    else if (strcasecmp(text, "on")==0)
		conf->alarm.day_bitmask &= WEEK-(1<<i);
	    g_free(text);
	}
	g_free(temp);

	temp = g_strconcat(day, "_time", NULL);
	gtuner_cfg_read_string(cf, "alarm", temp, &text);
	if (text)
	{
	    int hr, min;
	    if (sscanf(text, "%i:%i", &hr, &min)==2)
	    {
		conf->alarm.hour[i] = hr;
		conf->alarm.minute[i] = min;
	    }
	    g_free(text);
	}
	g_free(temp);
	temp = g_strconcat(day, "_station", NULL);
	gtuner_cfg_read_int(cf, "alarm", temp, &conf->alarm.station[i]);
	g_free(temp);
	free(day);
    }

#ifdef USE_PANEL
    text = 0;
    gtuner_cfg_read_string(cf, "applet", "applet_style", &text);
    if (text)
    {
	if (strcasecmp(text, "none")==0)
		conf->panel_applet_style=PANEL_STYLE_NONE;
	else if (strcasecmp(text, "off")==0)
		conf->panel_applet_style=PANEL_STYLE_OFF;
	else if (strcasecmp(text, "text")==0)
		conf->panel_applet_style=PANEL_STYLE_TEXT;
	else if (strcasecmp(text, "chan_buts")==0)
		conf->panel_applet_style=PANEL_STYLE_CHAN_BUTS;
	else if (strcasecmp(text, "skined")==0)
		conf->panel_applet_style=PANEL_STYLE_THEMED;
	g_free(text);
    }
    gtuner_cfg_read_string(cf, "applet", "applet_skin_path",
		    &conf->applet_skin_path);
#endif

    gtuner_cfg_free(cf);
}

struct radio_config *read_config()
{
    struct radio_config *conf;
    char *conf_file;
    int i;

    conf = malloc(sizeof(struct radio_config));

    /* Initialize to default values */
    memset(conf, 0, sizeof(struct radio_config));
    conf->card_port = -1;
    conf->card_name = NULL;
    conf->def_channel = 0;
    conf->def_volume = 0;
    conf->set_volume = FALSE;
    conf->set_channel = FALSE;
    conf->timer_on = FALSE;
    conf->verbose_messages = TRUE;
#ifdef USE_PANEL
    conf->start_hidden = FALSE;
#endif
    conf->save_on_exit = TRUE;
    conf->num_channels = 0;
    conf->first_channel = NULL;
    conf->panel_applet_style = PANEL_STYLE_NONE;
    conf->applet_skin_path = g_strdup("");
    for (i=0; i<7; i++)
	conf->alarm.station[i] = -1;

    memset(&conf->pix, 0, sizeof(struct pixmap_config));

    conf->pix.path = g_ptr_array_new();
    conf->pix.saved_lines = g_ptr_array_new();
    conf->fft_style = 0;

    process_file("/etc/gtunerrc", conf);
    conf_file = g_strconcat(getenv("HOME"), "/.gtunerrc", NULL);

    process_file(conf_file, conf);
    free(conf_file);

    if (!conf->pix.path->pdata)
	g_ptr_array_add (conf->pix.path, ".");

    if (conf->verbose_messages)
    display_options(conf);
	
    return conf;
}

void save_config(struct radio_config *conf)
{
    ConfigFile *cf;
    GList *node;
    char *conf_file, *temp;
    int i;

    conf_file = g_strconcat(getenv("HOME"), "/.gtunerrc", NULL);

    cf = gtuner_cfg_open_file(conf_file);
    if (!cf) cf = gtuner_cfg_new();

    gtuner_cfg_write_string(cf, "gtuner", "card_plugin", conf->card_name);
    gtuner_cfg_write_int(cf, "gtuner", "card_port", conf->card_port);
    gtuner_cfg_write_string(cf, "gtuner", "layout", conf->layout_file);
    gtuner_cfg_write_int(cf, "gtuner", "def_channel", conf->def_channel);
    gtuner_cfg_write_int(cf, "gtuner", "def_volume", conf->def_volume);
    gtuner_cfg_write_boolean(cf, "gtuner", "set_volume", conf->set_volume);
    gtuner_cfg_write_boolean(cf, "gtuner", "set_channel", conf->set_channel);
    gtuner_cfg_write_boolean(cf, "gtuner", "alarm_on", conf->alarm_on);
    gtuner_cfg_write_boolean(cf, "gtuner", "timer_on", conf->timer_on);
    gtuner_cfg_write_boolean(cf, "gtuner", "verbose_messages",
		    conf->verbose_messages);
#ifdef USE_PANEL
    gtuner_cfg_write_boolean(cf, "gtuner", "start_hidden",
		    conf->start_hidden);
#endif

    gtuner_cfg_write_boolean(cf, "gtuner", "save_on_exit", conf->save_on_exit);
    gtuner_cfg_write_int(cf, "gtuner", "sleep_time", conf->sleep.sleep_time);
    gtuner_cfg_write_int(cf, "gtuner", "fft_style", conf->fft_style);
    switch (conf->quit_mute)
    {
	default:
	case 0:
	    gtuner_cfg_write_string(cf, "gtuner", "quit_mute", "no"); break;
	case 1:
	    gtuner_cfg_write_string(cf, "gtuner", "quit_mute", "yes"); break;
	case 2:
	    gtuner_cfg_write_string(cf, "gtuner", "quit_mute", "ask"); break;
    }
    gtuner_cfg_write_boolean(cf, "plugins", "load_plugins", conf->load_plugins);
    node = enabled_list;
    temp = 0;
    while (node)
    {
	char *text = g_strconcat(((GenericPlugin*)(char *)node->data)->filename,
			":", temp, NULL);
	g_free(temp);
	temp = g_strdup(text);
	g_free(text);
	node = node->next;
    }
    if (temp)
    {
	gtuner_cfg_write_string(cf, "plugins", "load", temp);
	g_free(temp);
    }
    gtuner_cfg_write_int(cf, "channels", "num", conf->num_channels);
 
    for (i=0; i<conf->num_channels; i++)
    {
	char *num, *text, *freq;
	num = g_strdup_printf("%i", i);
	freq = g_strdup_printf("%f",
			get_channel(conf->first_channel, i)->frequency);
	text = g_strconcat(freq, " \"",
			get_channel(conf->first_channel, i)->name, "\"", NULL);
	gtuner_cfg_write_string(cf, "channels", num, text);
	g_free(num);
	g_free(text);
	g_free(freq);
    }

    for (i=0; i<7; i++)
    {
	char *day = malloc(4);
	char *time;
	switch(i)
	{
        case 0: snprintf(day, 4, "sun"); break;
        case 1: snprintf(day, 4, "mon"); break;
        case 2: snprintf(day, 4, "tue"); break;
        case 3: snprintf(day, 4, "wed"); break;
        case 4: snprintf(day, 4, "thr"); break;
        case 5: snprintf(day, 4, "fri"); break;
        case 6: snprintf(day, 4, "sat"); break;
	}
	temp = g_strconcat(day, "_state", NULL);
	if (!(conf->alarm.day_bitmask & (1<<i)))
	    gtuner_cfg_write_string(cf, "alarm", temp, "off");
	else
	    gtuner_cfg_write_string(cf, "alarm", temp, "on");
	g_free(temp);

	temp = g_strconcat(day, "_time", NULL);
	time = g_strdup_printf("%i:%i", conf->alarm.hour[i],
			conf->alarm.minute[i]);
	gtuner_cfg_write_string(cf, "alarm", temp, time);
	g_free(temp);
	g_free(time);
	temp = g_strconcat(day, "_station", NULL);
	gtuner_cfg_write_int(cf, "alarm", temp, conf->alarm.station[i]);
	g_free(temp);
	free(day);
    }
#ifdef USE_PANEL
    if (conf->panel_applet_style==PANEL_STYLE_NONE) temp = g_strdup("none");
    else if (conf->panel_applet_style==PANEL_STYLE_OFF) temp = g_strdup("off");
    else if (conf->panel_applet_style==PANEL_STYLE_TEXT) temp =g_strdup("text");
    else if (conf->panel_applet_style==PANEL_STYLE_CHAN_BUTS)
	    temp = g_strdup("chan_buts");
    else if (conf->panel_applet_style==PANEL_STYLE_THEMED)
	    temp = g_strdup("skined");
    else temp = g_strdup("none");
    gtuner_cfg_write_string(cf, "applet", "applet_style", temp);
    free(temp);
    gtuner_cfg_write_string(cf, "applet", "applet_skin_path",
		    conf->applet_skin_path);
#endif

    gtuner_cfg_write_file(cf,conf_file);
    gtuner_cfg_free(cf);
    free(conf_file);
}

void display_options(struct radio_config *conf)
{
    int i;
    printf("Configuration file = %s\n", conf->file_name);
    printf("Layout file = %s\n", conf->layout_file);
    printf("\tRadio card port = 0x%x\n", conf->card_port);
    printf("\tRadio card plugin = %s\n", conf->card_name);
    printf("\tDefault channel = %d\n", conf->def_channel);
    printf("\tDefault volume = %d\n", conf->def_volume);
    printf("\tSet volume on startup = %d\n", conf->set_volume);
    printf("\tSet channel on startup = %d\n", conf->set_channel);
    printf("\tStartup alarm on = %d\n", conf->alarm_on);
    printf("\tStartup timer on = %d\n", conf->timer_on);
    printf("\tVerbose messages = %d\n", conf->verbose_messages);
    printf("\tMute on exit = ");
    if (conf->quit_mute == 0) printf("no\n");
    if (conf->quit_mute == 1) printf("yes\n");
    if (conf->quit_mute == 2) printf("ask\n");
    printf("\tSleep time = %d\n", conf->sleep.sleep_time);
    printf("\tNumer of channels = %d\n", conf->num_channels);
    printf("\tAlarm settings:\n");
#ifdef USE_PANEL
    printf("\tStart hidden = %d\n", conf->start_hidden);
    printf("\tApplet style = %d\n", conf->panel_applet_style);
    printf("\tApplet skin path = %s\n", conf->applet_skin_path);
#endif
    for (i=0; i<7; i++)
    {
	switch(i)
	{
	case 0: printf("\t\tSunday:\t\t"); break;
	case 1: printf("\t\tMonday:\t\t"); break;
	case 2: printf("\t\tTuesday:\t"); break;
	case 3: printf("\t\tWednesday:\t"); break;
	case 4: printf("\t\tThursday:\t"); break;
	case 5: printf("\t\tFriday:\t\t"); break;
	case 6: printf("\t\tSaturday:\t"); break;
	}
	if (conf->alarm.day_bitmask & (1 << i))
	{
	    if (conf->alarm.minute[i] < 10)
		printf("%i:0%i", conf->alarm.hour[i], conf->alarm.minute[i]);
	    else
		printf("%i:%i", conf->alarm.hour[i], conf->alarm.minute[i]);
	    if (conf->alarm.station[i] < 0) printf("\n");
	    else
	    if (conf->alarm.station[i] >= conf->num_channels) printf("\n");
	    else
		printf("\t%s\n", (get_channel(conf->first_channel,
					conf->alarm.station[i]))->name);
	}
	else
	    printf("off\n");
    }
    for (i=0;i<conf->num_channels;i++)
    	printf("\tChannel %i  = %3.3fMhz %s\n", i,
		(get_channel(conf->first_channel,i))->frequency,
		(get_channel(conf->first_channel,i))->name);
    printf("\tFFT Style = %d\n", conf->fft_style);
    printf("\tPixmap paths:\n");
    i=0;
    while (conf->pix.path->pdata[i] != NULL)
    {
	printf("\t\t%s\n", (char*)conf->pix.path->pdata[i]);
	i++;
    }
   printf("\tgtuner icon = %s\n",
	(conf->pix.icon_xpm)? "Found" : "Not found");
   printf("\tPixmap up_arrow = %s\n",
	(conf->pix.up_arrow_xpm)? "Found" : "Not found");
   printf("\tPixmap down_arrow = %s\n",
	(conf->pix.down_arrow_xpm) ? "Found" : "Not found");
   printf("\tPixmap left_arrow = %s\n",
	(conf->pix.left_arrow_xpm) ? "Found" : "Not found");
   printf("\tPixmap right_arrow = %s\n",
	(conf->pix.right_arrow_xpm) ? "Found" : "Not found");
   printf("\tPixmap scan_left = %s\n",
        (conf->pix.scan_left_xpm) ? "Found" : "Not found");
   printf("\tPixmap scan_right = %s\n",
        (conf->pix.scan_right_xpm) ? "Found" : "Not found");
   printf("\tPixmap mute = %s\n",
	(conf->pix.mute_xpm) ? "Found" : "Not found");
   printf("\tPixmap unmute = %s\n",
	(conf->pix.unmute_xpm) ? "Found" : "Not found");
   printf("\tPixmap on = %s\n",
	(conf->pix.on_xpm) ? "Found" : "Not found");
   printf("\tPixmap off = %s\n",
	(conf->pix.off_xpm) ? "Found" : "Not found");
   printf("\tPixmap scan_left = %s\n",
	(conf->pix.scan_left_xpm) ? "Found" : "Not found");
   printf("\tPixmap scan_right = %s\n",
	(conf->pix.scan_right_xpm) ? "Found" : "Not found");
   printf("\tPixmap tune_left = %s\n",
	(conf->pix.tune_left_xpm) ? "Found" : "Not found");
   printf("\tPixmap tune_right = %s\n",
	(conf->pix.tune_right_xpm) ? "Found" : "Not found");
   printf("\tPixmap plus = %s\n",
	(conf->pix.plus_xpm) ? "Found" : "Not found");
   printf("\tPixmap minus = %s\n",
	(conf->pix.minus_xpm) ? "Found" : "Not found");
   printf("\tPixmap power = %s\n",
	(conf->pix.power_xpm) ? "Found" : "Not found");
   printf("\tPixmap stereo = %s\n",
        (conf->pix.stereo_xpm) ? "Found" : "Not found");

}
