/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "conf.h"
#include "hardware.h"
#include "plugins/plugin_handler.h"
#include "gui/popup_menu.h"
#include "gui/prefs.h"
#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#ifdef USE_GNOME
#include <gnome.h>
#ifdef USE_PANEL
#include <applet-widget.h>
#include "gui/applet.h"
#endif
#endif

/* Global variables */
struct radio_config *config = NULL;

extern void setup_main_window();
extern void setup_defaults();
extern int callback_timer_check_radio(void);

void gtuner_init()
{
    GList *node;
    CardPlugin *cp;

    config = read_config();

    initialize_cards();
    initialize_plugins();

    node = get_card_list();
    while (node)
    {
	cp = (CardPlugin*)node->data;
	if (cp)
	{
	    if (config->card_name)
	    if (strcmp(config->card_name, cp->filename)==0)
		set_current_card(cp, config->card_port);
	}
	node = node->next;
    }

    initialize_alarm(config);
    initialize_timer(config);
    initialize_popups();
    setup_main_window();
    setup_defaults();
    create_preferences();

    gtk_timeout_add(1000, (GtkFunction)callback_timer_check_radio, NULL);
}

void gtuner_mute_quit()
{
    card_mute();
    gtk_main_quit();
}

void ask_mute()
{
    GtkWidget *dialog = gtk_dialog_new();
    GtkWidget *label = gtk_label_new("Mute Radio?");
    GtkWidget *yes, *no, *hbox;
    
    gtk_container_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),5);
    gtk_window_set_title(GTK_WINDOW(dialog), "Question");
    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), label);
    hbox = gtk_hbox_new(FALSE, 5);
    gtk_box_pack_end(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		    hbox, TRUE, TRUE, 0);
#ifdef USE_GNOME
    yes = gnome_stock_button(GNOME_STOCK_BUTTON_YES);
    no = gnome_stock_button(GNOME_STOCK_BUTTON_NO);
#else
    yes = gtk_button_new_with_label("Yes");
    no = gtk_button_new_with_label("No");
#endif
    gtk_box_pack_start(GTK_BOX(hbox), yes, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(hbox), no, TRUE, TRUE, 0);
    
    gtk_window_set_default_size(GTK_WINDOW(dialog), 300, 75);
//    gtk_window_set_position(dialog, GtkWindowPosition    position);
    gtk_window_activate_focus(GTK_WINDOW(dialog));
    gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);

    gtk_widget_show_all(dialog);

    gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
			   GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
    gtk_signal_connect(GTK_OBJECT(yes), "clicked",
			   GTK_SIGNAL_FUNC(gtuner_mute_quit), NULL);
    gtk_signal_connect(GTK_OBJECT(no), "clicked",
			   GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
    
    gtk_main();
}

void gtuner_cleanup()
{
    if (config)
    {
	if (config->quit_mute == 1)
	    card_mute();
	else if (config->quit_mute == 2)
	    ask_mute();

	if (config->save_on_exit)
	    save_config(config);

	if (config->first_channel)
        {
	    while (config->first_channel->next)
		remove_channel(config->first_channel->next);
            remove_channel(config->first_channel);
        }
	if (config->file_name) free (config->file_name);
	free(config);
    }
    card_cleanup();
#ifdef USE_PANEL
    applet_destroy();
#endif
}

int main(int argc, char **argv)
{          
#ifdef USE_PANEL
    gtuner_init_applet(argc, argv);
#else
#ifdef USE_GNOME
    gnome_init("gtuner", VERSION, argc, argv);
#else
    gtk_init(&argc, &argv);
#endif
#endif

    gdk_imlib_init();
    gtuner_init();

#ifdef USE_PANEL
    applet_widget_gtk_main();
#else
    gtk_main();
#endif

    gtuner_cleanup();

    printf("gtuner: normal shutdown\n");

    gtk_exit(0);
    return 0;
}
