/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef USE_PANEL

#include "popup_menu.h"
#include "../plugins/plugins.h"
#include "applet.h"
#include "prefs.h"
#include "layout.h"
#include <gtk/gtk.h>
#include <applet-widget.h>
#include "panel-private.h"
#include <malloc.h>
#include <math.h>
#include <gdk/gdkprivate.h>
#include <X11/Xlib.h>
#include <X11/extensions/shape.h>
#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <gdk/gdkx.h>

extern struct radio_config *config;

static GtkWidget *panel_applet, *panel_box=0, *applet_base=0;
struct display *applet_display;
static PanelStyle applet_style = 0;
static PanelOrientType orient;
static struct channel_object *chans = 0;
static GtkWidget *tw; /* Themed Widget */
static applet_skin *skin_v = 0;
static applet_skin *skin_h = 0;

struct text_style_struct {
    GtkWidget *channel;
    GtkWidget *freq;
    GtkWidget *status;
} *text_style = 0;

static void applet_set_frequency    (struct display *, double);
static void applet_set_name	    (struct display *, char*);
static void applet_set_chan	    (struct display *, int);
static void applet_set_volume	    (struct display *, int);
static void applet_set_mute	    (struct display *, gboolean);
static void applet_set_stereo	    (struct display *, gboolean);
static void applet_set_tuned	    (struct display *, gboolean);
static void applet_set_alarm	    (struct display *, gboolean, int, int);
static void applet_set_timer	    (struct display *, gboolean, int);
static void applet_update();
static void applet_change_orient( GtkWidget *w, PanelOrientType o, gpointer data );
static void applet_render_skin();
static void applet_load_skin(gchar *path);

static struct display_operations applet_ops = {
    applet_set_frequency,
    applet_set_name,
    applet_set_chan,
    applet_set_volume,
    applet_set_mute,
    applet_set_stereo,
    applet_set_tuned,
    applet_set_alarm,
    applet_set_timer
};

static void toggle_stereo()
{
    plugin_set_stereo(!plugin_is_stereo());
}
static void toggle_timer()
{
    plugin_set_timer(!plugin_get_timer_state());
}
static void toggle_alarm()
{
    plugin_set_alarm_state(!plugin_get_alarm_state());
}

static void stations_popup()
{
    gint mx, my;
    GdkModifierType modmask;
    struct Pos {gint x;gint y;} pos;

    gdk_window_get_pointer(NULL,&mx,&my, &modmask);

    pos.x = mx;
    pos.y = my;
    gtk_menu_popup(GTK_MENU(popup_stations_widget), NULL,
		    NULL, NULL, &pos, 1, GDK_CURRENT_TIME);

}

static
gint applet_clicked_callback(char *none, GdkEvent *data)
{
    if (data->type == GDK_2BUTTON_PRESS)
    {
	if (data->button.button == 1)
	{
	    layout_toggle_windows(!layout_windows_state());
	}
    }
    return FALSE; /* tell calling fn that call not processed */
}

void gtuner_init_applet(int ac, gchar *av[])
{
    applet_widget_init("gtuner", VERSION, ac, av, 0, 0, 0);

    panel_applet = applet_widget_new("gtuner_applet");

    /* in the rare case that the communication with the panel
	*            failed, error out */
    if (!panel_applet)
	g_error("Can't create applet!\n");

    gtk_widget_realize(panel_applet);

    applet_display = malloc(sizeof(struct display));
    memset (applet_display, 0, sizeof(struct display));
    applet_display->ops = &applet_ops;

    panel_box = gtk_event_box_new();
    gtk_signal_connect_object(GTK_OBJECT(panel_box), "event",
			    GTK_SIGNAL_FUNC (applet_clicked_callback), NULL);

    applet_widget_add(APPLET_WIDGET(panel_applet), panel_box);
    gtk_widget_show(panel_box);
    applet_set_style(PANEL_STYLE_NONE);

    applet_widget_set_tooltip (APPLET_WIDGET(panel_applet), "gtuner");
    applet_widget_register_stock_callback (APPLET_WIDGET(panel_applet),
    	   "about", GNOME_STOCK_MENU_ABOUT, _("About..."),
    	   GTK_SIGNAL_FUNC(popup_about), NULL);
    applet_widget_register_stock_callback (APPLET_WIDGET(panel_applet),
    	   "preferences", GNOME_STOCK_MENU_PREF, _("Preferences..."),
    	   GTK_SIGNAL_FUNC(show_preferences), NULL);
    applet_widget_register_stock_callback (APPLET_WIDGET(panel_applet),
    	   "Stations", GNOME_STOCK_MENU_JUMP_TO, "Stations...",
    	   GTK_SIGNAL_FUNC(stations_popup), NULL);
    applet_widget_register_callback (APPLET_WIDGET(panel_applet),
    	   "Mute", "Toggle Mute", GTK_SIGNAL_FUNC(plugin_toggle_mute), NULL);
    applet_widget_register_callback (APPLET_WIDGET(panel_applet),
    	   "Stereo", "Toggle Stereo", GTK_SIGNAL_FUNC(toggle_stereo), NULL);
    applet_widget_register_stock_callback (APPLET_WIDGET(panel_applet),
    	   "Timer", GNOME_STOCK_MENU_TIMER, "Toggle Timer",
	   GTK_SIGNAL_FUNC(toggle_timer), NULL);
    applet_widget_register_callback (APPLET_WIDGET(panel_applet),
    	   "Alarm", "Toggle Alarm", GTK_SIGNAL_FUNC(toggle_alarm), NULL);

    gtk_widget_show (panel_applet);

    gtk_signal_connect(GTK_OBJECT(panel_applet), "change_orient",
		    GTK_SIGNAL_FUNC( applet_change_orient ), NULL );
}

void applet_destroy()
{
    if (IS_APPLET_WIDGET(panel_applet))
	applet_widget_remove (APPLET_WIDGET(panel_applet));
}

void applet_set_style(PanelStyle style)
{
    if (panel_box)
    {
	if (chans)
	{
	    GSList *tmp = g_slist_find(channel_list, chans);
	    g_slist_remove_link(channel_list, tmp);
	    tmp->data = 0;
	    g_slist_free(tmp);
	    free(chans);
	    chans = 0;
	}
	if (applet_base)
	{
/*	    gtk_container_remove(GTK_CONTAINER(panel_box), applet_base);*/
	    gtk_widget_hide(applet_base);
	    gtk_widget_destroy(applet_base);
	}
	if (applet_style==PANEL_STYLE_THEMED)
	{
	    GdkWindowPrivate *sock_win = (GdkWindowPrivate *) GTK_PLUG(panel_applet)->socket_window;
	    Window root_return;
	    Window parent_return;
	    Window *children_return;
	    unsigned int nchildren_return;

	    /* find the evil parent window */
	    XQueryTree(GDK_DISPLAY(), GDK_WINDOW_XWINDOW(sock_win),
		    &root_return, &parent_return, &children_return,
		    &nchildren_return);
	    XFree(children_return);
	    /* Set the mask for our parent, too */
	    XShapeCombineMask(GDK_DISPLAY(), parent_return,
			    ShapeBounding, 0, 0, 0, ShapeSet);
	}
    }

    applet_style = style;
    switch(style)
    {
    case PANEL_STYLE_TEXT:
	{
	    if (text_style)
		free(text_style);
	    text_style = malloc (sizeof(struct text_style_struct));

	    applet_base = gtk_vbox_new(0,0);
	    gtk_container_set_border_width(GTK_CONTAINER(applet_base), 2);
	    gtk_container_add(GTK_CONTAINER(panel_box), applet_base);

	    text_style->channel = gtk_label_new("gtuner "VERSION);
	    text_style->freq = gtk_label_new("");
	    text_style->status = gtk_label_new("");
	    gtk_box_pack_start(GTK_BOX(applet_base), text_style->channel, FALSE, FALSE, 0);
	    gtk_box_pack_start(GTK_BOX(applet_base), text_style->freq, FALSE, FALSE, 0);
	    gtk_box_pack_start(GTK_BOX(applet_base), text_style->status, FALSE, FALSE, 0);
	}
	applet_change_orient(NULL, orient, NULL);
	applet_update();
	break;
    case PANEL_STYLE_NONE:
	{
	    GtkWidget *temp;
	    applet_base = gtk_frame_new(NULL);
	    gtk_frame_set_shadow_type(GTK_FRAME(applet_base),
			    GTK_SHADOW_ETCHED_IN );
	    gtk_container_add(GTK_CONTAINER(panel_box), applet_base);

	    temp = gtk_label_new("gtuner");
	    gtk_container_add(GTK_CONTAINER(applet_base), temp);
	}
	break;
    case PANEL_STYLE_OFF:
	{
	    applet_base = gtk_label_new("gtuner");
	    gtk_container_add(GTK_CONTAINER(panel_box), applet_base);
	    gtk_widget_hide(applet_base);
	}
	break;
    case PANEL_STYLE_CHAN_BUTS:
	{
	    chans = malloc(sizeof(struct channel_object));
	    chans->base = NULL;
	    chans->refresh = NULL;
	    switch (orient)
	    {
	    case ORIENT_LEFT:
	    case ORIENT_RIGHT:
		chans->data.v_box = malloc(sizeof(struct h_chan_box));
		chans->data.v_box->but_width = 50;
		chans->data.v_box->but_height = 20;
		chans->data.v_box->current_offset = 0;
		chans->data.v_box->show_numbers = TRUE;
		chans->data.v_box->show_text = TRUE;
		chans->data.v_box->cols = 1;
		chans->data.v_box->rows = 3;
		vchannel_box_new(chans);
		break;
	    default:
		chans->data.h_box = malloc(sizeof(struct h_chan_box));
		chans->data.h_box->but_width = 50;
		chans->data.h_box->but_height = 20;
		chans->data.h_box->current_offset = 0;
		chans->data.h_box->show_numbers = TRUE;
		chans->data.h_box->show_text = TRUE;
		chans->data.h_box->cols = 2;
		chans->data.h_box->rows = 2;
		hchannel_box_new(chans);
		break;
	    }
	    g_slist_append(channel_list, chans);
	    if (chans->refresh)
		(chans->refresh)(chans, config->first_channel);
	    gtk_widget_show(chans->base);
	    applet_base = chans->base;
	    gtk_container_add(GTK_CONTAINER(panel_box), applet_base);
	    break;
	}
    case PANEL_STYLE_THEMED:
	{
	    applet_base = gtk_event_box_new();
	    tw = gtk_drawing_area_new();
	    gtk_widget_show(tw);
	    gtk_container_add(GTK_CONTAINER(applet_base), tw);
	    gtk_container_add(GTK_CONTAINER(panel_box), applet_base);
	    gtk_widget_realize(tw);
	    applet_load_skin(config->applet_skin_path);
	    break;
	}
    default:
	applet_base = 0;
	break;
    }
    if (style != PANEL_STYLE_OFF)
	gtk_widget_show_all(applet_base);
}

static void applet_change_orient( GtkWidget *w, PanelOrientType o, gpointer data )
{
    orient = o;

    if( orient == ORIENT_LEFT || orient == ORIENT_RIGHT )
    {
	switch (applet_style)
	{
	case PANEL_STYLE_TEXT:
	    gtk_widget_set_usize(applet_base,48,-1);
	    break;
	case PANEL_STYLE_CHAN_BUTS:
	    applet_set_style(PANEL_STYLE_CHAN_BUTS);
	    gtk_widget_set_usize(applet_base,48,-1);
	    break;
	case PANEL_STYLE_THEMED:
	    if (!skin_v) break;
	    gtk_drawing_area_size(GTK_DRAWING_AREA(tw), skin_v->width, skin_v->height);
	    gtk_widget_set_usize(tw, skin_v->width, skin_v->height);
	    applet_render_skin();
	    break;
	default:
	    break;
	}
    }
    else
    {
	switch (applet_style)
	{
	case PANEL_STYLE_TEXT:
	    gtk_widget_set_usize(applet_base,-1,44);
	    break;
	case PANEL_STYLE_CHAN_BUTS:
	    applet_set_style(PANEL_STYLE_CHAN_BUTS);
	    gtk_widget_set_usize(applet_base,-1,44);
	    break;
	case PANEL_STYLE_THEMED:
	    if (!skin_h) break;
	    gtk_drawing_area_size(GTK_DRAWING_AREA(tw), skin_h->width, skin_h->height);
	    gtk_widget_set_usize(tw, skin_h->width, skin_h->height);
	    applet_render_skin();
	    break;
	default:
	    break;
	}
    }
}

static void applet_set_frequency    (struct display *applet, double freq)
{
    applet->frequency = freq;
    applet_update();
}

static void applet_set_name	    (struct display *applet, char *name)
{
    gchar *text;
    text = g_strdup_printf("gtuner: %s", name);
    applet_widget_set_tooltip (APPLET_WIDGET(panel_applet), text);
    g_free(text);

    if (applet->chan_name)
	    g_free(applet->chan_name);
    applet->chan_name = g_strdup(name);
    applet_update();
}

static void applet_set_chan	    (struct display *applet, int chan)
{
    applet->chan_number = chan;
    applet_update();
}

static void applet_set_volume	    (struct display *applet, int vol)
{
    applet->volume = vol;
    applet_update();
}

static void applet_set_mute	    (struct display *applet, gboolean state)
{
    applet->muted = state;
    applet_update();
}

static void applet_set_stereo	    (struct display *applet, gboolean state)
{
    applet->stereo = state;
    applet_update();
}

static void applet_set_tuned	    (struct display *applet, gboolean state)
{
    applet->tuned = state;
    applet_update();
}

static void applet_set_alarm	    (struct display *applet, gboolean state, int hour, int min)
{
    applet->alarm_on = state;
    applet->alarm_hours = hour;
    applet->alarm_min = min;
    applet_update();
}

static void applet_set_timer	    (struct display *applet, gboolean state, int time)
{
    applet->timer_on = state;
    applet->timer_time = time;
    applet_update();
}

static void applet_update()
{
    switch(applet_style)
    {
    case PANEL_STYLE_TEXT:
	{
	    char text[64];
	    if (applet_display->chan_name)
		gtk_label_set_text(GTK_LABEL(text_style->channel),
				applet_display->chan_name);
	    else
		gtk_label_set_text(GTK_LABEL(text_style->channel),"gtuner "VERSION);
	    if (!strcmp(applet_display->chan_name, ""))
		gtk_label_set_text(GTK_LABEL(text_style->channel),"gtuner "VERSION);
	    sprintf(text, "%3.2fMhz", applet_display->frequency);
	    gtk_label_set_text(GTK_LABEL(text_style->freq), text);
	    strcpy(text, "       ");
	    if (applet_display->muted)
		    strcpy(text, "  Mute");
	    else
	    {
		if (applet_display->tuned)
		    text[3] = 'T';
		if (applet_display->stereo)
		{
		    text[5] = ']';
		    text[6] = '[';
		}
	    }
	    if (applet_display->alarm_on)
		text[0] = 'A';
	    if (applet_display->timer_on)
		text[1] = 'Z';
	    gtk_label_set_text(GTK_LABEL(text_style->status),text);
	}
	break;
    case PANEL_STYLE_THEMED:
	applet_render_skin();
	break;
    case PANEL_STYLE_NONE:
	break;
    default:
    }
}

static void applet_skin_draw_channels(gboolean direction)
{
    int num = applet_display->chan_number;
    int i=0, temp, j,k=0;
    temp = num;
    while (temp >= 10)
    {
	temp /= 10;
	i++;
    }
    if (direction)
    {
	if (num >= 0)
	{
	    for (j=i; j>=0; j--)
	    {
		int digit = (int)(num/pow(10,j))%10;
		if (skin_v->font_num)
		    gdk_draw_pixmap(skin_v->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				skin_v->font_num, 0, digit*skin_h->font_numx/10,
				skin_h->channump.y, skin_h->channump.x+k*
				skin_h->font_numx/10, -1, skin_h->font_numx/10);
		k++;
	    }
	}
    } else
    {
	if (num >= 0)
	{
	    for (j=i; j>=0; j--)
	    {
		int digit = (int)(num/pow(10,j))%10;
		if (skin_h->font_num)
		    gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				skin_h->font_num, digit*skin_h->font_numx/10, 0,
				skin_h->channump.x+k*skin_h->font_numx/10,
				skin_h->channump.y, skin_h->font_numx/10, -1);
		k++;
	    }
	}
    }
}

static void get_char_xy(double w, double h, int *x, int *y, char c)
{
    if ((c >= 'A')&&(c <= 'Z'))
    {
	*x = (c-'A')*w;
	*y = 0;
    } else
    if ((c >= 'a')&&(c <= 'z'))
    {
	*x = (c-'a')*w;
	*y = 0;
    } else
    if ((c >= '0')&&(c <= '9'))
    {
	*x = (c-'0')*w;
	*y = h;
    } else
    {
	switch(c)
	{
	case '`': *x = 10*w; *y=h; break;
	case '~': *x = 11*w; *y=h; break;
	case '!': *x = 12*w; *y=h; break;
	case '@': *x = 13*w; *y=h; break;
	case '#': *x = 14*w; *y=h; break;
	case '$': *x = 15*w; *y=h; break;
	case '%': *x = 16*w; *y=h; break;
	case '^': *x = 17*w; *y=h; break;
	case '&': *x = 18*w; *y=h; break;
	case '*': *x = 19*w; *y=h; break;
	case '(': *x = 20*w; *y=h; break;
	case ')': *x = 21*w; *y=h; break;
	case '_': *x = 22*w; *y=h; break;
	case '+': *x = 23*w; *y=h; break;
	case '-': *x = 24*w; *y=h; break;
	case '=': *x = 25*w; *y=h; break;
	case '[': *x = 0*w; *y=2*h; break;
	case ']': *x = 1*w; *y=2*h; break;
	case '{': *x = 2*w; *y=2*h; break;
	case '}': *x = 3*w; *y=2*h; break;
	case ':': *x = 4*w; *y=2*h; break;
	case ';': *x = 5*w; *y=2*h; break;
	case '\"': *x = 6*w; *y=2*h; break;
	case '\'': *x = 7*w; *y=2*h; break;
	case '|': *x =  8*w; *y=2*h; break;
	case '\\': *x = 9*w; *y=2*h; break;
	case '<': *x = 10*w; *y=2*h; break;
	case '>': *x = 11*w; *y=2*h; break;
	case ',': *x = 12*w; *y=2*h; break;
	case '.': *x = 13*w; *y=2*h; break;
	case '?': *x = 14*w; *y=2*h; break;
	case '/': *x = 15*w; *y=2*h; break;
	case ' ': *x = 16*w; *y=2*h; break;
	default:
		*x = 14*w;
		*y = 2*h;
		break;
	}
    }
}

static void applet_skin_draw_station()
{
    if (skin_h->font_disp)
    {
	if (applet_display->chan_name)
	{
	    int i, x, y;
	    double width, height;
	    width = skin_h->fontx/26.0;
	    height = skin_h->fonty/3;
	    for (i=0; i<strlen(applet_display->chan_name); i++)
	    {
		GdkRectangle r;
		r.x = skin_h->disptlp.x;
		r.y = skin_h->disptlp.y;
		r.width = skin_h->dispbrp.x - skin_h->disptlp.x;
		r.height = skin_h->dispbrp.y - skin_h->disptlp.y;
		gdk_gc_set_clip_rectangle(tw->style->fg_gc[GTK_WIDGET_STATE(tw)],&r);
		get_char_xy(width, height, &x, &y, applet_display->chan_name[i]);
		gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				    skin_h->font_disp, x, y,
				    skin_h->disptlp.x+i*width, skin_h->disptlp.y, width, height);
		r.x = 0;
		r.y = 0;
		r.width = -1;
		r.height = -1;
		gdk_gc_set_clip_rectangle(tw->style->fg_gc[GTK_WIDGET_STATE(tw)],&r);
	    }
	} else
	{
	    if (skin_h->mhzp.x != -1)
		if (skin_h->mhz)
		    gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				    skin_h->mhz, 0, 0,
				    skin_h->mhzp.x, skin_h->mhzp.y, -1, -1);
	}
    }
}

static void applet_skin_set_mask(GdkBitmap *mask)
{
    GdkWindowPrivate *sock_win = (GdkWindowPrivate *) GTK_PLUG(panel_applet)->socket_window;
    Window root_return;
    Window parent_return;
    Window *children_return;
    unsigned int nchildren_return;

    /* find the evil parent window */
    XQueryTree(GDK_DISPLAY(), GDK_WINDOW_XWINDOW(sock_win),
	&root_return, &parent_return, &children_return, &nchildren_return);
    XFree(children_return);

    /* Set the mask for our parent, too */
    XShapeCombineMask(GDK_DISPLAY(), parent_return,
	ShapeBounding, 0, 0, ((GdkWindowPrivate *)mask)->xwindow,
	ShapeSet);

    gdk_window_shape_combine_mask(tw->window,mask, 0, 0);
}

static void applet_render_skin()
{
	if (skin_h)
	{
	    if (!skin_h->display) return;
	    if (skin_h->background)
		gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				skin_h->background, 0, 0, 0, 0, skin_h->width, skin_h->height);
	    if (skin_h->alarmp.x != -1)
		if (skin_h->alarm)
		{
		    if (applet_display->alarm_on)
			gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				    skin_h->alarm, skin_h->alarmx/2, 0,
				    skin_h->alarmp.x, skin_h->alarmp.y, -1, -1);
		    else
			gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				    skin_h->alarm, 0, 0,
				    skin_h->alarmp.x, skin_h->alarmp.y, skin_h->alarmx/2, -1);
		}
	    if (skin_h->timerp.x != -1)
		if (skin_h->timer)
		{
		    if (applet_display->timer_on)
			gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				    skin_h->timer, skin_h->timerx/2, 0,
				    skin_h->timerp.x, skin_h->timerp.y, -1, -1);
		    else
			gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				    skin_h->timer, 0, 0,
				    skin_h->timerp.x, skin_h->timerp.y, skin_h->timerx/2, -1);
		}
	    if (skin_h->stereop.x != -1)
		if (skin_h->stereo)
		{
		    if (applet_display->stereo)
			gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				    skin_h->stereo, skin_h->stereox/2, 0,
				    skin_h->stereop.x, skin_h->stereop.y, -1, -1);
		    else
			gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				    skin_h->stereo, 0, 0,
				    skin_h->stereop.x, skin_h->stereop.y, skin_h->stereox/2, -1);
		}
	    if (skin_h->tunedp.x != -1)
		if (skin_h->tuned)
		{
		    if (applet_display->tuned)
			gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				    skin_h->tuned, skin_h->tunedx/2, 0,
				    skin_h->tunedp.x, skin_h->tunedp.y, -1, -1);
		    else
			gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				    skin_h->tuned, 0, 0,
				    skin_h->tunedp.x, skin_h->tunedp.y, skin_h->tunedx/2, -1);
		}
	    if (skin_h->mutep.x != -1)
		if (skin_h->mute)
		{
		    if (applet_display->muted)
			gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				    skin_h->mute, skin_h->mutex/2, 0,
				    skin_h->mutep.x, skin_h->mutep.y, -1, -1);
		    else
			gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				    skin_h->mute, 0, 0,
				    skin_h->mutep.x, skin_h->mutep.y, skin_h->mutex/2, -1);
		}
	    if (skin_h->volp.x != -1)
		if (skin_h->vol)
		{
		    gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				    skin_h->vol, 0, 0,
				    skin_h->volp.x, skin_h->volp.y, skin_h->volx/2, -1);
		    gdk_draw_pixmap(skin_h->display, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				    skin_h->vol, skin_h->volx/2, 0,
				    skin_h->volp.x, skin_h->volp.y, skin_h->volx/2*applet_display->volume/GLOBAL_MAX_VOL, -1);
		}
	    applet_skin_draw_channels(0);
	    applet_skin_draw_station();

	}
    if (orient == ORIENT_LEFT || orient == ORIENT_RIGHT)
    {
	GdkImlibImage *disp_copy;
	GdkPixmap *im;

	if (skin_v->mask)
	    applet_skin_set_mask(skin_v->mask);

	disp_copy = gdk_imlib_create_image_from_drawable(skin_h->display,
			skin_h->mask, 0, 0, skin_h->width,
			skin_h->height);
	gdk_imlib_rotate_image(disp_copy, 90);
	gdk_imlib_flip_image_vertical(disp_copy);
	gdk_imlib_render(disp_copy,disp_copy->rgb_width,disp_copy->rgb_height);
	im = gdk_imlib_copy_image (disp_copy);
	gdk_imlib_destroy_image (disp_copy);

        gdk_window_set_back_pixmap(tw->window,im,FALSE);
	gdk_window_clear(tw->window);
	gdk_draw_pixmap(tw->window, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				im, 0, 0, 0, 0,
				skin_h->height, skin_h->width);
	gdk_pixmap_unref(im);
    }
    else
    {
	if (skin_h->mask)
	    applet_skin_set_mask(skin_h->mask);
        gdk_window_set_back_pixmap(tw->window,skin_h->display,FALSE);
	gdk_window_clear(tw->window);
	gdk_draw_pixmap(tw->window, tw->style->fg_gc[GTK_WIDGET_STATE(tw)],
				skin_h->display, 0, 0, 0, 0,
				skin_h->width, skin_h->height);

    }
}

static GdkPixmap *load_pixmap(char *path, char *file, gint *width)
{
    GdkImlibImage *im = NULL;
    gchar *filename;
    GdkPixmap *pix;

    if (!file) return NULL;
    if (strcmp(file, "")==0) return NULL;

    filename = g_strconcat(path, "/", file, NULL);
    im = gdk_imlib_load_image (filename);
    g_free(filename);
    if (!im) return NULL;
    gdk_imlib_render(im,im->rgb_width,im->rgb_height);
    *width=im->rgb_width;
    pix = gdk_imlib_copy_image (im);
    gdk_imlib_destroy_image (im);
    return pix;
}

void load_mask(char *path, char *file, GdkBitmap **horz, GdkBitmap **vert)
{
    GdkImlibImage *im = NULL;
    gchar *filename;

    if (!file) return;
    if (strcmp(file, "")==0) return;

    filename = g_strconcat(path, "/", file, NULL);
    im = gdk_imlib_load_image (filename);
    g_free(filename);
    if (!im) return;
    gdk_imlib_render(im,im->rgb_width,im->rgb_height);
    *horz = gdk_imlib_copy_mask (im);
    gdk_imlib_rotate_image(im, 90);
    gdk_imlib_flip_image_vertical(im);
    gdk_imlib_render(im,im->rgb_width,im->rgb_height);
    *vert = gdk_imlib_copy_mask (im);
    gdk_imlib_destroy_image (im);
}

static GdkPixmap *load_pixmap_xy(char *path, char *file, gint *x, gint *y)
{
    GdkImlibImage *im = NULL;
    gchar *filename;
    GdkPixmap *pix;

    if (!file) return NULL;
    if (strcmp(file, "")==0) return NULL;

    filename = g_strconcat(path, "/", file, NULL);
    im = gdk_imlib_load_image (filename);
    g_free(filename);
    if (!im) return NULL;
    gdk_imlib_render(im,im->rgb_width,im->rgb_height);
    *x=im->rgb_width;
    *y=im->rgb_height;
    pix = gdk_imlib_copy_image (im);
    gdk_imlib_destroy_image (im);
    return pix;
}

static void get_position(char *buf, posn *pos)
{
    if (!buf)
    {
	pos->x = pos->y = -1;
	return;
    }
    if (sscanf(buf, "%i %i", &pos->x, &pos->y)==2)
	return;
    pos->x = pos->y = -1;
}

void free_skin(applet_skin *skin)
{
    if (skin->display) gdk_pixmap_unref(skin->display);
    if (skin->background) gdk_pixmap_unref(skin->background);
    if (skin->font_disp) gdk_pixmap_unref(skin->font_disp);
    if (skin->font_num) gdk_pixmap_unref(skin->font_num);
    if (skin->alarm) gdk_pixmap_unref(skin->alarm);
    if (skin->timer) gdk_pixmap_unref(skin->timer);
    if (skin->tuned) gdk_pixmap_unref(skin->tuned);
    if (skin->stereo) gdk_pixmap_unref(skin->stereo);
    if (skin->mute) gdk_pixmap_unref(skin->mute);
    if (skin->mhz) gdk_pixmap_unref(skin->mhz);
    if (skin->vol) gdk_pixmap_unref(skin->vol);
    if (skin->mask) gdk_bitmap_unref(skin->mask);
    free(skin);
}

static void applet_load_skin(gchar *path)
{
    gchar *prefix, *datafile;
    gchar *buf = NULL;
    gint temp;

    datafile = g_strconcat(path, "/theme.data", NULL);

    if (!g_file_exists(datafile))
    {
	printf("Unable to open skin data file: %s\n",datafile);
	g_free(datafile);
	return;
    }

    if (skin_h) free_skin(skin_h);
    if (skin_v) free_skin(skin_v);
    skin_v = malloc (sizeof (applet_skin));
    skin_h = malloc (sizeof (applet_skin));
    memset (skin_h, 0, sizeof (applet_skin));
    memset (skin_v, 0, sizeof (applet_skin));

    prefix = g_strconcat ("=", datafile, "=/main/", NULL);
    gnome_config_push_prefix (prefix);
    g_free (prefix);

    skin_h->width = gnome_config_get_int("skin_width=");
    skin_h->height = gnome_config_get_int("skin_height=");
    skin_v->width = skin_h->height;
    skin_v->height = skin_h->width;

    gnome_config_pop_prefix();
    prefix = g_strconcat ("=", datafile, "=/pixmaps/", NULL);
    gnome_config_push_prefix (prefix);
    g_free (prefix);

    skin_h->display = gdk_pixmap_new(NULL, skin_h->width, skin_h->height,
			gdk_visual_get_best_depth ());
    skin_v->display = gdk_pixmap_new(NULL, skin_v->width, skin_v->height,
			gdk_visual_get_best_depth ());
    buf = gnome_config_get_string("background=");
    skin_h->background = load_pixmap(path, buf, &temp);
    g_free(buf);
    buf = gnome_config_get_string("font_disp=");
    skin_h->font_disp = load_pixmap_xy(path, buf, &skin_h->fontx, &skin_h->fonty);
    g_free(buf);
    buf = gnome_config_get_string("font_num=");
    skin_h->font_num = load_pixmap(path, buf, &skin_h->font_numx);
    g_free(buf);
    buf = gnome_config_get_string("alarm=");
    skin_h->alarm = load_pixmap(path, buf, &skin_h->alarmx);
    g_free(buf);
    buf = gnome_config_get_string("timer=");
    skin_h->timer = load_pixmap(path, buf, &skin_h->timerx);
    g_free(buf);
    buf = gnome_config_get_string("tuned=");
    skin_h->tuned = load_pixmap(path, buf, &skin_h->tunedx);
    g_free(buf);
    buf = gnome_config_get_string("stereo=");
    skin_h->stereo = load_pixmap(path, buf, &skin_h->stereox);
    g_free(buf);
    buf = gnome_config_get_string("mute=");
    skin_h->mute = load_pixmap(path, buf, &skin_h->mutex);
    g_free(buf);
    buf = gnome_config_get_string("mhz=");
    skin_h->mhz = load_pixmap(path, buf, &temp);
    g_free(buf);
    buf = gnome_config_get_string("vol=");
    skin_h->vol = load_pixmap(path, buf, &skin_h->volx);
    g_free(buf);
    buf = gnome_config_get_string("mask=");
    load_mask(path, buf, &skin_h->mask, &skin_v->mask);
    g_free(buf);

    gnome_config_pop_prefix();
    prefix = g_strconcat ("=", datafile, "=/alignment/", NULL);
    gnome_config_push_prefix (prefix);
    g_free (prefix);

    buf = gnome_config_get_string("disp_top_left=");
    get_position(buf, &skin_h->disptlp);
    g_free(buf);
    buf = gnome_config_get_string("disp_bottom_right=");
    get_position(buf, &skin_h->dispbrp);
    g_free(buf);
    buf = gnome_config_get_string("channum=");
    get_position(buf, &skin_h->channump);
    g_free(buf);
    buf = gnome_config_get_string("alarm=");
    get_position(buf, &skin_h->alarmp);
    g_free(buf);
    buf = gnome_config_get_string("timer=");
    get_position(buf, &skin_h->timerp);
    g_free(buf);
    buf = gnome_config_get_string("tuned=");
    get_position(buf, &skin_h->tunedp);
    g_free(buf);
    buf = gnome_config_get_string("stereo=");
    get_position(buf, &skin_h->stereop);
    g_free(buf);
    buf = gnome_config_get_string("mute=");
    get_position(buf, &skin_h->mutep);
    g_free(buf);
    buf = gnome_config_get_string("mhz=");
    get_position(buf, &skin_h->mhzp);
    g_free(buf);
    buf = gnome_config_get_string("vol=");
    get_position(buf, &skin_h->volp);
    g_free(buf);

    g_free(datafile);

    applet_change_orient(NULL, orient, NULL);
}

#endif

