/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "gui.h"
#include "../callbacks.h"
#include <stdio.h>

extern struct radio_config *config;

static void hchannel_box_refresh(struct channel_object *, struct channel *);
static void vchannel_box_refresh(struct channel_object *, struct channel *);

static
void hchannel_box_scroll(GtkWidget *but, struct channel_object *obj)
{
    struct h_chan_box *box = obj->data.h_box;

    if (but == box->left_arrow)
    {
	if (box->current_offset >= box->rows)
	    box->current_offset -= box->rows;
	if (!box->current_offset)
	    gtk_widget_set_sensitive(box->left_arrow, FALSE);
    }
    else if (but == box->right_arrow)
    {
	gtk_widget_set_sensitive(box->left_arrow, TRUE);
	box->current_offset += box->rows;
    }
    if ((box->current_offset + box->cols*box->rows) >= config->num_channels)
	gtk_widget_set_sensitive(box->right_arrow, FALSE);
    else
	gtk_widget_set_sensitive(box->right_arrow, TRUE);

    hchannel_box_refresh(obj, config->first_channel);
}

void hchannel_box_new(struct channel_object *obj)
{
    struct channel_button *temp;
    struct h_chan_box *hcb = obj->data.h_box;
    int i, j;

    obj->base = gtk_hbox_new (FALSE, 5);
//    gtk_container_border_width(GTK_CONTAINER(obj->base), 2);
    obj->refresh = hchannel_box_refresh;

    if (!config->pix.left_arrow_xpm)
        hcb->left_arrow = gtk_button_new_with_label("<");
    else
        {
            hcb->left_arrow = gtk_button_new();
            hcb->left_arrow_pix = gtk_pixmap_new(
                                        (config->pix.left_arrow_xpm),
                                        config->pix.left_arrow_mask);
            gtk_widget_show(hcb->left_arrow_pix);
            gtk_container_add(GTK_CONTAINER(hcb->left_arrow),
				hcb->left_arrow_pix);
        }
    gtk_widget_show(hcb->left_arrow);
    gtk_widget_set_sensitive(hcb->left_arrow, FALSE);
    gtk_box_pack_start(GTK_BOX(obj->base), hcb->left_arrow, 0,0,0);
    gtk_signal_connect(GTK_OBJECT(hcb->left_arrow), "clicked",
			GTK_SIGNAL_FUNC(hchannel_box_scroll), obj);

    hcb->button_array = g_ptr_array_new();

    for (i=0; i<hcb->cols; i++)
	for (j=0; j<hcb->rows; j++)
	{
	    temp = new_channel_button(hcb->but_width, hcb->but_height,
		    hcb->show_numbers,i*hcb->rows+j , hcb->show_text, "---");
	    gtk_widget_set_sensitive(temp->base, FALSE);
	    g_ptr_array_add(hcb->button_array, temp);
	}

    hcb->button_table = gtk_table_new(hcb->rows, hcb->cols, TRUE);
    for (i=0; i< hcb->cols; i++)
	for (j=0; j<hcb->rows; j++)
	{
	    gtk_table_attach_defaults(GTK_TABLE(hcb->button_table),
		((struct channel_button *)
		(g_ptr_array_index(hcb->button_array, i*hcb->rows+j)))->base,
		i, i+1, j, j+1);
	}
    gtk_widget_show(hcb->button_table);
    gtk_container_add(GTK_CONTAINER(obj->base), hcb->button_table);

    if (!config->pix.right_arrow_xpm)
        hcb->right_arrow = gtk_button_new_with_label(">");
    else
        {
            hcb->right_arrow = gtk_button_new();
            hcb->right_arrow_pix = gtk_pixmap_new(
                                        (config->pix.right_arrow_xpm),
                                        config->pix.right_arrow_mask);
            gtk_widget_show(hcb->right_arrow_pix);
            gtk_container_add(GTK_CONTAINER(hcb->right_arrow),
                                hcb->right_arrow_pix);
        }                       
    gtk_widget_show(hcb->right_arrow);
    if ((hcb->current_offset + hcb->cols*hcb->rows) >= config->num_channels)
	gtk_widget_set_sensitive(hcb->right_arrow, FALSE);
    gtk_box_pack_end(GTK_BOX(obj->base), hcb->right_arrow, 0,0,0);
    gtk_signal_connect(GTK_OBJECT(hcb->right_arrow), "clicked",
			GTK_SIGNAL_FUNC(hchannel_box_scroll), obj);
}

static
void hchannel_box_refresh(struct channel_object *obj, struct channel *chan)
{
    int i, j;
    char str[64];
    struct channel_button *but;
    struct h_chan_box *box = obj->data.h_box;

    for (i=0; i< box->cols; i++)
	for (j=0; j<box->rows; j++)
	{
	    but = ((struct channel_button *)
		(g_ptr_array_index(box->button_array, i*box->rows+j)));
	    sprintf(str,"%i", i*box->rows+j+box->current_offset);
	    if (get_channel(chan, i*box->rows+j+box->current_offset))
	    {
		gtk_label_set(GTK_LABEL(but->label), ((struct channel*)
		   get_channel(chan, i*box->rows+j+box->current_offset))->name);

		gtk_label_set(GTK_LABEL(but->number), str);

		gtk_widget_set_sensitive(but->base, TRUE);
		if (but->signal) gtk_signal_disconnect(GTK_OBJECT(but->base),
							but->signal);
		but->signal = gtk_signal_connect(GTK_OBJECT(but->base),
				"clicked", (GtkSignalFunc)
				callback_set_channel,
				get_channel(config->first_channel,
				i*box->rows+j+box->current_offset));
	    }
	    else
	    {
		gtk_label_set(GTK_LABEL(but->label), "---");
		gtk_label_set(GTK_LABEL(but->number), str);
		gtk_widget_set_sensitive(but->base, FALSE);
	    }
	}
}

static
void vchannel_box_scroll(GtkWidget *but, struct channel_object *obj)
{
    struct v_chan_box *box = obj->data.v_box;

    if (but == box->top_arrow)
    {
        if (box->current_offset >= box->cols)
            box->current_offset -= box->cols;
        if (!box->current_offset)
            gtk_widget_set_sensitive(box->top_arrow, FALSE);
    }
    else if (but == box->bottom_arrow)
    {
        gtk_widget_set_sensitive(box->top_arrow, TRUE);
        box->current_offset += box->cols;
    }
    if ((box->current_offset + box->cols*box->rows) >= config->num_channels)
        gtk_widget_set_sensitive(box->bottom_arrow, FALSE);
    else
        gtk_widget_set_sensitive(box->bottom_arrow, TRUE);

    vchannel_box_refresh(obj, config->first_channel);
}

void vchannel_box_new(struct channel_object *obj)
{
    struct channel_button *temp;
    struct v_chan_box *vcb = obj->data.v_box;
    int i, j;

    obj->base = gtk_vbox_new (FALSE, 5);
//    gtk_container_border_width(GTK_CONTAINER(obj->base), 2);
    obj->refresh = vchannel_box_refresh;

    if (!config->pix.up_arrow_xpm)
        vcb->top_arrow = gtk_button_new_with_label("/\\");
    else
	{
	    vcb->top_arrow = gtk_button_new();
            vcb->top_arrow_pix = gtk_pixmap_new(
					(config->pix.up_arrow_xpm), 
					config->pix.up_arrow_mask);
	    gtk_widget_show(vcb->top_arrow_pix);
	    gtk_container_add(GTK_CONTAINER(vcb->top_arrow),vcb->top_arrow_pix);
	}
    gtk_widget_show(vcb->top_arrow);
    gtk_widget_set_sensitive(vcb->top_arrow, FALSE);
    gtk_box_pack_start(GTK_BOX(obj->base), vcb->top_arrow, 0,0,0);
    gtk_signal_connect(GTK_OBJECT(vcb->top_arrow), "clicked",
                        GTK_SIGNAL_FUNC(vchannel_box_scroll), obj);

    vcb->button_array = g_ptr_array_new();

    for (i=0; i<vcb->cols; i++)
        for (j=0; j<vcb->rows; j++)
        {
            temp = new_channel_button(vcb->but_width, vcb->but_height,
                    vcb->show_numbers,j*vcb->cols+i , vcb->show_text, "---");
            gtk_widget_set_sensitive(temp->base, FALSE);
            g_ptr_array_add(vcb->button_array, temp);
        }

    vcb->button_table = gtk_table_new(vcb->rows, vcb->cols, TRUE);
    for (i=0; i< vcb->cols; i++)
        for (j=0; j<vcb->rows; j++)
        {
            gtk_table_attach_defaults(GTK_TABLE(vcb->button_table),
                ((struct channel_button *)
                (g_ptr_array_index(vcb->button_array, i*vcb->rows+j)))->base,
                i, i+1, j, j+1);
        }
    gtk_widget_show(vcb->button_table);
    gtk_container_add(GTK_CONTAINER(obj->base), vcb->button_table);

    if (!config->pix.down_arrow_xpm)
        vcb->bottom_arrow = gtk_button_new_with_label("\\/");
    else
        {
            vcb->bottom_arrow = gtk_button_new();
            vcb->bottom_arrow_pix = gtk_pixmap_new(
                                        (config->pix.down_arrow_xpm),
                                        config->pix.down_arrow_mask);
            gtk_widget_show(vcb->bottom_arrow_pix);
            gtk_container_add(GTK_CONTAINER(vcb->bottom_arrow),
				vcb->bottom_arrow_pix);
        }
    gtk_widget_show(vcb->bottom_arrow);
    if ((vcb->current_offset + vcb->cols*vcb->rows) >= config->num_channels)
        gtk_widget_set_sensitive(vcb->bottom_arrow, FALSE);
    gtk_box_pack_end(GTK_BOX(obj->base), vcb->bottom_arrow, 0,0,0);
    gtk_signal_connect(GTK_OBJECT(vcb->bottom_arrow), "clicked",
                        GTK_SIGNAL_FUNC(vchannel_box_scroll), obj);
}

static
void vchannel_box_refresh(struct channel_object *obj, struct channel *chan)
{
    int i, j;
    char str[64];
    struct channel_button *but;
    struct v_chan_box *box = obj->data.v_box;

    for (i=0; i< box->cols; i++)
        for (j=0; j<box->rows; j++)
        {
            but = ((struct channel_button *)
                (g_ptr_array_index(box->button_array, i*box->rows+j)));
            sprintf(str,"%i", j*box->cols+i+box->current_offset);
            if (get_channel(chan, j*box->cols+i+box->current_offset))
            {
                gtk_label_set(GTK_LABEL(but->label), ((struct channel*)
                   get_channel(chan, j*box->cols+i+box->current_offset))->name);

                gtk_label_set(GTK_LABEL(but->number), str);

                gtk_widget_set_sensitive(but->base, TRUE);
                if (but->signal) gtk_signal_disconnect(GTK_OBJECT(but->base),
                                                        but->signal);
                but->signal = gtk_signal_connect(GTK_OBJECT(but->base),
                                "clicked", (GtkSignalFunc)
                                callback_set_channel,
                                get_channel(config->first_channel,
                                j*box->cols+i+box->current_offset));
            }
            else
            {
                gtk_label_set(GTK_LABEL(but->label), "---");
                gtk_label_set(GTK_LABEL(but->number), str);
		gtk_widget_set_sensitive(but->base, FALSE);
            }
        }
}
