/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "gui.h"
#include "disp_lcd.h"
#include "fft.h"
#include "popup_menu.h"
#include <malloc.h>
#include <time.h>
#include <stdio.h>
#ifdef USE_GNOME
#include <libgnomeui/libgnomeui.h>
#endif

#define WIDTH 50
#define HEIGHT 15

struct radio_config *config;

GdkPixmap *empty;
GdkPixmap *used1;
GdkPixmap *used2;
GdkPixmap *used3;
GdkPixmap *used4;
GdkPixmap *top_left=0;
GdkPixmap *top_right=0;
GdkPixmap *top=0;
GdkPixmap *left=0;
GdkPixmap *right=0;
GdkPixmap *bot_left=0;
GdkPixmap *bottom=0;
GdkPixmap *bot_right=0;

static
void lcd_draw_pixel (GtkWidget *lcd, int x, int y, int col)
{
    GdkPixmap *pix=0;

    if (!lcd) return;
    if ((x < 0) || (x >= WIDTH) || (y < 0) || (y >= HEIGHT)) return;

    switch (col)
    {
	case 0: pix = empty; break;
	case 1: pix = used1; break;
	case 2: pix = used2; break;
	case 3: pix = used3; break;
	case 4: pix = used4; break;
    }

    gdk_draw_pixmap (lcd->window, lcd->style->black_gc,
		     pix, 0,0, 2+x*4, 2+y*4, 4,4);
}

static void lcd_draw_char (struct display_lcd *lcd, int x, int y,
				char *c[], int col)
{
    int i, j;

    for (i=0; i<5; i++)
	for (j=0; j<5; j++)
	{
	    if ((*c)[i+j*6] == '#')
		lcd_draw_pixel(lcd->lcd, x+i, y+j, col);
	    else if ((*c)[i+j*6] == '.')
		lcd_draw_pixel(lcd->lcd, x+i, y+j, FALSE);
	}
}

static void lcd_draw_bell (struct display_lcd *lcd, int x, int y, int col)
{
    lcd_draw_pixel(lcd->lcd, x+2,	y, col);
    lcd_draw_pixel(lcd->lcd, x+1,	y+1, col);
    lcd_draw_pixel(lcd->lcd, x+2,	y+1, col);
    lcd_draw_pixel(lcd->lcd, x+3,	y+1, col);
    lcd_draw_pixel(lcd->lcd, x+1,	y+2, col);
    lcd_draw_pixel(lcd->lcd, x+2,	y+2, col);
    lcd_draw_pixel(lcd->lcd, x+3,    y+2, col);
    lcd_draw_pixel(lcd->lcd, x,	y+3, col);
    lcd_draw_pixel(lcd->lcd, x+1,	y+3, col);
    lcd_draw_pixel(lcd->lcd, x+2,	y+3, col);
    lcd_draw_pixel(lcd->lcd, x+3,	y+3, col);
    lcd_draw_pixel(lcd->lcd, x+4,	y+3, col);
}

static void lcd_draw_zed (struct display_lcd *lcd, int x, int y, int col)
{
    lcd_draw_pixel(lcd->lcd, x,	y, col);
    lcd_draw_pixel(lcd->lcd, x+1,	y, col);
    lcd_draw_pixel(lcd->lcd, x+2,	y, col);
    lcd_draw_pixel(lcd->lcd, x+3,	y, col);
    lcd_draw_pixel(lcd->lcd, x+2,	y+1, col);
    lcd_draw_pixel(lcd->lcd, x+1,	y+2, col);
    lcd_draw_pixel(lcd->lcd, x,	y+3, col);
    lcd_draw_pixel(lcd->lcd, x+1,	y+3, col);
    lcd_draw_pixel(lcd->lcd, x+2,	y+3, col);
    lcd_draw_pixel(lcd->lcd, x+3,	y+3, col);
}

static void lcd_draw_mute(struct display_lcd *lcd, int x, int y, int col)
{
    int i, j;
    for (i=0; i<15; i++)
	for(j=0; j<3; j++)
	    if ((*icon_mute)[i+j*16] == '#')
		lcd_draw_pixel(lcd->lcd,i+x, j+y, col);
}

static void lcd_draw_tuned(struct display_lcd *lcd, int x, int y, int col)
{
    int i, j;
    for (i=0; i<3; i++)
	for(j=0; j<3; j++)
	    if ((*icon_tuned)[i+j*4] == '#')
		lcd_draw_pixel(lcd->lcd,i+x,j+y, col);
}

static void lcd_draw_stereo(struct display_lcd *lcd, int x, int y, int col)
{
    int i, j;
    for (i=0; i<7; i++)
	for(j=0; j<4; j++)
	    if ((*icon_stereo)[i+j*8] == '#')
		lcd_draw_pixel(lcd->lcd,i+x,j+y, col);
}

static void lcd_draw_vol(struct display_lcd *lcd, int x, int y, int vol,int col)
{
    int i, j;
    for (i=0; i<32; i++)
        for(j=0; j<4; j++)
            if ((*icon_vol)[i+j*33] == '#')
                lcd_draw_pixel(lcd->lcd,i+x,j+y, col);
    for (i=0; i<vol; i++)
	lcd_draw_pixel(lcd->lcd, i/2+x+15, i%2+y+1, 4);
}

static void lcd_write (struct display_lcd *lcd,int x,int y,char *text, int col)
{
    int i;
    for (i=0; i<strlen(text); i++)
    {
	switch (text[i])
        {
	case '1': lcd_draw_char(lcd, x+5*i, y, char_1, col); break;
	case '2': lcd_draw_char(lcd, x+5*i, y, char_2, col); break;
	case '3': lcd_draw_char(lcd, x+5*i, y, char_3, col); break;
	case '4': lcd_draw_char(lcd, x+5*i, y, char_4, col); break;
	case '5': lcd_draw_char(lcd, x+5*i, y, char_5, col); break;
	case '6': lcd_draw_char(lcd, x+5*i, y, char_6, col); break;
	case '7': lcd_draw_char(lcd, x+5*i, y, char_7, col); break;
	case '8': lcd_draw_char(lcd, x+5*i, y, char_8, col); break;
	case '9': lcd_draw_char(lcd, x+5*i, y, char_9, col); break;
	case '0': lcd_draw_char(lcd, x+5*i, y, char_0, col); break;
	case ' ': lcd_draw_char(lcd, x+5*i, y, char_space, col); break;
	case 'a': case 'A': lcd_draw_char(lcd, x+5*i, y, char_A, col); break;
	case 'b': case 'B': lcd_draw_char(lcd, x+5*i, y, char_B, col); break;
	case 'c': case 'C': lcd_draw_char(lcd, x+5*i, y, char_C, col); break;
	case 'd': case 'D': lcd_draw_char(lcd, x+5*i, y, char_D, col); break;
	case 'e': case 'E': lcd_draw_char(lcd, x+5*i, y, char_E, col); break;
	case 'f': case 'F': lcd_draw_char(lcd, x+5*i, y, char_F, col); break;
	case 'g': case 'G': lcd_draw_char(lcd, x+5*i, y, char_G, col); break;
	case 'h': case 'H': lcd_draw_char(lcd, x+5*i, y, char_H, col); break;
	case 'i': case 'I': lcd_draw_char(lcd, x+5*i, y, char_I, col); break;
	case 'j': case 'J': lcd_draw_char(lcd, x+5*i, y, char_J, col); break;
	case 'k': case 'K': lcd_draw_char(lcd, x+5*i, y, char_K, col); break;
	case 'l': case 'L': lcd_draw_char(lcd, x+5*i, y, char_L, col); break;
	case 'm': case 'M': lcd_draw_char(lcd, x+5*i, y, char_M, col); break;
	case 'n': case 'N': lcd_draw_char(lcd, x+5*i, y, char_N, col); break;
	case 'o': case 'O': lcd_draw_char(lcd, x+5*i, y, char_O, col); break;
	case 'p': case 'P': lcd_draw_char(lcd, x+5*i, y, char_P, col); break;
	case 'q': case 'Q': lcd_draw_char(lcd, x+5*i, y, char_Q, col); break;
	case 'r': case 'R': lcd_draw_char(lcd, x+5*i, y, char_R, col); break;
	case 's': case 'S': lcd_draw_char(lcd, x+5*i, y, char_S, col); break;
	case 't': case 'T': lcd_draw_char(lcd, x+5*i, y, char_T, col); break;
	case 'u': case 'U': lcd_draw_char(lcd, x+5*i, y, char_U, col); break;
	case 'v': case 'V': lcd_draw_char(lcd, x+5*i, y, char_V, col); break;
	case 'w': case 'W': lcd_draw_char(lcd, x+5*i, y, char_W, col); break;
	case 'x': case 'X': lcd_draw_char(lcd, x+5*i, y, char_X, col); break;
	case 'y': case 'Y': lcd_draw_char(lcd, x+5*i, y, char_Y, col); break;
	case 'z': case 'Z': lcd_draw_char(lcd, x+5*i, y, char_Z, col); break;
	case '-': lcd_draw_char(lcd, x+5*i, y, char_minus, col); break;
	case '+': lcd_draw_char(lcd, x+5*i, y, char_plus, col); break;
	case '=': lcd_draw_char(lcd, x+5*i, y, char_eq, col); break;
	case '_': lcd_draw_char(lcd, x+5*i, y, char__, col); break;
	case '|': lcd_draw_char(lcd, x+5*i, y, char_pipe, col); break;
	case '\\': lcd_draw_char(lcd, x+5*i, y, char_bs, col); break;
	case '/': lcd_draw_char(lcd, x+5*i, y, char_slash, col); break; 
	case '*': lcd_draw_char(lcd, x+5*i, y, char_star, col); break; 
	case '(': lcd_draw_char(lcd, x+5*i, y, char_ob, col); break; 
	case ')': lcd_draw_char(lcd, x+5*i, y, char_cb, col); break; 
	case '[': lcd_draw_char(lcd, x+5*i, y, char_osb, col); break;
	case ']': lcd_draw_char(lcd, x+5*i, y, char_csb, col); break;
	case '{': lcd_draw_char(lcd, x+5*i, y, char_ocb, col); break;
	case '}': lcd_draw_char(lcd, x+5*i, y, char_ccb, col); break;
	case '&': lcd_draw_char(lcd, x+5*i, y, char_and, col); break;
	case '^': lcd_draw_char(lcd, x+5*i, y, char_hat, col); break;
	case '%': lcd_draw_char(lcd, x+5*i, y, char_per, col); break;
	case '#': lcd_draw_char(lcd, x+5*i, y, char_hash, col); break;
	case '$': lcd_draw_char(lcd, x+5*i, y, char_dol, col); break;
	case '@': lcd_draw_char(lcd, x+5*i, y, char_at, col); break;
	case '!': lcd_draw_char(lcd, x+5*i, y, char_not, col); break;
	case '~': lcd_draw_char(lcd, x+5*i, y, char_tilde, col); break;
	case '\"': lcd_draw_char(lcd, x+5*i, y, char_quote, col); break;
	case '\'': lcd_draw_char(lcd, x+5*i, y, char_apos, col); break;
	case '`': lcd_draw_char(lcd, x+5*i, y, char_bpos, col); break;
	case ':': lcd_draw_char(lcd, x+5*i, y, char_col, col); break;
	case ';': lcd_draw_char(lcd, x+5*i, y, char_scol, col); break;
	case '?': lcd_draw_char(lcd, x+5*i, y, char_ques, col); break;
	case '<': lcd_draw_char(lcd, x+5*i, y, char_left, col); break;
	case '>': lcd_draw_char(lcd, x+5*i, y, char_right, col); break;
	case '.': lcd_draw_char(lcd, x+5*i, y, char_dot, col); break;
	case ',': lcd_draw_char(lcd, x+5*i, y, char_comma, col); break;
	default : lcd_draw_char(lcd, x+5*i, y, char_ques, col); break;
	}
    }
}

static
GdkBitmap *setup_pixmaps(struct display_lcd *lcd)
{
    GdkBitmap *mask;
    GtkStyle *style;
    GtkWidget *window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

    gtk_widget_realize(window);

    style = gtk_widget_get_style (lcd->lcd);
    top_left = gdk_pixmap_create_from_xpm_d (window->window,  &mask,
			&style->bg[GTK_STATE_NORMAL], (gchar**) pix_top_left);
    left = gdk_pixmap_create_from_xpm_d (window->window,  &mask,
			&style->bg[GTK_STATE_NORMAL], (gchar**) pix_left);
    top = gdk_pixmap_create_from_xpm_d (window->window,  &mask,
			&style->bg[GTK_STATE_NORMAL], (gchar**) pix_top);
    top_right = gdk_pixmap_create_from_xpm_d (window->window,  &mask,
			&style->bg[GTK_STATE_NORMAL], (gchar**) pix_top_right);
    right = gdk_pixmap_create_from_xpm_d (window->window,  &mask,
			&style->bg[GTK_STATE_NORMAL], (gchar**) pix_right);
    bot_left = gdk_pixmap_create_from_xpm_d (window->window,  &mask,
			&style->bg[GTK_STATE_NORMAL], (gchar**) pix_bot_left);
    bottom = gdk_pixmap_create_from_xpm_d (window->window,  &mask,    
			&style->bg[GTK_STATE_NORMAL], (gchar**) pix_bottom);
    bot_right = gdk_pixmap_create_from_xpm_d (window->window,  &mask,    
			&style->bg[GTK_STATE_NORMAL], (gchar**) pix_bot_right);
    empty = gdk_pixmap_create_from_xpm_d (window->window, &mask,
			&style->bg[GTK_STATE_NORMAL], (gchar**) pix_empty);
    used1 = gdk_pixmap_create_from_xpm_d (window->window,  &mask,
                        &style->bg[GTK_STATE_NORMAL], (gchar**) pix_used_1);
    used2 = gdk_pixmap_create_from_xpm_d (window->window,  &mask,
                        &style->bg[GTK_STATE_NORMAL], (gchar**) pix_used_2);
    used3 = gdk_pixmap_create_from_xpm_d (window->window,  &mask,
                        &style->bg[GTK_STATE_NORMAL], (gchar**) pix_used_3);
    used4 = gdk_pixmap_create_from_xpm_d (window->window,  &mask,
			&style->bg[GTK_STATE_NORMAL], (gchar**) pix_used_4);

    gtk_widget_destroy(window);

    return mask;
}

static
void draw_borders(struct display_lcd *lcd)
{
    int i;

    gdk_draw_pixmap (lcd->lcd->window, lcd->lcd->style->black_gc,
		top_left, 0,0, 0,0, 2,2);
    gdk_draw_pixmap (lcd->lcd->window, lcd->lcd->style->black_gc,
		top_right, 0,0, 2+WIDTH*4, 0, 2,2);
    for (i=0;i<WIDTH;i++)
    {
	gdk_draw_pixmap (lcd->lcd->window, lcd->lcd->style->black_gc,
			top, 0,0, 2+i*4,0, 4,2);
	gdk_draw_pixmap (lcd->lcd->window, lcd->lcd->style->black_gc,
			bottom, 0,0, 2+i*4,2+HEIGHT*4, 4,2);
    }
    for (i=0;i<HEIGHT;i++)
    {
	gdk_draw_pixmap (lcd->lcd->window, lcd->lcd->style->black_gc,
			left, 0,0, 0,2+i*4, 2,4);
	gdk_draw_pixmap (lcd->lcd->window, lcd->lcd->style->black_gc,
			right, 0,0, 2+WIDTH*4,2+i*4, 2,4);

    }
    gdk_draw_pixmap (lcd->lcd->window, lcd->lcd->style->black_gc,
		    bot_left, 0,0, 0, 2+HEIGHT*4, 2,2);
    gdk_draw_pixmap (lcd->lcd->window, lcd->lcd->style->black_gc,
		    bot_right, 0,0, 2+WIDTH*4, 2+HEIGHT*4, 2,2);
}

void lcd_set_frequency(struct display *disp, double freq)
{
    disp->frequency = freq;
    if ((disp->data.lcd->state == 0) || (disp->data.lcd->state == 2))
	disp->data.lcd->state += 10000;
}

void lcd_set_name(struct display *disp,  char *name)
{
    if (disp->chan_name)
	free(disp->chan_name);
    disp->chan_name = malloc(strlen(name)+1);
    strcpy(disp->chan_name, name);

    if ((disp->data.lcd->state == 0) || (disp->data.lcd->state == 1))
	disp->data.lcd->state += 10000;
}

void lcd_set_chan(struct display *disp, int num)
{
    disp->chan_number = num;
    if (disp->data.lcd->state == 0)
	disp->data.lcd->state += 10000;
}

void lcd_set_volume(struct display *disp, int vol)
{
    disp->volume = vol;
    if (disp->data.lcd->state == 0)
	disp->data.lcd->state += 10000;
}

void lcd_set_mute(struct display *disp, gboolean mute)
{
    disp->muted = mute;
    if (disp->data.lcd->state == 0)
	disp->data.lcd->state += 10000;
}

void lcd_set_stereo(struct display *disp, gboolean stereo)
{
    disp->stereo = stereo;
    if (disp->data.lcd->state == 0)
	disp->data.lcd->state += 10000;
}

void lcd_set_tuned(struct display *disp, gboolean tuned)
{
    disp->tuned = tuned;
    if (disp->data.lcd->state == 0)
	disp->data.lcd->state += 10000;
}

void lcd_set_alarm(struct display *disp, gboolean state, int hours, int min)
{
    disp->alarm_on = state;
    disp->alarm_hours = hours;
    disp->alarm_min = min;
    if ((disp->data.lcd->state == 0) || (disp->data.lcd->state == 4))
	disp->data.lcd->state += 10000;
}

void lcd_set_timer(struct display *disp, gboolean state, int time)
{
    disp->timer_on = state;
    disp->timer_time = time;
    if (disp->data.lcd->state == 0)
	disp->data.lcd->state += 10000;
}

static struct display_operations lcd_ops = {
    lcd_set_frequency,
    lcd_set_name,
    lcd_set_chan,
    lcd_set_volume,
    lcd_set_mute,
    lcd_set_stereo,
    lcd_set_tuned,
    lcd_set_alarm,
    lcd_set_timer
};

static int lcd_busy = 0;
int lcd_timer_event(struct display_lcd *lcd)
{
    char str[256];
    time_t t;
    int i, j, k, l;
    struct tm *tm;
    struct fft_data *fft;

    if (!lcd) return FALSE;
    if (lcd_busy) return TRUE;
    lcd_busy = 1; 

    switch(lcd->state)
    {
	case 0:
		switch (lcd->sub_state)
		{
		case 0:
			sprintf(str, "%d", lcd->parent->chan_number);
			if (lcd->parent->chan_number >=0)
			    lcd_write(lcd, 0, 0, str, 1);
			if (lcd->parent->alarm_on)
			    lcd_draw_bell(lcd, WIDTH-5, HEIGHT-4, 4);
			if (lcd->parent->timer_on)
			    lcd_draw_zed(lcd, WIDTH-10, HEIGHT-4, 4);

			if (lcd->parent->chan_name) {
			    if (strlen(lcd->parent->chan_name)!=0)
			    {
				if (strlen(lcd->parent->chan_name)*5 <= WIDTH)
				    lcd_write(lcd, WIDTH/2 -
				    strlen(lcd->parent->chan_name)*5/2+1,
				    5, lcd->parent->chan_name, 1);
				else lcd->sub_state++;
			    }
			    else
			    {
				sprintf(str, "%2.2f", lcd->parent->frequency);
				lcd_write(lcd, WIDTH/2 -
				strlen(str)*5/2-2, 5, str, 1);

				for (i=0; i<11; i++)
				    for(j=0; j<4; j++)
					if ((*icon_mhz)[i+j*12] == '#')
					    lcd_draw_pixel(lcd->lcd, WIDTH/2 +
						strlen(str)*5/2
						   + i - 1, j+6, TRUE);
			    }
			}

			if (lcd->parent->muted)
			    lcd_draw_mute(lcd, 5, HEIGHT-4, 1);

			if (lcd->parent->tuned)
			    lcd_draw_tuned(lcd, 0, HEIGHT-4, 2);

			if (lcd->parent->stereo)
			    lcd_draw_stereo(lcd, WIDTH-7, 0, 4);

			lcd_draw_vol(lcd, 10, 0, lcd->parent->volume, 3);

			lcd->sub_state++;
			break;
		case 1: break;
		default:
			{
			    i = -1;
			    if (lcd->parent->chan_name)
				i = strlen(lcd->parent->chan_name)*5;

			    if (lcd->scroll_pos < 3)
				lcd->scroll_pos = 200-i/2;
			    if (i >= 0) {
			    if (lcd->scroll_pos < 400)
			    {
				lcd_write(lcd, lcd->scroll_pos-200 ,
				    5, lcd->parent->chan_name, 1);

				lcd_draw_pixel(lcd->lcd,lcd->scroll_pos-201,5,0);
				lcd_draw_pixel(lcd->lcd,lcd->scroll_pos-201,6,0);
				lcd_draw_pixel(lcd->lcd,lcd->scroll_pos-201,7,0);
				lcd_draw_pixel(lcd->lcd,lcd->scroll_pos-201,8,0);
				lcd_draw_pixel(lcd->lcd,lcd->scroll_pos-201,9,0);

				lcd->scroll_pos++;
				if (lcd->scroll_pos-200 >= WIDTH/3)
				    lcd->scroll_pos += 400;
			    } else
			    {
				lcd->scroll_pos--;
				if (lcd->scroll_pos-600 <= WIDTH*2/3-i)
				    lcd->scroll_pos -= 400;
				lcd_write(lcd, lcd->scroll_pos-600 ,
				    5, lcd->parent->chan_name, 1);
			    }
			    }
			    break;
			}
		}
		break;
	case 1:
		if (lcd->parent->chan_name)
		{
		    if (strlen(lcd->parent->chan_name)!=0)
			lcd_write(lcd, WIDTH/2 -
			    strlen(lcd->parent->chan_name)*5/2+1,
			    5, lcd->parent->chan_name, 1);
		    else
			lcd_write(lcd, WIDTH/2 -
			    strlen("---")*5/2+1, 5, "---", 1);
		}
		break;
	case 2:
		sprintf(str, "%2.2f", lcd->parent->frequency);
		lcd_write(lcd, WIDTH/2 -
		    strlen(str)*5/2-2, 5, str, 1);

		for (i=0; i<11; i++)
		    for(j=0; j<4; j++)
			if ((*icon_mhz)[i+j*12] == '#')
			    lcd_draw_pixel(lcd->lcd, WIDTH/2 + strlen(str)*5/2
					   + i - 1, j+6, 1);
		break;
	case 3:
		switch (lcd->sub_state)
		{
		case 0 : lcd->sub_state++; 
			gtk_timeout_remove(lcd->timeout);
			fft_init(lcd);
			memset (lcd->fft_data, 0, FFT_SIZE);
			break;
		case 1:
		}
		fft = get_fft_block();
		if (!fft)
		    {
			lcd_write(lcd, WIDTH/2 -
				strlen("Can't open")*5/2+1,
				1, "Can't open", 1);
			lcd_write(lcd, WIDTH/2 -
				strlen("/dev/dsp")*5/2+1,
				8, "/dev/dsp", 1);
			break;
		    }

		for (i=0; i<WIDTH; i++)
		{
		    switch (config->fft_style)
		    {
		    default:
		    case 0:
			{
			    l = lcd->fft_data[i*FFT_SIZE/WIDTH];
			    for (j = 0; j <= HEIGHT; j++)
			    {
				k = fft->data[i*FFT_SIZE/WIDTH];
				if (((j >= k) && (j <= l)) ||
					((j <= k) && (j >= l)))
				{
				    if (j > k)
					lcd_draw_pixel(lcd->lcd, i, HEIGHT-j, 0);
				    else
				    {
					k = j*100/HEIGHT;
					if (k < 70)
					    lcd_draw_pixel(lcd->lcd,i,HEIGHT-j, 3);
					else  if (k < 90)
					    lcd_draw_pixel(lcd->lcd,i,HEIGHT-j, 1);
					else
					    lcd_draw_pixel(lcd->lcd,i,HEIGHT-j, 2);
				    }
				}
			    }
			    break;
			}
		    case 1:
			{
			    l = fft->data[i*FFT_SIZE/WIDTH];

			    for (j = 0; j <= HEIGHT; j++)
			    {
				k = l*100/HEIGHT;
				if (j > l)
				    lcd_draw_pixel(lcd->lcd, i, HEIGHT-j, 0);
				else
				if (k < 60)
				    lcd_draw_pixel(lcd->lcd, i, HEIGHT-j, 3);
				else  if (k < 85)
				    lcd_draw_pixel(lcd->lcd, i, HEIGHT-j, 1);
				else
				    lcd_draw_pixel(lcd->lcd, i, HEIGHT-j, 2);
			    }
			}
		    }
		}
		memcpy(lcd->fft_data, fft->data, FFT_SIZE);
		break;
	case 4: if (!lcd->sub_state)
		{
		    lcd_write(lcd, WIDTH/2-22, 1, "Alarm:", 1);
		    if (lcd->parent->alarm_on)
		    {
			lcd_write(lcd, WIDTH/2+9, 1, "ON", 3);
			sprintf(str, "%2.2d", lcd->parent->alarm_hours);
			lcd_write(lcd, WIDTH/2-12, 8, str, 1);
			sprintf(str, ":%2.2d", lcd->parent->alarm_min);
			lcd_write(lcd, WIDTH/2-2, 8, str, 1);
		    }
		    else
		    {
			lcd_write(lcd, WIDTH/2+9, 1, "OFF", 3);
		    }
		    lcd->sub_state++;
		}
		break;
	case 5: if (lcd->sub_state > 9) lcd->sub_state = 0;
		if (!lcd->sub_state)
		{
		    time(&t);
		    tm = localtime(&t);
		    sprintf(str, "%2d:%2.2d", tm->tm_hour, tm->tm_min);
		    lcd_write(lcd, WIDTH/2-12, 5, str, 4);
		}
		else if (lcd->sub_state == 5)
		    lcd_write(lcd, WIDTH/2-2, 5, " ", 1);
		lcd->sub_state++;
		break;
    }
    if (lcd->state >= 20000)
    {
	lcd->state -= 10000;

	if (lcd->state == 3+10000)
	{
	    gtk_timeout_remove(lcd->timeout);
	}
	else if (lcd->state == 4+10000)
	{
	    fft_close();
	    lcd->timeout = gtk_timeout_add(100, (GtkFunction)lcd_timer_event, lcd);         
	}
    }
    if (lcd->state >= 10000)
    {
	lcd->sub_state = 0;
	for (i=0; i<WIDTH; i++)
	    for (j=0; j<HEIGHT; j++)
		lcd_draw_pixel(lcd->lcd, i, j, FALSE);
	draw_borders(lcd);

	lcd_busy = 0;    
	lcd->state -= 10000;
	lcd_timer_event(lcd);    
    }

    lcd_busy = 0;
    return TRUE;
}

static
gint lcd_clicked_callback(struct display_lcd *lcd, GdkEvent *data)
{
    if (data->type == GDK_BUTTON_PRESS)
    {
	if (data->button.button == 1)
	    {
		if (lcd->state >= 10000) lcd->state -= 10000;
		if (lcd->state >= 10000) lcd->state -= 10000;
		switch(lcd->state)
		{
		case 5: lcd->state = 0;
			break;
		default: lcd->state++;
		}
		lcd->state += 20000;
		lcd_timer_event(lcd);
		return TRUE;
	    }
	else
	if (data->button.button == 3)
	    {
#ifdef USE_GNOME
		gnome_popup_menu_do_popup(popup_menu, NULL, NULL,
				(gpointer)data, popup_menu);
					    
#else
		int mx = data->button.x;
		int my = data->button.y;
		GdkModifierType modmask;
		GtkWidget *popup;
		struct Pos {gint x;gint y;} pos;

		gdk_window_get_pointer(NULL,&mx,&my, &modmask);

		popup = gtk_item_factory_get_widget (popup_menu, "");
		pos.x = mx;
		pos.y = my;
		gtk_menu_popup(GTK_MENU(popup), NULL, NULL, NULL, &pos, 1, GDK_CURRENT_TIME);

#endif
		return TRUE;
	    }
    }
    /* Tell calling code that we have not handled this event; pass it on. */
    return FALSE;
}

static
gint lcd_draw_event(struct display_lcd *lcd, GdkEvent *data)
{
    lcd->state += 10000;
    lcd_timer_event(lcd);

    return TRUE;
}

void disp_lcd_new(struct display *display)
{
    GdkBitmap *mask;

    display->base = gtk_hbox_new(FALSE, 0);

    display->ops = &lcd_ops;
    display->data.lcd = malloc (sizeof(struct display_lcd));
    display->data.lcd->vbox = gtk_vbox_new(FALSE, 0);
    display->data.lcd->event_box = gtk_event_box_new();
    display->data.lcd->lcd = gtk_drawing_area_new();
    display->data.lcd->state = 10000;
    display->data.lcd->sub_state = 0;
    display->data.lcd->scroll_pos = 0;
    display->data.lcd->parent = display;

    gtk_widget_show(display->data.lcd->vbox);
    gtk_widget_show(display->data.lcd->event_box);
    gtk_widget_show(display->data.lcd->lcd);

    gtk_container_add(GTK_CONTAINER(display->data.lcd->event_box),
			display->data.lcd->vbox);
    gtk_box_pack_start(GTK_BOX(display->base), display->data.lcd->event_box,
			TRUE, FALSE, 0);

    gtk_drawing_area_size(GTK_DRAWING_AREA(display->data.lcd->lcd),
				4+WIDTH*4, 4+HEIGHT*4);
    gtk_box_pack_start(GTK_BOX(display->data.lcd->vbox),
				display->data.lcd->lcd, TRUE, FALSE, 0);
    mask = setup_pixmaps(display->data.lcd);

    display->data.lcd->fft_data = malloc (FFT_SIZE);

    display->data.lcd->timeout = gtk_timeout_add(100,
			(GtkFunction)lcd_timer_event, display->data.lcd);

    gtk_signal_connect_object(GTK_OBJECT(display->data.lcd->event_box), "event",
				GTK_SIGNAL_FUNC (lcd_clicked_callback),
				(GtkObject*)display->data.lcd);
    gtk_signal_connect_object (GTK_OBJECT(display->data.lcd->lcd),
				/*"draw"*/ "event",
				GTK_SIGNAL_FUNC(lcd_draw_event),
				(GtkObject*)display->data.lcd);
}
