/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "fft.h"
#include "soundfft.c"
#include <gdk/gdk.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <malloc.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <unistd.h>

#define LOW 75 
#define HIGH 18000

char dev[10] = "/dev/dsp";
short *data1;
int snd_dev=-1;
int *logmap, gdk_input;
struct fft_data *fft_dat;
struct display_lcd *lcd;
extern int lcd_timer_event(struct display_lcd *lcd);

void make_logmap()
{
    int   i,w=FFT_SIZE;
    float freq,up;

    logmap   = (int*) malloc((w+1)*sizeof(int));

    up = log(HIGH/LOW)/log(2);
    for (i = 0; i <= w; i++) {
        freq = pow(2,up*i/w)* LOW;
        logmap[i] = (int)(DATA_SIZE*freq/44100);
    }
}

void readin_audio(gpointer data, int fd, GdkInputCondition condition)
{
    int re, im, i, j, ab, max;
    int len = DATA_SIZE>>1;

    if (snd_dev != -1)
	read(snd_dev, data1, DATA_SIZE*2);

    if (snd_dev == -1) return;
    
    RealFFT(data1);

    for (i=0; i<FFT_SIZE; i++)
    {
        j = logmap[i];
        max = 0;
        do {
            if (j >= len)
                break;
            re = data1[BitReversed[j]];
            im = data1[BitReversed[j]+1];
            ab = re*re+im*im;
            if (ab > max)
                max = ab;
            j++;
        } while (j < logmap[i+1]);
//      fft_dat->data[i] = (int)sqrt(max) / 256;
        fft_dat->data[i] = (int)sqrt(max) * logmap[i/4+0] / logmap[i/4+10] /128;
    }

    lcd_timer_event(lcd);
}

void fft_init(struct display_lcd *disp)
{
    int frag, fmt, chans, rate, blocksize;

    if (snd_dev != -1) return;
    snd_dev = open(dev, O_RDONLY);
    lcd = disp;

    fft_dat = malloc(sizeof(struct fft_data));

    frag = 0x7fff000c; /* 1024 (better depend on rate) */
    fmt = 1;
    chans = 1;
    rate = 44100;
    blocksize = 8192;

    ioctl(snd_dev, SNDCTL_DSP_SETFRAGMENT, &frag);
    ioctl(snd_dev, SNDCTL_DSP_SETFMT, &fmt);
    ioctl(snd_dev, SNDCTL_DSP_CHANNELS, &chans);
    ioctl(snd_dev, SNDCTL_DSP_SPEED, &rate);
    ioctl(snd_dev, SNDCTL_DSP_GETBLKSIZE,  &blocksize);

    InitializeFFT(DATA_SIZE);
    make_logmap();

    data1 = malloc(DATA_SIZE*sizeof(short));

    gdk_input = gdk_input_add(snd_dev, GDK_INPUT_READ, readin_audio, NULL);
}

struct fft_data *get_fft_block ()
{
    return fft_dat;
}

void fft_close()
{
    gdk_input_remove(gdk_input);
    if (snd_dev != -1)
	close(snd_dev);
    snd_dev = -1;
    free (fft_dat);
    free (logmap);
    free (data1);

    EndFFT();
}
