/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "gui.h"
#include "../hardware.h"
#include "../callbacks.h"
#include <malloc.h>

extern struct radio_config *config;

struct freq_info {
    struct freq_bar *fbar;
    int action;
};

static
void hfbar_freq_callback (GtkWidget *window, struct freq_bar *hfbar)
{
    hfbar->frequency = GTK_ADJUSTMENT(hfbar->data.hfbar->scale)->value;
    if (hfbar->data.hfbar->ruler)
    {
	GTK_RULER(hfbar->data.hfbar->ruler)->position = hfbar->frequency;
	gtk_ruler_draw_pos(GTK_RULER(hfbar->data.hfbar->ruler));
    }

    if (hfbar->data.hfbar->ignore)
    {
	hfbar->data.hfbar->ignore = FALSE;
	return;
    }
    callback_set_freq(window, hfbar->frequency);
}

static
void hfbar_button_callback (GtkWidget *window, struct freq_info *info)
{
    switch (info->action)
    {
	case 0: callback_scan(-card_get_frequency_step ());
		break;
	case 1: gtk_adjustment_set_value(GTK_ADJUSTMENT (info->fbar->
			data.hfbar->scale), info->fbar->frequency -
			card_get_frequency_step ());
		break;
	case 2: gtk_adjustment_set_value(GTK_ADJUSTMENT (info->fbar->
			data.hfbar->scale), info->fbar->frequency +
			card_get_frequency_step ());
		break;
	case 3: callback_scan(card_get_frequency_step ());
		break;
    }
}

static
void hfbar_set_freq(struct freq_bar* hfbar, double freq)
{
    if (hfbar->frequency != freq)
	hfbar->data.hfbar->ignore = TRUE;
    gtk_adjustment_set_value(GTK_ADJUSTMENT(hfbar->data.hfbar->scale), freq);
    hfbar->frequency = freq;
}

static
void hfbar_update(struct freq_bar* hfbar)
{
    if (hfbar->data.hfbar->ruler)
    {
	gtk_ruler_set_range (GTK_RULER(hfbar->data.hfbar->ruler),
		card_get_min_frequency (), card_get_max_frequency (),
		card_get_min_frequency (), 0);
    }
    GTK_ADJUSTMENT(hfbar->data.sfbar->scale)->lower = card_get_min_frequency();
    GTK_ADJUSTMENT(hfbar->data.sfbar->scale)->upper = card_get_max_frequency()+1;
}

void hfreq_bar_new(struct freq_bar *fbar, int mode)
{
    struct freq_info *info;

    if (!fbar) return;

    fbar->data.hfbar = malloc (sizeof(struct horiz_fbar));

    fbar->frequency = card_get_frequency ();
    fbar->data.hfbar->scale = gtk_adjustment_new(fbar->frequency,
		card_get_min_frequency (), card_get_max_frequency ()+1,
		card_get_frequency_step (),1,1);

    fbar->base = gtk_hbox_new(FALSE, 0);
    fbar->set_freq = hfbar_set_freq;
    fbar->update = hfbar_update;
    fbar->data.hfbar->ignore = FALSE;

    fbar->data.hfbar->slider = gtk_hscale_new(GTK_ADJUSTMENT
					(fbar->data.hfbar->scale));

    gtk_scale_set_digits( GTK_SCALE (fbar->data.hfbar->slider), 2);

    switch (mode)
    {
	case 2: gtk_scale_set_value_pos(GTK_SCALE(fbar->data.hfbar->slider),
			GTK_POS_TOP); break;
	case 3: gtk_scale_set_value_pos(GTK_SCALE(fbar->data.hfbar->slider),
			GTK_POS_BOTTOM); break;
	case 4: gtk_scale_set_value_pos(GTK_SCALE(fbar->data.hfbar->slider),
			GTK_POS_LEFT); break;
	case 5: gtk_scale_set_value_pos(GTK_SCALE(fbar->data.hfbar->slider),
			GTK_POS_RIGHT); break;
    }
    if (mode <= 1)
	gtk_scale_set_draw_value(GTK_SCALE (fbar->data.hfbar->slider), FALSE);

    if (!config->pix.left_arrow_xpm)
        fbar->data.hfbar->left = gtk_button_new_with_label("<");
    else
	{
	    fbar->data.hfbar->left = gtk_button_new();
	    fbar->data.hfbar->left_xpm = gtk_pixmap_new(
					(config->pix.left_arrow_xpm),
					config->pix.left_arrow_mask);
	    gtk_container_add(GTK_CONTAINER(fbar->data.hfbar->left),
				fbar->data.hfbar->left_xpm);
	}

    if (!config->pix.right_arrow_xpm)
        fbar->data.hfbar->right = gtk_button_new_with_label(">");
    else
	{
	    fbar->data.hfbar->right = gtk_button_new();
	    fbar->data.hfbar->right_xpm = gtk_pixmap_new(
					(config->pix.right_arrow_xpm),
					config->pix.right_arrow_mask);
	    gtk_container_add(GTK_CONTAINER(fbar->data.hfbar->right),
				fbar->data.hfbar->right_xpm);
	}

    if (!config->pix.scan_left_xpm)
        fbar->data.hfbar->scanleft = gtk_button_new_with_label("<<");
    else
        {
	    fbar->data.hfbar->scanleft = gtk_button_new();
	    fbar->data.hfbar->scanleft_xpm =
			gtk_pixmap_new( config->pix.scan_left_xpm,
					config->pix.scan_left_mask);
	    gtk_container_add(GTK_CONTAINER(fbar->data.hfbar->scanleft),
				fbar->data.hfbar->scanleft_xpm);
	}

    if (!config->pix.scan_right_xpm)
        fbar->data.hfbar->scanright = gtk_button_new_with_label(">>");
    else    
        {    
	    fbar->data.hfbar->scanright = gtk_button_new();
	    fbar->data.hfbar->scanright_xpm =
		    gtk_pixmap_new( config->pix.scan_right_xpm,
				    config->pix.scan_right_mask);
	    gtk_container_add(GTK_CONTAINER(fbar->data.hfbar->scanright),
			        fbar->data.hfbar->scanright_xpm);
        }

    fbar->data.hfbar->vcontainer = gtk_vbox_new(0, FALSE);

    gtk_box_pack_start(GTK_BOX(fbar->base), fbar->data.hfbar->scanleft, 0,0,0);
    gtk_box_pack_start(GTK_BOX(fbar->base), fbar->data.hfbar->left, 0,0,0);
    gtk_container_add(GTK_CONTAINER(fbar->base),fbar->data.hfbar->vcontainer);
    gtk_box_pack_end(GTK_BOX(fbar->base), fbar->data.hfbar->scanright, 0,0,0);
    gtk_box_pack_end(GTK_BOX(fbar->base), fbar->data.hfbar->right, 0,0,0);

    gtk_box_pack_end(GTK_BOX(fbar->data.hfbar->vcontainer),
			fbar->data.hfbar->slider, 0,0,0);

    fbar->data.hfbar->ruler = NULL;
    if (mode == 1)
    {
	GtkWidget *box;
	GtkStyle *style;

	fbar->data.hfbar->ruler = gtk_hruler_new();
	gtk_ruler_set_metric( GTK_RULER(fbar->data.hfbar->ruler), GTK_PIXELS );
	gtk_widget_set_sensitive(fbar->data.hfbar->ruler,FALSE);
	gtk_ruler_set_range (GTK_RULER(fbar->data.hfbar->ruler),
		card_get_min_frequency (), card_get_max_frequency (),
		card_get_min_frequency (), 0);

	fbar->data.hfbar->cont2 = gtk_hbox_new(0, FALSE);

	box = gtk_button_new_with_label(".FM.");
	style = gtk_style_copy(gtk_widget_get_style(box));
	style->font = gdk_font_load(
		"-adobe-helvetica-medium-r-*-*-*-80-*-*-*-*-*-*");
	gtk_widget_set_style(GTK_BIN (box)->child, style);

	gtk_widget_set_usize(box, 19, -1);
	gtk_widget_set_sensitive(box, FALSE);
	gtk_box_pack_start(GTK_BOX(fbar->data.hfbar->cont2),
			    box, 0,0,0);
	box = gtk_button_new_with_label("Mhz");
	gtk_widget_set_style(GTK_BIN (box)->child, style);
	gtk_style_unref(style);
	/*
	style = gtk_style_copy(gtk_widget_get_style(fbar->data.hfbar->ruler));
	style->font = gdk_font_load(
		"-adobe-helvetica-medium-r-*-*-*-80-*-*-*-*-*-*");
	gtk_widget_set_style(fbar->data.hfbar->ruler, style);
	gtk_style_unref(style);
 */
	gtk_widget_set_usize(box, 19, -1);
	gtk_widget_set_sensitive(box,FALSE);
	gtk_box_pack_end(GTK_BOX(fbar->data.hfbar->cont2),
			    box, 0,0,0);

        gtk_container_add(GTK_CONTAINER(fbar->data.hfbar->cont2),
                                fbar->data.hfbar->ruler);
	gtk_container_add(GTK_CONTAINER(fbar->data.hfbar->vcontainer),
                                fbar->data.hfbar->cont2);
    }

    gtk_signal_connect(GTK_OBJECT(fbar->data.hfbar->scale), "value_changed",
			(GtkSignalFunc)hfbar_freq_callback , fbar);

    info = malloc (sizeof(struct freq_info));
    info->action = 1;
    info->fbar = fbar;
    gtk_signal_connect(GTK_OBJECT(fbar->data.hfbar->left), "clicked",
                        (GtkSignalFunc)hfbar_button_callback , info);

    info = malloc (sizeof(struct freq_info));
    info->action = 2;
    info->fbar = fbar;
    gtk_signal_connect(GTK_OBJECT(fbar->data.hfbar->right), "clicked",
                        (GtkSignalFunc)hfbar_button_callback , info);

    info = malloc (sizeof(struct freq_info));
    info->action = 0;
    info->fbar = fbar;
    gtk_signal_connect(GTK_OBJECT(fbar->data.hfbar->scanleft), "clicked",
                        (GtkSignalFunc)hfbar_button_callback , info);

    info = malloc (sizeof(struct freq_info));
    info->action = 3;
    info->fbar = fbar;
    gtk_signal_connect(GTK_OBJECT(fbar->data.hfbar->scanright), "clicked",
                        (GtkSignalFunc)hfbar_button_callback , info);

    gtk_widget_show_all(fbar->base);
}

static
void sfbar_set_freq(struct freq_bar* fbar, double freq)
{
    if (fbar->frequency != freq)
        fbar->data.sfbar->ignore = TRUE;
    gtk_adjustment_set_value(GTK_ADJUSTMENT(fbar->data.sfbar->scale), freq);
    fbar->frequency = freq;
}

static
void sfbar_update(struct freq_bar* fbar)
{
    GTK_ADJUSTMENT(fbar->data.sfbar->scale)->lower = card_get_min_frequency();
    GTK_ADJUSTMENT(fbar->data.sfbar->scale)->upper = card_get_max_frequency()+1;
}

static
void sfbar_freq_callback (GtkWidget *window, struct freq_bar *fbar)
{
    fbar->frequency = GTK_ADJUSTMENT(fbar->data.sfbar->scale)->value;

    if (fbar->data.sfbar->ignore)
    {
        fbar->data.sfbar->ignore = FALSE;
        return;
    }
    callback_set_freq(window, fbar->frequency);
}

static
void sfbar_button_callback (GtkWidget *window, struct freq_info *info)
{
    switch (info->action)
    {
        case 0: callback_scan(-card_get_frequency_step ());
                break;
        case 1: gtk_adjustment_set_value(GTK_ADJUSTMENT (info->fbar->
                        data.sfbar->scale), info->fbar->frequency -
                        card_get_frequency_step ());
                break;
        case 2: gtk_adjustment_set_value(GTK_ADJUSTMENT (info->fbar->
                        data.sfbar->scale), info->fbar->frequency +
                        card_get_frequency_step ());
                break;
        case 3: callback_scan(card_get_frequency_step ());
                break;
    }
}

void hsfreq_bar_new(struct freq_bar *fbar)
{
    struct freq_info *info;

    if (!fbar) return;

    fbar->data.sfbar = malloc (sizeof(struct scrolled_fbar));

    fbar->frequency = card_get_frequency ();
    fbar->data.sfbar->scale = gtk_adjustment_new(fbar->frequency,
                card_get_min_frequency (), card_get_max_frequency () +1,
		card_get_frequency_step (),1,1);

    fbar->base = gtk_hbox_new(FALSE, 0);
    fbar->set_freq = sfbar_set_freq;
    fbar->update = sfbar_update;
    fbar->data.sfbar->ignore = FALSE;

    fbar->data.sfbar->scrollbar = gtk_hscrollbar_new(GTK_ADJUSTMENT
                                        (fbar->data.sfbar->scale));

    gtk_range_set_update_policy (GTK_RANGE (fbar->data.sfbar->scrollbar),
                                   GTK_UPDATE_CONTINUOUS);
    if (!config->pix.scan_left_xpm)
        fbar->data.sfbar->scanleft = gtk_button_new_with_label("<<");
    else
	{
	    fbar->data.sfbar->scanleft = gtk_button_new();
	    fbar->data.sfbar->scanleft_xpm =
			gtk_pixmap_new( config->pix.scan_left_xpm,
					config->pix.scan_left_mask);
	    gtk_widget_show(fbar->data.sfbar->scanleft_xpm);
	    gtk_container_add(GTK_CONTAINER(fbar->data.sfbar->scanleft),
				fbar->data.sfbar->scanleft_xpm);
	}

    if (!config->pix.scan_right_xpm)
        fbar->data.sfbar->scanright = gtk_button_new_with_label(">>");
    else
        {
            fbar->data.sfbar->scanright = gtk_button_new();
            fbar->data.sfbar->scanright_xpm =
                    gtk_pixmap_new( config->pix.scan_right_xpm,
                                    config->pix.scan_right_mask);
            gtk_widget_show(fbar->data.sfbar->scanright_xpm);
            gtk_container_add(GTK_CONTAINER(fbar->data.sfbar->scanright),
                                fbar->data.sfbar->scanright_xpm);
        }

    gtk_signal_connect(GTK_OBJECT(fbar->data.sfbar->scale), "value_changed",
                        (GtkSignalFunc)sfbar_freq_callback , fbar);

    info = malloc (sizeof(struct freq_info));
    info->action = 0;
    info->fbar = fbar;
    gtk_signal_connect(GTK_OBJECT(fbar->data.sfbar->scanleft), "clicked",
                        (GtkSignalFunc)sfbar_button_callback , info);

    info = malloc (sizeof(struct freq_info));
    info->action = 3;
    info->fbar = fbar;
    gtk_signal_connect(GTK_OBJECT(fbar->data.sfbar->scanright), "clicked",
                        (GtkSignalFunc)sfbar_button_callback , info);

    gtk_box_pack_start(GTK_BOX(fbar->base), fbar->data.sfbar->scanleft, 0,0,0);
    gtk_container_add(GTK_CONTAINER(fbar->base),fbar->data.sfbar->scrollbar);
    gtk_box_pack_end(GTK_BOX(fbar->base), fbar->data.sfbar->scanright, 0,0,0);

    gtk_widget_show_all(fbar->base);
}
