/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* gui.h */
#ifndef GUI_H
#define GUI_H

#include "../channel.h"
#include "../conf.h"
#include <glib.h>
#include <gtk/gtk.h>

struct display;
extern GSList *display_list;
extern GSList *freq_bar_list;
extern GSList *volume_list;
extern GSList *channel_list;
extern GSList *mute_list;
extern GSList *stereo_list;

struct display_lcd {
    int state;
    int sub_state;
    int scroll_pos;
    gint timeout;

    GtkWidget *main;
    GtkWidget *lcd;
    GtkWidget *vbox;
    GtkWidget *event_box;

    struct display *parent;
    char *fft_data;
};

struct display_operations {
    void (*freq) (struct display *, double);
    void (*name) (struct display *, char*);
    void (*chan) (struct display *, int);
    void (*volume) (struct display *, int);
    void (*mute) (struct display *, gboolean);
    void (*stereo) (struct display *, gboolean);
    void (*tuned) (struct display *, gboolean);
    void (*alarm) (struct display *, gboolean, int, int);
    void (*timer) (struct display *, gboolean, int);
};

struct display {
    GtkWidget *base;
    char *chan_name;
    int chan_number;
    int volume;
    double frequency;
    gboolean muted;
    gboolean stereo;
    gboolean tuned;
    gboolean alarm_on;
    int alarm_hours;
    int alarm_min;
    gboolean timer_on;
    int timer_time;
    union {
        struct display_lcd *lcd;
    } data;
    struct display_operations *ops;
};

struct h_chan_box {
    GtkWidget *left_arrow;
    GtkWidget *left_arrow_pix;
    GtkWidget *right_arrow;
    GtkWidget *right_arrow_pix;
    GtkWidget *button_table;
    GPtrArray *button_array;
    int but_width;
    int but_height;
    int current_offset;
    gboolean show_numbers;
    gboolean show_text;
    int cols;
    int rows;
};

struct v_chan_box {
    GtkWidget *top_arrow;
    GtkWidget *top_arrow_pix;
    GtkWidget *bottom_arrow;
    GtkWidget *bottom_arrow_pix;
    GtkWidget *button_table;
    GPtrArray *button_array;
    int but_width;
    int but_height;
    int current_offset;
    gboolean show_numbers;
    gboolean show_text;
    int cols;
    int rows;
};

struct channel_button {
    GtkWidget *base;
    GtkWidget *label;
    GtkWidget *hbox;
    GtkWidget *vbox;
    GtkWidget *number;
    gboolean show_number;
    gboolean show_text;
    int signal;
};

struct channel_button_data {
    int but_width;
    int but_height;
    int channel_num;
    gboolean show_numbers;
    gboolean show_text;
    struct channel_button *button;
};

struct channel_object {
    GtkWidget *base;
    union {
	struct channel_button_data *channel;
	struct h_chan_box *h_box;
	struct v_chan_box *v_box;
    } data;
    void (*refresh) (struct channel_object*, struct channel *);
};

struct volume_bar_v {
    GtkObject *scale;
};

struct volume_bar_h {
    GtkObject *scale;
};

struct volume_plus_minus {
    GtkWidget *plus;
    GtkWidget *minus;
    GtkWidget *plus_xpm;
    GtkWidget *minus_xpm;
    gboolean horizontal;
};

struct volume_dial {
    GtkObject *scale;
};

struct volume_changer {
    int vol;
    gboolean ignore;
    GtkWidget *base;
    union {
	struct volume_bar_v *vvb;
	struct volume_bar_h *vhb;
	struct volume_plus_minus *vpm;
	struct volume_dial *dial;
    } data;
    void (*set_vol) (struct volume_changer*, int);
};

struct horiz_fbar {
    GtkObject *scale;
    GtkWidget *slider;
    GtkWidget *left;
    GtkWidget *right;
    GtkWidget *scanleft;
    GtkWidget *scanright;
    GtkWidget *left_xpm;
    GtkWidget *right_xpm;
    GtkWidget *scanleft_xpm;
    GtkWidget *scanright_xpm;
    GtkWidget *vcontainer;
    GtkWidget *cont2;
    GtkWidget *ruler;
    gboolean ignore;
};

struct scrolled_fbar {
    GtkObject *scale;
    GtkWidget *scrollbar;
    GtkWidget *scanleft;
    GtkWidget *scanright;
    GtkWidget *scanleft_xpm;
    GtkWidget *scanright_xpm;
    gboolean ignore;
};

struct freq_bar {
    GtkWidget *base;
    double frequency;
    union {
	struct horiz_fbar *hfbar;
	struct scrolled_fbar *sfbar;
    } data;
    void (*set_freq) (struct freq_bar*, double);
    void (*update) (struct freq_bar*);
};

struct stereo_button {
    GtkWidget *base;
    void (*set_state) (struct stereo_button *, gboolean);
};

struct mute_button {
    GtkWidget *base;
    GtkWidget *but;
    void (*set_state) (struct mute_button*, gboolean);
};

#define DISP_IF(x,y) if (((struct display*)x->data)->ops->y)
#define DISP(x,y) ((struct display*)x->data)->ops->y((struct display*)x->data

#define DISP_SET_FREQ(x,y) DISP_IF(x,freq) DISP(x,freq), y);
#define DISP_SET_NAME(x,y) DISP_IF(x,name) DISP(x,name), y);
#define DISP_SET_CHAN(x,y) DISP_IF(x,chan) DISP(x,chan), y);
#define DISP_SET_VOLUME(x,y) DISP_IF(x,volume) DISP(x,volume), y);
#define DISP_SET_MUTE(x,y) DISP_IF(x,mute) DISP(x,mute), y);
#define DISP_SET_STEREO(x,y) DISP_IF(x,stereo) DISP(x,stereo), y);
#define DISP_SET_TUNED(x,y) DISP_IF(x,tuned) DISP(x,tuned), y);
#define DISP_SET_ALARM(x,y,h,m) DISP_IF(x,alarm) DISP(x,alarm),y,h,m);
#define DISP_SET_TIMER(x,y,t) DISP_IF(x,timer) DISP(x,timer),y,t);

#define FREQ_BAR_SET(x,y) if (((struct freq_bar*)x->data)->set_freq) \
	((struct freq_bar*)x->data)->set_freq((struct freq_bar*)x->data,y);
#define FREQ_BAR_UPDATE(x) if (((struct freq_bar*)x->data)->update) \
	((struct freq_bar*)x->data)->update((struct freq_bar*)x->data);
#define VOL_BAR_SET(x,y) if (((struct volume_changer*)x->data)->set_vol) \
	((struct volume_changer*)x->data)-> \
	set_vol((struct volume_changer*)x->data,y);
#define MUTE_BUT_SET(x,y) if (((struct mute_button*)x->data)->set_state) \
        ((struct mute_button*)x->data)->set_state((struct mute_button*)x->data,y);
#define STEREO_BUT_SET(x,y) if (((struct stereo_button*)x->data)->set_state) \
        ((struct stereo_button*)x->data)->set_state((struct stereo_button*)x->data,y);
#define CH_BUTS_REFRESH(x,y) if (((struct channel_object*)x->data)->refresh) \
	((struct channel_object*)x->data)->refresh((struct channel_object*)x->data,y);

void hchannel_box_new(struct channel_object *obj);
void vchannel_box_new(struct channel_object *obj);
struct channel_button *new_channel_button(int w, int h, gboolean chan_num,
			int channel_num, gboolean chan_text, char *text);
void channel_button_new (struct channel_object *obj,
			 struct channel *first_chan);
void disp_lcd_new(struct display*);
void disp_refresh(struct display *);
void volume_vbar_new(struct volume_changer *vc);
void volume_hbar_new(struct volume_changer *vc);
void volume_plus_minus_new(struct volume_changer *vc, gboolean horizontal);
void volume_dial_new(struct volume_changer *vc);
void hfreq_bar_new(struct freq_bar *fbar, int mode);
void hsfreq_bar_new(struct freq_bar *fbar);
GtkWidget *quit_button_new();
struct mute_button *mute_button_new();
struct mute_button *mute_checkbutton_new();
struct stereo_button *stereo_button_new();
struct stereo_button *stereo_checkbutton_new();

void setup_main_window();
void setup_defaults();

#endif /* GUI_H */
