/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "gui.h"
#include "../callbacks.h"
#include "../hardware.h"
#include "popup_menu.h"
#include "layout.h"
#include <signal.h>
#include <malloc.h>
#include <stdio.h>
#ifdef USE_GNOME
#include <gnome.h>
#ifdef USE_PANEL
#include "applet.h"
#endif
#endif

GtkWidget *main_window = 0;
extern struct radio_config *config;
GSList *display_list=0;
GSList *freq_bar_list=0;
GSList *volume_list=0;
GSList *channel_list=0;
GSList *mute_list=0;
GSList *stereo_list=0;

struct channel_button *new_channel_button(int w, int h, gboolean chan_num,
                                int channel_num, gboolean chan_text, char *text)
{
    struct channel_button *but;
    GtkStyle *style;

    char str[64];

    but = malloc(sizeof(struct channel_button));
    but->show_number = chan_num;
    but->show_text = chan_text;

    but->base = gtk_button_new();
    but->hbox = gtk_hbox_new(FALSE, 2);
    but->vbox = gtk_vbox_new(FALSE, 2);

    sprintf(str,"%i",channel_num);

    but->number = gtk_label_new(str);
    style = gtk_style_copy(gtk_widget_get_style(but->number));
    style->font = gdk_font_load("-misc-fixed-medium-r-*-*-*-50-*-*-*-*-*-*");
    gtk_widget_set_style(but->number, style);

    if (chan_num) gtk_widget_show(but->number);
    gtk_box_pack_start(GTK_BOX(but->hbox), but->vbox, 0,0,0);
    gtk_box_pack_start(GTK_BOX(but->vbox), but->number, 0,0,0);

    but->label = gtk_label_new(text);
    style = gtk_style_copy(gtk_widget_get_style(but->label));
    style->font = gdk_font_load("-adobe-helvetica-medium-r-*-*-*-80-*-*-*-*-*-*");
    gtk_widget_set_style(but->label, style);

    if (chan_text) gtk_widget_show(but->label);
    gtk_container_add(GTK_CONTAINER(but->hbox), but->label);

    gtk_widget_show(but->hbox);
    gtk_widget_show(but->vbox);

    gtk_container_add(GTK_CONTAINER(but->base), but->hbox);

    gtk_widget_set_usize(but->base, w, h);
    gtk_widget_show(but->base);

    but->signal = 0;
    return but;
}

void disp_refresh(struct display *disp)
{
    disp->chan_name = NULL;
    disp->chan_number = -1;
    disp->volume = 0;
    if (config->set_volume) disp->volume = config->def_volume;
    disp->frequency = 0;
    if (config->set_channel)
    {
	struct channel* chan =
		get_channel(config->first_channel, config->def_channel);
	disp->frequency = chan->frequency;
	disp->chan_name = chan->name;
    }
    disp->muted = FALSE;
    disp->stereo = FALSE;
    disp->tuned = FALSE;
    disp->alarm_on = config->alarm.active;
    disp->timer_on = config->sleep.active;
    disp->alarm_hours = config->alarm.hour[0];
    disp->alarm_min = config->alarm.minute[0];
    disp->timer_time = config->sleep.sleep_time;
}

void test_hbox()
{
    GtkWidget *main_cont;
    GtkWidget *container;
    GtkWidget *container2;
    GtkWidget *container3;
    GtkWidget *quit;
    struct channel_object *hcbox;
    struct channel_object *vcbox;
    struct display *lcd_disp;
    struct volume_changer *vol;
    struct freq_bar *hfbar;
    struct freq_bar *rfbar;
    struct mute_button *mute;
    struct stereo_button *stereo;

    main_cont = gtk_vbox_new(FALSE, 10);

    hcbox = malloc(sizeof(struct channel_object));
    hcbox->base = NULL;
    hcbox->refresh = NULL;
    hcbox->data.h_box = malloc(sizeof(struct h_chan_box));
    hcbox->data.h_box->but_width = 50;
    hcbox->data.h_box->but_height = 20;
    hcbox->data.h_box->current_offset = 0;
    hcbox->data.h_box->show_numbers = TRUE;
    hcbox->data.h_box->show_text = TRUE;
    hcbox->data.h_box->cols = 2;
    hcbox->data.h_box->rows = 5;

    hchannel_box_new(hcbox);
    g_slist_append(channel_list, hcbox);
    if (hcbox->refresh)
	(hcbox->refresh)(hcbox, config->first_channel);

    gtk_widget_show(hcbox->base);

    vcbox = malloc(sizeof(struct channel_object));
    vcbox->base = NULL;
    vcbox->refresh = NULL;
    vcbox->data.v_box = malloc(sizeof(struct v_chan_box));
    vcbox->data.v_box->but_width = 50;
    vcbox->data.v_box->but_height = 20;
    vcbox->data.v_box->current_offset = 0;
    vcbox->data.v_box->show_numbers = TRUE;
    vcbox->data.v_box->show_text = TRUE;
    vcbox->data.v_box->cols = 2;
    vcbox->data.v_box->rows = 2;

    vchannel_box_new(vcbox);
    g_slist_append(channel_list, vcbox);
    if (hcbox->refresh)
	(hcbox->refresh)(vcbox, config->first_channel);

    gtk_widget_show(vcbox->base);

    lcd_disp = malloc(sizeof(struct display));
    disp_lcd_new(lcd_disp);
    disp_refresh(lcd_disp);
    g_slist_append(display_list, lcd_disp);

    gtk_widget_show(lcd_disp->base);

    container = gtk_hbox_new(FALSE, 10);
    gtk_container_border_width(GTK_CONTAINER(container), 5);

    gtk_widget_show(container);

    vol = malloc (sizeof(struct volume_changer));
    vol->vol = 0;
    volume_vbar_new(vol);
    g_slist_append(volume_list, vol);
    gtk_container_add(GTK_CONTAINER(container), vol->base);

    vol = malloc (sizeof(struct volume_changer));
    vol->vol = 0;
    volume_hbar_new(vol);
    g_slist_append(volume_list, vol);
    gtk_container_add(GTK_CONTAINER(container), vol->base);

    vol = malloc (sizeof(struct volume_changer));
    vol->vol = 0;
    volume_plus_minus_new(vol, FALSE);
    g_slist_append(volume_list, vol);
    gtk_container_add(GTK_CONTAINER(container), vol->base);

    vol = malloc (sizeof(struct volume_changer));
    vol->vol = 0;
    volume_dial_new(vol);
    g_slist_append(volume_list, vol);
    gtk_container_add(GTK_CONTAINER(container), vol->base);

    gtk_container_add(GTK_CONTAINER(main_cont), container);
    gtk_container_add(GTK_CONTAINER(container), vcbox->base);
    gtk_container_add(GTK_CONTAINER(container), hcbox->base);
    gtk_container_add(GTK_CONTAINER(container), lcd_disp->base);

    container3 = gtk_vbox_new(FALSE, 5);
    gtk_container_border_width(GTK_CONTAINER(container3), 5);
    gtk_widget_show(container3);
    gtk_container_add(GTK_CONTAINER(container),  container3);

    mute = mute_button_new();
    g_slist_append(mute_list, mute);
    gtk_container_add(GTK_CONTAINER(container3), mute->base);
    mute = mute_checkbutton_new();
    g_slist_append(mute_list, mute);
    gtk_container_add(GTK_CONTAINER(container3), mute->base);

    stereo = stereo_button_new();
    g_slist_append(stereo_list, stereo);
    gtk_container_add(GTK_CONTAINER(container3), stereo->base);
    stereo = stereo_checkbutton_new();
    g_slist_append(stereo_list, stereo);
    gtk_container_add(GTK_CONTAINER(container3), stereo->base);

    quit = quit_button_new();
    gtk_container_add(GTK_CONTAINER(container3), quit);

    container2 = gtk_vbox_new(FALSE, 10);
    gtk_container_add(GTK_CONTAINER(main_cont), container2);
    gtk_container_border_width(GTK_CONTAINER(container2), 5);
    gtk_widget_show(container2);

    hfbar = malloc(sizeof(struct freq_bar));
    hfreq_bar_new(hfbar, 0);
    g_slist_append(freq_bar_list, hfbar);
    gtk_container_add(GTK_CONTAINER(container2), hfbar->base);

    rfbar = malloc(sizeof(struct freq_bar));
    hfreq_bar_new(rfbar, 1);
    g_slist_append(freq_bar_list, rfbar);
    gtk_container_add(GTK_CONTAINER(container2), rfbar->base);

    rfbar = malloc(sizeof(struct freq_bar));
    hfreq_bar_new(rfbar, 2);
    g_slist_append(freq_bar_list, rfbar);
    gtk_container_add(GTK_CONTAINER(container2), rfbar->base);

    rfbar = malloc(sizeof(struct freq_bar));
    hsfreq_bar_new(rfbar);
    g_slist_append(freq_bar_list, rfbar);
    gtk_container_add(GTK_CONTAINER(container2), rfbar->base);

#ifdef USE_GNOME
    gnome_app_set_contents(GNOME_APP(main_window), main_cont);
#else
    gtk_container_add(GTK_CONTAINER(main_window), main_cont);
#endif

    gtk_widget_show(main_cont);
}

void setup_main_window()
{
    gint error = FALSE;

    signal(SIGINT, (void*)gtk_main_quit);

    display_list = g_slist_alloc();
    freq_bar_list = g_slist_alloc();
    volume_list = g_slist_alloc();
    channel_list = g_slist_alloc();
    mute_list = g_slist_alloc();
    stereo_list = g_slist_alloc();

#ifdef USE_PANEL
    g_slist_append(display_list, applet_display);
    disp_refresh(applet_display);
#endif

    if (config->layout_file)
    {
	layout_init();
	if (layout_parse(config->layout_file) == FALSE)
	    error = TRUE;
	if (layout_generate() == FALSE)
	    error = TRUE;
    }
    else
	error = TRUE;

    if (error)
    {
	printf("gtuner: an error occured in the layout generator\n");
	printf("	some functionality may be affected\n");
	printf("        starting with test layout...\n");

#ifdef USE_GNOME
	main_window = gnome_app_new("gtuner", "gtuner");
#else
	main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
#endif
	gtk_accel_group_attach (accel_group, GTK_OBJECT (main_window));
	gtk_signal_connect(GTK_OBJECT(main_window), "delete_event",
				    (GtkSignalFunc) gtk_main_quit, NULL);

	gtk_window_set_title(GTK_WINDOW(main_window), "gtuner");
	gtk_widget_realize(main_window);

	gdk_window_set_icon_name(main_window->window, "gtuner");
	if (config->pix.icon_xpm)
	    gdk_window_set_icon(main_window->window, NULL,
			    config->pix.icon_xpm, config->pix.icon_mask);

	test_hbox();
	gtk_widget_show(main_window);
    }
#ifdef USE_PANEL
    if (config->start_hidden)
	layout_toggle_windows(FALSE);
#endif
}

void setup_defaults()
{
    struct volume_changer vc;
    if (config->set_channel)
    {
	callback_set_channel(NULL,
		get_channel(config->first_channel, config->def_channel));
    } else
    callback_set_freq (NULL, card_get_frequency());

    if (config->set_volume)
    {
	vc.vol = config->def_volume;
	callback_set_volume(NULL, &vc);
    }

#ifdef USE_PANEL
    applet_set_style(config->panel_applet_style);
#endif
}
