/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "layout.h"
#include "layout_private.h"
#include "gui.h"
#include "popup_menu.h"
#include <fcntl.h>
#include <sys/stat.h>
#include <malloc.h>
#include <stdio.h>
#include <unistd.h>
#ifdef USE_GNOME
#include <gnome.h>
#endif

void		layout_parse_file	(gint fd,const gchar *filename);
static guint	layout_parse_window	(GScanner *scanner);
static guint	layout_parse_statement	(GScanner *scanner);
static guint	layout_hash		(const char *name);
static gint	layout_compare		(const char *a, const char *b);
static gint	layout_compare_child_info
					(child_info *a, char *b);
windowLayout*	layout_rc_window_new	();
static guint	layout_parse_window_child
					(GScanner *scanner,
					 windowLayout *rc_window);
static guint	layout_set_window_close (GScanner *scanner,
					 windowLayout *rc_window);
static guint	layout_parse_widget	(GScanner *scanner);
widgetLayout*	layout_rc_widget_new	(guint type);
guint		layout_type_parse	(GScanner *scanner,
					 widgetLayout *widget, guint type);
void		layout_generate_tree	();
void		layout_hash_func	(gpointer key, gpointer value,
					 gpointer user_data);
void		layout_generate_container_tree
					(GNode *widget_node);
GtkWidget*	layout_generate_object	(GNode *node);
GtkWidget*	layout_generate_object_freq_bar
					(GNode *node);
GtkWidget*	layout_generate_object_volume
					(GNode *node);
GtkWidget*	layout_generate_object_display
					(GNode *node);
GtkWidget*	layout_generate_object_quit
					(GNode *node);
GtkWidget*	layout_generate_object_mute
					(GNode *node);
GtkWidget*	layout_generate_object_stereo
					(GNode *node);
GtkWidget*	layout_generate_object_channel
					(GNode *node);
GtkWidget*	layout_generate_object_chanbox
					(GNode *node);
guint		layout_parse_container	(GScanner *scanner,
					 widgetLayout *widget, gboolean insert,
					 guint type);
guint		layout_parse_container_add
					(GScanner *scanner,
					 widgetLayout *widget, guint type);
guint		layout_parse_container_pack
					(GScanner *scanner,
					 widgetLayout *widget, guint type);
guint		layout_parse_container_border
					(GScanner *scanner,
					 widgetLayout *widget, guint type);
guint		layout_info_station	(GScanner *scanner,
					 widgetLayout *widget, guint type);
guint		layout_info_show_numbers(GScanner *scanner,
					 widgetLayout *widget, guint type);
guint		layout_info_show_text	(GScanner *scanner,
					 widgetLayout *widget, guint type);
guint		layout_info_size	(GScanner *scanner,
					 widgetLayout *widget, guint type);
guint		layout_info_cols	(GScanner *scanner,
					 widgetLayout *widget, guint type);
guint		layout_info_rows	(GScanner *scanner,
					 widgetLayout *widget, guint type);
void		layout_func_container	(gpointer value, gpointer user_data);
GtkWidget*	layout_generate_container
					(GNode *node);
GtkWidget*	layout_generate_h_container
					(widgetLayout *rc_widget);
GtkWidget*	layout_generate_v_container
					(widgetLayout *rc_widget);
GtkWidget*	layout_generate_handle_box
					(widgetLayout *rc_widget);

static const GScannerConfig     layout_scanner_config =
{
    (
       " \t\n"
     )                    /* cset_skip_characters */,
    (
       G_CSET_a_2_z
       "_"
       G_CSET_A_2_Z
     )                    /* cset_identifier_first */,
    (
       G_CSET_a_2_z
       "_-0123456789"
       G_CSET_A_2_Z
     )                    /* cset_identifier_nth */,
    ( "#\n" )             /* cpair_comment_single */,

    TRUE                  /* case_sensitive */,

    TRUE                  /* skip_comment_multi */,
    TRUE                  /* skip_comment_single */,
    TRUE                  /* scan_comment_multi */,
    TRUE                  /* scan_identifier */,
    FALSE                 /* scan_identifier_1char */,
    FALSE                 /* scan_identifier_NULL */,
    TRUE                  /* scan_symbols */,
    TRUE                  /* scan_binary */,
    TRUE                  /* scan_octal */,
    TRUE                  /* scan_float */,
    TRUE                  /* scan_hex */,
    TRUE                  /* scan_hex_dollar */,
    TRUE                  /* scan_string_sq */,
    TRUE                  /* scan_string_dq */,
    TRUE                  /* numbers_2_int */,
    FALSE                 /* int_2_float */,
    FALSE                 /* identifier_2_string */,
    TRUE                  /* char_2_token */,
    TRUE                  /* symbol_2_token */,
    FALSE                 /* scope_0_fallback */,
};

static const struct
{
  gchar *name;
  guint token;
} symbols[] = {
  { "window", 	TOKEN_WINDOW },
  { "child", 	TOKEN_WINDOW_CHILD },
  { "close",	TOKEN_WINDOW_CLOSE },
  { "hide",	TOKEN_WINDOW_CLOSE_HIDE },
  { "destroy",	TOKEN_WINDOW_CLOSE_DESTROY },
  { "table", 	TOKEN_TABLE },				//
  { "h_container", 	TOKEN_HCONTAINER },
  { "v_container", 	TOKEN_VCONTAINER },
  { "pack_start", 	TOKEN_PACK_START },
  { "pack_end", 	TOKEN_PACK_END },
  { "add", 	TOKEN_ADD },
  { "border", 	TOKEN_BORDER },
  { "handle_box", 	TOKEN_HANDLE_BOX },
  { "toolbar", 	TOKEN_TOOLBAR },			//
  { "add_space", 	TOKEN_TOOLBAR_ADD_SPACE },	//
  { "insert_item", 	TOKEN_TOOLBAR_INSERT },		//
  { "space_width", 	TOKEN_TOOLBAR_SPACE_WIDTH },	//
  { "space_style", 	TOKEN_TOOLBAR_SPACE_STYLE },	//
  { "show_tooltips", 	TOKEN_TOOLBAR_SHOW_TOOLTIPS },	//
  { "toolbar_style", 	TOKEN_TOOLBAR_STYLE },		//

  { "type", 	TOKEN_TYPE },
  { "hscale", 	TOKEN_TYPE_VOLUME_HSCALE },
  { "vscale", 	TOKEN_TYPE_VOLUME_VSCALE },
  { "horiz_pm", 	TOKEN_TYPE_VOLUME_HORIZ_PM },
  { "vert_pm",	TOKEN_TYPE_VOLUME_VERT_PM },
  { "dial", 	TOKEN_TYPE_VOLUME_DIAL },
  { "lcd", 	TOKEN_TYPE_DISPLAY_LCD },
  { "plain", 	TOKEN_TYPE_FREQ_PLAIN },
  { "ruler", 	TOKEN_TYPE_FREQ_RULER },
  { "disp", 	TOKEN_TYPE_FREQ_DISP },
  { "scroll", 	TOKEN_TYPE_FREQ_SCROLL },
  { "button", 	TOKEN_TYPE_BUTTON },
  { "toggle", 	TOKEN_TYPE_TOGGLE },
  { "checkbox", 	TOKEN_TYPE_CHECK },
  { "verticle", 	TOKEN_TYPE_CHANNELBOX_VERT },
  { "horizontal", 	TOKEN_TYPE_CHANNELBOX_HORIZ },

  { "station",	TOKEN_INFO_CHANNEL_NUMBER },
  { "show_numbers",	TOKEN_INFO_SHOW_NUMBERS },
  { "show_text",	TOKEN_INFO_SHOW_TEXT },	
  { "size",	TOKEN_INFO_SIZE },			//
  { "cols",	TOKEN_INFO_COLS },
  { "rows",	TOKEN_INFO_ROWS },

  { "volume", 	TOKEN_OBJECT_VOLUME },
  { "display", 	TOKEN_OBJECT_DISPLAY },
  { "freq_bar", 	TOKEN_OBJECT_FREQ_BAR },
  { "quit", 	TOKEN_OBJECT_QUIT },
  { "mute", 	TOKEN_OBJECT_MUTE },
  { "stereo", 	TOKEN_OBJECT_STEREO },
  { "channel", 	TOKEN_OBJECT_CHANNEL },	
  { "channel_box", 	TOKEN_OBJECT_CHANNEL_BOX },

  { "true", TOKEN_TRUE },
  { "false", TOKEN_FALSE },
};

static GHashTable *rc_window_ht = NULL;
static GHashTable *rc_widget_ht = NULL;

static const guint n_symbols = sizeof (symbols) / sizeof (symbols[0]);
extern struct radio_config *config;

static guint
layout_hash (const char *name)
{
  guint result;

  result = 0;
  while (*name)
    result += (result << 3) + *name++;

  return result;
}

static gint
layout_compare (const char *a, const char *b)
{
    return (strcmp (a, b) == 0);
}

static gint
layout_compare_child_info (child_info *a, char *b)
{
    if (!a)
	return TRUE;
    return (strcmp (a->name, b) != 0);
}

void layout_init (void)
{
    rc_window_ht = g_hash_table_new ((GHashFunc) layout_hash,
                                  (GCompareFunc) layout_compare);
    rc_widget_ht = g_hash_table_new ((GHashFunc) layout_hash,
                                  (GCompareFunc) layout_compare);

    layoutRoot = g_node_new(NULL);
}

int layout_parse(const gchar *filename)
{
    gint fd;

    if (!filename)
    {
	printf("gtuner: Cannot open layout definition\n");
	return 0;
    }
    g_return_val_if_fail (layoutRoot != NULL, 0);
    g_return_val_if_fail (rc_window_ht != NULL, 0);
    g_return_val_if_fail (rc_widget_ht != NULL, 0);

    fd = open (filename, O_RDONLY);
    if (fd < 0)
    {
	printf("gtuner: Error opening layout file: %s\n", filename);
	return 0;
    }

    layout_parse_file(fd, filename);
    close(fd);

    return TRUE;
}

void layout_parse_file(gint fd, const gchar *filename)
{
    GScanner *scanner;
    guint    i;
    gboolean done;

    scanner = g_scanner_new ((const gpointer)&layout_scanner_config);
    g_scanner_input_file (scanner, fd);
    scanner->input_name = filename;

    g_scanner_freeze_symbol_table (scanner);
    for (i = 0; i < n_symbols; i++)
        g_scanner_add_symbol (scanner, symbols[i].name,
		GINT_TO_POINTER (symbols[i].token));
    g_scanner_thaw_symbol_table (scanner);

    done = FALSE;
    while (!done)
    {
	if (g_scanner_peek_next_token (scanner) == G_TOKEN_EOF)
	    done = TRUE;
	else
	{
	    guint expected_token;
	    expected_token = layout_parse_statement (scanner);

	    if (expected_token != G_TOKEN_NONE)
	    {
		gchar *symbol_name;
		gchar *msg;

		msg = NULL;
		symbol_name = NULL;
		if (scanner->scope_id == 0)
		{
		    /* if we are in scope 0, we know the symbol names
		     * that are associated with certaintoken values.
		     * so we look them up to make the error messages
		     * more readable.
		     */
		  if (expected_token > TOKEN_INVALID &&
		      expected_token < TOKEN_LAST)
		    {
		      for (i = 0; i < n_symbols; i++)
			if (symbols[i].token == expected_token)
			  msg = symbols[i].name;
		      if (msg)
			msg = g_strconcat ("e.g. `", msg, "'", NULL);
		    }
		  if (scanner->token > TOKEN_INVALID &&
		      scanner->token < TOKEN_LAST)
		    {
		      symbol_name = "???";
		      for (i = 0; i < n_symbols; i++)
			if (symbols[i].token == scanner->token)
			  symbol_name = symbols[i].name;
		    }

		}
		g_scanner_unexp_token (scanner,
				       expected_token,
				       NULL,
				       "keyword",
				       symbol_name,
				       msg,
				       TRUE);
		g_free (msg);
		done = TRUE;
	    }
	}
    }
    g_scanner_destroy (scanner);
}

static guint
layout_parse_statement (GScanner *scanner)
{
    guint token;

    token = g_scanner_peek_next_token (scanner);

    switch (token)
    {
    case TOKEN_WINDOW:
	    return layout_parse_window(scanner);
    case TOKEN_TABLE:
    case TOKEN_HCONTAINER:
    case TOKEN_VCONTAINER:
    case TOKEN_HANDLE_BOX:
    case TOKEN_TOOLBAR:
    case TOKEN_OBJECT_VOLUME:
    case TOKEN_OBJECT_DISPLAY:
    case TOKEN_OBJECT_FREQ_BAR:
    case TOKEN_OBJECT_QUIT:
    case TOKEN_OBJECT_MUTE:
    case TOKEN_OBJECT_STEREO:
    case TOKEN_OBJECT_CHANNEL:
    case TOKEN_OBJECT_CHANNEL_BOX:
	return layout_parse_widget(scanner);
    default:
	g_scanner_get_next_token (scanner);
	return G_TOKEN_SYMBOL;
    }
}

static guint
layout_parse_window (GScanner *scanner)
{
    guint token;
    windowLayout *rc_window;
    gboolean insert = FALSE;

    token = g_scanner_get_next_token (scanner);
    if (token != TOKEN_WINDOW)
	return TOKEN_WINDOW;

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_STRING)
	return G_TOKEN_STRING;

    rc_window = g_hash_table_lookup (rc_window_ht, scanner->value.v_string);

    if (!rc_window)
    {
	insert = TRUE;
	rc_window = layout_rc_window_new();
	rc_window->name = g_strdup (scanner->value.v_string);
    }

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_LEFT_CURLY)
    {
	if (insert)
	    free (rc_window);

	return G_TOKEN_LEFT_CURLY;
    }

    token = g_scanner_peek_next_token (scanner);
    while (token != G_TOKEN_RIGHT_CURLY)
    {
	switch (token)
	{
	case TOKEN_WINDOW_CHILD:
	    token = layout_parse_window_child(scanner, rc_window);
	    break;
	case TOKEN_WINDOW_CLOSE:
	    token = layout_set_window_close(scanner, rc_window);
	    break;
	default:
	    g_scanner_get_next_token (scanner);
	    token = G_TOKEN_RIGHT_CURLY;
	    break;
	}

	if (token != G_TOKEN_NONE)
	{
	    if (insert)
	    {
		if (rc_window->name)
		    g_free(rc_window->name);
		if (rc_window->child)
		    g_free(rc_window->child);
		g_free(rc_window);
	    }
	    return token;
	}
	token = g_scanner_peek_next_token (scanner);
    }

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_RIGHT_CURLY)
    {
	if (insert)
	{
	    if (rc_window->name)
		g_free(rc_window->name);
	    if (rc_window->child)
		g_free(rc_window->child);
	    g_free(rc_window);
	}
	return G_TOKEN_RIGHT_CURLY;
    }

    if (insert)
    {
	g_hash_table_insert (rc_window_ht, rc_window->name, rc_window);
    }

    return G_TOKEN_NONE;
}

windowLayout *layout_rc_window_new()
{
    windowLayout *new_window;

    new_window = malloc (sizeof(struct _windowLayout));
    new_window->name = NULL;
    new_window->child = NULL;
    new_window->close = TOKEN_WINDOW_CLOSE_HIDE;

    return new_window;
}

static guint
layout_parse_window_child (GScanner *scanner, windowLayout *rc_window)
{
    guint token;

    token = g_scanner_get_next_token (scanner);
    if (token != TOKEN_WINDOW_CHILD)
	return TOKEN_WINDOW_CHILD;

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_EQUAL_SIGN)
	return G_TOKEN_EQUAL_SIGN;

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_STRING)
	return G_TOKEN_STRING;

    if (rc_window->child)
	g_free(rc_window->child);
    rc_window->child = g_strdup (scanner->value.v_string);

    return G_TOKEN_NONE;
}

static guint
layout_set_window_close (GScanner *scanner, windowLayout *rc_window)
{
    guint token;

    token = g_scanner_get_next_token (scanner);
    if (token != TOKEN_WINDOW_CLOSE)
        return TOKEN_WINDOW_CLOSE;

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_EQUAL_SIGN)
        return G_TOKEN_EQUAL_SIGN;

    token = g_scanner_get_next_token (scanner);
    switch (token)
    {
    case TOKEN_WINDOW_CLOSE_QUIT:
    case TOKEN_WINDOW_CLOSE_HIDE:
    case TOKEN_WINDOW_CLOSE_DESTROY:
	rc_window->close = token;
	break;
    default:
	return G_TOKEN_SYMBOL;
    }

    return G_TOKEN_NONE;
}

static guint
layout_parse_widget(GScanner *scanner)
{
    guint token;
    guint type;
    gboolean insert = FALSE;
    widgetLayout *widget;

    token = g_scanner_get_next_token (scanner);
    switch (token)
    {
    case TOKEN_TABLE:
    case TOKEN_HCONTAINER:
    case TOKEN_VCONTAINER:
    case TOKEN_HANDLE_BOX:
    case TOKEN_TOOLBAR:
    case TOKEN_OBJECT_VOLUME:
    case TOKEN_OBJECT_DISPLAY:
    case TOKEN_OBJECT_FREQ_BAR:
    case TOKEN_OBJECT_QUIT:
    case TOKEN_OBJECT_MUTE:
    case TOKEN_OBJECT_STEREO:
    case TOKEN_OBJECT_CHANNEL:
    case TOKEN_OBJECT_CHANNEL_BOX:
	type = token;
	break;
    default:
	return G_TOKEN_SYMBOL;
    }
    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_STRING)
	return G_TOKEN_STRING;

    widget = g_hash_table_lookup (rc_widget_ht, scanner->value.v_string);

    if (!widget)
    {
	insert = TRUE;
	widget = layout_rc_widget_new (type);
	widget->name = g_strdup (scanner->value.v_string);
    }

    token = g_scanner_peek_next_token (scanner);
    if (token != G_TOKEN_LEFT_CURLY)
    {
	if (insert)
	{
	    if (widget->name)
		free (widget->name);
	    if (widget->data.container)
		free (widget->data.container); /* will free any union */
	    free (widget);
	}
	return G_TOKEN_LEFT_CURLY;
    }

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_LEFT_CURLY)
    {
	if (insert)
	{
	    if (widget->name)
		free (widget->name);
	    if (widget->data.container)
		free (widget->data.container); /* will free any union */
	    free (widget);
	}

	return G_TOKEN_LEFT_CURLY;
    }

    switch (type)
    {
    case TOKEN_TABLE:
    case TOKEN_HCONTAINER:
    case TOKEN_VCONTAINER:
    case TOKEN_HANDLE_BOX:
    case TOKEN_TOOLBAR:
        return layout_parse_container(scanner, widget, insert, type);
    }

    token = g_scanner_peek_next_token (scanner);
    while (token != G_TOKEN_RIGHT_CURLY)
    {
	switch (token)
        {
	case TOKEN_TYPE:
	    token = layout_type_parse (scanner, widget, type);
	    break;
	case TOKEN_INFO_CHANNEL_NUMBER:
	    token = layout_info_station (scanner, widget, type);
	    break;
	case TOKEN_INFO_SHOW_NUMBERS:
	    token = layout_info_show_numbers (scanner, widget, type);
	    break;
	case TOKEN_INFO_SHOW_TEXT:
	    token = layout_info_show_text (scanner, widget, type);
	    break;
	case TOKEN_INFO_SIZE:
	    token = layout_info_size (scanner, widget, type);
	    break;
	case TOKEN_INFO_COLS:
	    token = layout_info_cols (scanner, widget, type);
	    break;
	case TOKEN_INFO_ROWS:
	    token = layout_info_rows (scanner, widget, type);
	    break;
	default:
	    g_scanner_get_next_token (scanner);
	    token = G_TOKEN_RIGHT_CURLY;
	    break;
	}

	if (token != G_TOKEN_NONE)
	{
	    if (insert)
	    {
		if (widget->name)
		    free (widget->name);
		if (widget->data.container)
		    free (widget->data.container); /* will free any union */
		free (widget);
	    }
	    return token;
	}
	token = g_scanner_peek_next_token (scanner);
    }

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_RIGHT_CURLY)
    {
	if (insert)
	{
	    if (widget->name)
		free (widget->name);
	    if (widget->data.container)
		free (widget->data.container); /* will free any union */
	    free (widget);
	}
	return G_TOKEN_RIGHT_CURLY;
    }


    if (insert)
    {
        g_hash_table_insert (rc_widget_ht, widget->name, widget);
    }

    return G_TOKEN_NONE;
}

widgetLayout *layout_rc_widget_new(guint type)
{
    widgetLayout *new_widget;

    new_widget = malloc (sizeof(struct _widgetLayout));
    new_widget->name = NULL;
    new_widget->type = type;

    switch (type)
    {
//    case TOKEN_TABLE:
    case TOKEN_HCONTAINER:
    case TOKEN_VCONTAINER:
	new_widget->data.container = malloc (sizeof(struct _containerLayout));
	new_widget->data.container->single_child = FALSE;
	new_widget->data.container->border_width = 0;
	new_widget->data.container->child_list = NULL;
	new_widget->data.container->data.box =
			malloc (sizeof(struct _box_container));
	new_widget->data.container->data.box->homogeneous = FALSE;
        new_widget->data.container->data.box->spacing = 5;
	break;
    case TOKEN_HANDLE_BOX:
	new_widget->data.container = malloc (sizeof(struct _containerLayout));
	new_widget->data.container->single_child = TRUE;
	new_widget->data.container->border_width = 0;
	new_widget->data.container->child_list = NULL;
	new_widget->data.container->data.handle =
			malloc (sizeof(struct _handle_container));
	new_widget->data.container->data.handle->type = GTK_SHADOW_OUT;
	break;
//    case TOKEN_TOOLBAR:
    case TOKEN_OBJECT_VOLUME:
	new_widget->data.volume = malloc (sizeof(struct _volumeLayout));
	new_widget->data.volume->sub_type = TOKEN_TYPE_VOLUME_VERT_PM;
	break;
    case TOKEN_OBJECT_DISPLAY:
	new_widget->data.display = malloc (sizeof(struct _displayLayout));
	new_widget->data.display->sub_type = TOKEN_TYPE_DISPLAY_LCD;
	break;
    case TOKEN_OBJECT_FREQ_BAR:
	new_widget->data.freq_bar = malloc (sizeof(struct _freq_barLayout));
	new_widget->data.freq_bar->sub_type = TOKEN_TYPE_FREQ_PLAIN;
	break;
    case TOKEN_OBJECT_QUIT:
	new_widget->data.quit = malloc (sizeof(struct _quitLayout));
	new_widget->data.quit->sub_type = TOKEN_TYPE_BUTTON;
	break;
    case TOKEN_OBJECT_MUTE:
        new_widget->data.mute = malloc (sizeof(struct _muteLayout));
        new_widget->data.mute->sub_type = TOKEN_TYPE_TOGGLE;
        break;
    case TOKEN_OBJECT_STEREO:
	new_widget->data.stereo = malloc (sizeof(struct _stereoLayout));
	new_widget->data.stereo->sub_type = TOKEN_TYPE_TOGGLE;
	break;
    case TOKEN_OBJECT_CHANNEL:
	new_widget->data.chan = malloc (sizeof(struct _channelLayout));
	new_widget->data.chan->sub_type = TOKEN_TYPE_BUTTON;
	new_widget->data.chan->station = 0;
	new_widget->data.chan->show_number = TRUE;
	new_widget->data.chan->show_text = TRUE;
	new_widget->data.chan->but_width = 50;
	new_widget->data.chan->but_height = 20;
	break;
    case TOKEN_OBJECT_CHANNEL_BOX:
	new_widget->data.chanbox = malloc (sizeof(struct _chanboxLayout));
	new_widget->data.chanbox->sub_type = TOKEN_TYPE_CHANNELBOX_HORIZ;
	new_widget->data.chanbox->but_width = 50;
	new_widget->data.chanbox->but_height = 20;
	new_widget->data.chanbox->show_numbers = TRUE;
	new_widget->data.chanbox->show_text = TRUE;
	new_widget->data.chanbox->cols = 2;
	new_widget->data.chanbox->rows = 3;
	break;
    }

    return new_widget;
}

guint layout_type_parse (GScanner *scanner, widgetLayout *widget, guint type)
{
    guint token;

    switch (type)
    {
    case TOKEN_OBJECT_VOLUME:
    case TOKEN_OBJECT_DISPLAY:
    case TOKEN_OBJECT_FREQ_BAR:
    case TOKEN_OBJECT_QUIT:
    case TOKEN_OBJECT_MUTE:
    case TOKEN_OBJECT_STEREO:
    case TOKEN_OBJECT_CHANNEL:
    case TOKEN_OBJECT_CHANNEL_BOX:
	break;
    default:
        return G_TOKEN_SYMBOL;
    }

    token = g_scanner_get_next_token (scanner);
    if (token != TOKEN_TYPE)
	return TOKEN_TYPE;

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_EQUAL_SIGN)
	return G_TOKEN_EQUAL_SIGN;

    token = g_scanner_get_next_token (scanner);
    switch (type)
    {
    case TOKEN_OBJECT_VOLUME:
	switch (token)
	{
	case TOKEN_TYPE_VOLUME_HSCALE:
	case TOKEN_TYPE_VOLUME_VSCALE:
	case TOKEN_TYPE_VOLUME_HORIZ_PM:
	case TOKEN_TYPE_VOLUME_VERT_PM:
	case TOKEN_TYPE_VOLUME_DIAL:
	    widget->data.volume->sub_type = token;
	    break;
	default:
	    return G_TOKEN_SYMBOL;
	}
	break;
    case TOKEN_OBJECT_DISPLAY:
	switch (token)
	{
	case TOKEN_TYPE_DISPLAY_LCD:
	    widget->data.display->sub_type = token;
	    break;
	default:
	    return G_TOKEN_SYMBOL;
	}
	break;
    case TOKEN_OBJECT_FREQ_BAR:
	switch (token)
	{
	case TOKEN_TYPE_FREQ_PLAIN:
	case TOKEN_TYPE_FREQ_RULER:
	case TOKEN_TYPE_FREQ_DISP:
	case TOKEN_TYPE_FREQ_SCROLL:
	    widget->data.freq_bar->sub_type = token;
	    break;
	default:
	    return G_TOKEN_SYMBOL;
	}
	break;
    case TOKEN_OBJECT_QUIT:
	switch (token)
	{
	case TOKEN_TYPE_BUTTON:
	    widget->data.quit->sub_type = token;
	    break;
	default:
	    return G_TOKEN_SYMBOL;
	}
	break;
    case TOKEN_OBJECT_MUTE:
	switch (token)
	{
	case TOKEN_TYPE_TOGGLE:
	case TOKEN_TYPE_CHECK:
	    widget->data.mute->sub_type = token;
	    break;
	default:
	    return G_TOKEN_SYMBOL;
	}
	break;
    case TOKEN_OBJECT_STEREO:
	switch (token)
	{
	case TOKEN_TYPE_TOGGLE:
	case TOKEN_TYPE_CHECK:
	    widget->data.stereo->sub_type = token;
	    break;
	default:
	    return G_TOKEN_SYMBOL;
	}
	break;
    case TOKEN_OBJECT_CHANNEL:
	switch (token)
	{
	case TOKEN_TYPE_BUTTON:
	    widget->data.chan->sub_type = token;
	    break;
	default:
	    return G_TOKEN_SYMBOL;
	}
	break;
    case TOKEN_OBJECT_CHANNEL_BOX:
	switch (token)
	{
	case TOKEN_TYPE_CHANNELBOX_VERT:
	case TOKEN_TYPE_CHANNELBOX_HORIZ:
	    widget->data.chanbox->sub_type = token;
	    break;
	default:
	    return G_TOKEN_SYMBOL;
	}
	break;
    default:
	return G_TOKEN_SYMBOL;
    }

    return G_TOKEN_NONE;
}

guint layout_parse_container(GScanner *scanner, widgetLayout *widget,
			     gboolean insert, guint type)
{
    guint token;

    token = g_scanner_peek_next_token (scanner);
    while (token != G_TOKEN_RIGHT_CURLY)
    {
	switch (token)
	{
	case TOKEN_ADD:
	    token = layout_parse_container_add (scanner, widget, type);
	    break;
	case TOKEN_PACK_START:
	case TOKEN_PACK_END:
	    token = layout_parse_container_pack (scanner, widget, type);
	    break;
	case TOKEN_BORDER:
	    token = layout_parse_container_border (scanner, widget, type);
	    break;
	default:
	    g_scanner_get_next_token (scanner);
	    token = G_TOKEN_RIGHT_CURLY;
	    break;
	}

	if (token != G_TOKEN_NONE)
	{
	    if (insert)
	    {
		if (widget->name)
		    free (widget->name);
		if (widget->data.container)
		    free (widget->data.container); /* will free any union */
		free (widget);
	    }
	    return token;
	}
	token = g_scanner_peek_next_token (scanner);
    }

  token = g_scanner_get_next_token (scanner);
  if (token != G_TOKEN_RIGHT_CURLY)
    {
      if (insert)
        {
	    if (widget->name)
		free (widget->name);
	    if (widget->data.container)
		free (widget->data.container); /* will free any union */
	    free (widget);
        }
      return G_TOKEN_RIGHT_CURLY;
    }

    if (insert)
	g_hash_table_insert (rc_widget_ht, widget->name, widget);

    return G_TOKEN_NONE;
}

guint layout_parse_container_add(GScanner *scanner, widgetLayout *widget,
				 guint type)
{
    guint token;
    child_info *child_info;
    GSList *list = NULL;

    switch (type)
    {
    case TOKEN_TABLE:
    case TOKEN_HCONTAINER:
    case TOKEN_VCONTAINER:
    case TOKEN_HANDLE_BOX:
    case TOKEN_TOOLBAR:
	break;
    default:
	return G_TOKEN_SYMBOL;
    }

    g_return_val_if_fail(widget != NULL, G_TOKEN_ERROR);
    g_return_val_if_fail(widget->data.container != NULL, G_TOKEN_ERROR);
    if (!widget->data.container->child_list)
	widget->data.container->child_list =
		g_slist_alloc();

    token = g_scanner_get_next_token (scanner);
    if (token != TOKEN_ADD)
	return TOKEN_ADD;

    child_info = malloc (sizeof(struct _child_info));
    memset (child_info, 0, sizeof(struct _child_info));

    switch (type)
    {
    case TOKEN_TABLE:
	break;
    case TOKEN_HCONTAINER:
    case TOKEN_VCONTAINER:
    case TOKEN_HANDLE_BOX:
	{
	    token = g_scanner_get_next_token (scanner);
	    if (token != G_TOKEN_STRING)
		return G_TOKEN_STRING;

	    if (g_slist_length(widget->data.container->child_list) > 1)
		if (widget->data.container->single_child)
		{
		    printf("gtuner: container `%s' can only have one child\n",
				widget->name);
		    return G_TOKEN_ERROR;
		}

	    if (g_slist_length(widget->data.container->child_list) > 1)
	    {
		list = g_slist_find_custom(widget->data.container->child_list,
				g_strdup (scanner->value.v_string),
				(GCompareFunc) layout_compare_child_info);
	    }

	    child_info->name = g_strdup (scanner->value.v_string);
	    child_info->pack = PACK_NORMAL;

	    if (list)
	    {
		if (list->data)
		    g_free(list->data);
		list->data = child_info;
	    }
	    else
	    {
		g_slist_append (widget->data.container->child_list,
				child_info);
	    }

	    break;
	}
    case TOKEN_TOOLBAR:
        break;
    }

    return G_TOKEN_NONE;
}

guint layout_parse_container_pack(GScanner *scanner, widgetLayout *widget,
                                 guint type)
{
    guint token;
    child_info *child_info;
    GSList *list = NULL;
    packType pack;

    switch (type)
    {
    case TOKEN_HCONTAINER:
    case TOKEN_VCONTAINER:
	break;
    default:
	return G_TOKEN_SYMBOL;
    }

    g_return_val_if_fail(widget != NULL, G_TOKEN_ERROR);
    g_return_val_if_fail(widget->data.container != NULL, G_TOKEN_ERROR);
    if (!widget->data.container->child_list)
	widget->data.container->child_list =
		g_slist_alloc();

    token = g_scanner_get_next_token (scanner);
    if ((token != TOKEN_PACK_START) && (token != TOKEN_PACK_END))
	return TOKEN_PACK_START;

    if (token == TOKEN_PACK_START)
	pack = PACK_START;
    else
	pack = PACK_END;

    child_info = malloc (sizeof(struct _child_info));
    memset (child_info, 0, sizeof(struct _child_info));

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_STRING)
	return G_TOKEN_STRING;

    if (g_slist_length(widget->data.container->child_list) > 1)
    list = g_slist_find_custom(widget->data.container->child_list,
			g_strdup (scanner->value.v_string),
			(GCompareFunc) layout_compare_child_info);

    child_info->name = g_strdup (scanner->value.v_string);

    child_info->pack = pack;

    if (list)
    {
	if (list->data)
	    g_free(list->data);
	list->data = child_info;
    }
    else
    {
	g_slist_append (widget->data.container->child_list,
			child_info);
    }

    return G_TOKEN_NONE;
}

guint layout_parse_container_border(GScanner *scanner, widgetLayout *widget,
                                 guint type)
{
    guint token;

    switch (type)
    {
    case TOKEN_TABLE:
    case TOKEN_HCONTAINER:
    case TOKEN_VCONTAINER:
    case TOKEN_HANDLE_BOX:
        break;
    default:
        return G_TOKEN_SYMBOL;
    }

    g_return_val_if_fail(widget != NULL, G_TOKEN_ERROR);
    g_return_val_if_fail(widget->data.container != NULL, G_TOKEN_ERROR);

    token = g_scanner_get_next_token (scanner);
    if (token != TOKEN_BORDER)
	return TOKEN_BORDER;

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_EQUAL_SIGN)
	return G_TOKEN_EQUAL_SIGN;

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_INT)
	return G_TOKEN_INT;

   widget->data.container->border_width = scanner->value.v_int;

   return G_TOKEN_NONE;
}

guint layout_info_station (GScanner *scanner, widgetLayout *widget, guint type)
{
    guint token;

    token = g_scanner_get_next_token (scanner);
    if (token != TOKEN_INFO_CHANNEL_NUMBER)
	return TOKEN_INFO_CHANNEL_NUMBER;

    switch (type)
    {
    case TOKEN_OBJECT_CHANNEL:
	break;
    default:
	return G_TOKEN_SYMBOL;
    }

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_EQUAL_SIGN)
	return G_TOKEN_EQUAL_SIGN;

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_INT)
	return G_TOKEN_INT;

    switch (type)
    {
    case TOKEN_OBJECT_CHANNEL:
	widget->data.chan->station = scanner->value.v_int;
        break;
    default:
        return G_TOKEN_SYMBOL;
    }

    return G_TOKEN_NONE;
}

guint layout_info_show_numbers (GScanner *scanner,
				widgetLayout *widget, guint type)
{
    guint token;

    token = g_scanner_get_next_token (scanner);
    if (token != TOKEN_INFO_SHOW_NUMBERS)
	return TOKEN_INFO_SHOW_NUMBERS;

    switch (type)
    {
    case TOKEN_OBJECT_CHANNEL:
    case TOKEN_OBJECT_CHANNEL_BOX:
	break;
    default:
	return G_TOKEN_SYMBOL;
    }

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_EQUAL_SIGN)
	return G_TOKEN_EQUAL_SIGN;

    token = g_scanner_get_next_token (scanner);
    if ((token != TOKEN_TRUE) && (token != TOKEN_FALSE))
	return TOKEN_TRUE;

    switch (type)
    {
    case TOKEN_OBJECT_CHANNEL:
	if (token == TOKEN_TRUE)
	    widget->data.chan->show_number = TRUE;
	else
	    widget->data.chan->show_number = FALSE;
	break;
    case TOKEN_OBJECT_CHANNEL_BOX:
	if (token == TOKEN_TRUE)
	    widget->data.chanbox->show_numbers = TRUE;
	else
	    widget->data.chanbox->show_numbers = FALSE;
	break;
    default:
        return G_TOKEN_SYMBOL;
    }

    return G_TOKEN_NONE;
}

guint layout_info_show_text (GScanner *scanner,
                                widgetLayout *widget, guint type)
{
    guint token;

    token = g_scanner_get_next_token (scanner);
    if (token != TOKEN_INFO_SHOW_TEXT)
	return TOKEN_INFO_SHOW_TEXT;

    switch (type)
    {
    case TOKEN_OBJECT_CHANNEL:
    case TOKEN_OBJECT_CHANNEL_BOX:
	break;
    default:
	return G_TOKEN_SYMBOL;
    }

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_EQUAL_SIGN)
	return G_TOKEN_EQUAL_SIGN;

    token = g_scanner_get_next_token (scanner);
    if ((token != TOKEN_TRUE) && (token != TOKEN_FALSE))
	return TOKEN_TRUE;

    switch (type)
    {
    case TOKEN_OBJECT_CHANNEL:
	if (token == TOKEN_TRUE)
	    widget->data.chan->show_text = TRUE;
	else
	    widget->data.chan->show_text = FALSE;
	break;
    case TOKEN_OBJECT_CHANNEL_BOX:
	if (token == TOKEN_TRUE)
	    widget->data.chanbox->show_text = TRUE;
	else
	    widget->data.chanbox->show_text = FALSE;
	break;
    default:
	return G_TOKEN_SYMBOL;
    }

    return G_TOKEN_NONE;
}

guint layout_info_size (GScanner *scanner,
                                widgetLayout *widget, guint type)
{
    guint token;
    gint width, height;

    token = g_scanner_get_next_token (scanner);
    if (token != TOKEN_INFO_SIZE)
	return TOKEN_INFO_SIZE;

    switch (type)
    {
    case TOKEN_OBJECT_CHANNEL:
    case TOKEN_OBJECT_CHANNEL_BOX:
	break;
    default:
	return G_TOKEN_SYMBOL;
    }

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_EQUAL_SIGN)
	return G_TOKEN_EQUAL_SIGN;

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_INT)
	return G_TOKEN_INT;
    width = scanner->value.v_int;

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_COMMA)
	return G_TOKEN_COMMA;

    token = g_scanner_get_next_token (scanner);    
    if (token != G_TOKEN_INT)    
	return G_TOKEN_INT;
    height = scanner->value.v_int;

    switch (type)
    {
    case TOKEN_OBJECT_CHANNEL:
	if (token == TOKEN_TRUE)
	    widget->data.chan->but_width = width;
	else
	    widget->data.chan->but_height = height;
	break;
    case TOKEN_OBJECT_CHANNEL_BOX:
	if (token == TOKEN_TRUE)
	    widget->data.chanbox->but_width = width;
	else
	    widget->data.chanbox->but_height = height;
	break;
    default:
        return G_TOKEN_SYMBOL;
    }

    return G_TOKEN_NONE;
}

guint layout_info_cols (GScanner *scanner,
			widgetLayout *widget, guint type)
{
    guint token;

    token = g_scanner_get_next_token (scanner);
    if (token != TOKEN_INFO_COLS)
	return TOKEN_INFO_COLS;

    switch (type)
    {
    case TOKEN_OBJECT_CHANNEL_BOX:
	break;
    default:
	return G_TOKEN_SYMBOL;
    }

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_EQUAL_SIGN)
	return G_TOKEN_EQUAL_SIGN;

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_INT)
	return G_TOKEN_INT;

    switch (type)
    {
    case TOKEN_OBJECT_CHANNEL_BOX:
	widget->data.chanbox->cols = scanner->value.v_int;
    }
    return G_TOKEN_NONE;
}

guint layout_info_rows (GScanner *scanner,
			widgetLayout *widget, guint type)
{
    guint token;

    token = g_scanner_get_next_token (scanner);
    if (token != TOKEN_INFO_ROWS)
	return TOKEN_INFO_ROWS;

    switch (type)
    {
    case TOKEN_OBJECT_CHANNEL_BOX:
	break;
    default:
	return G_TOKEN_SYMBOL;
    }

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_EQUAL_SIGN)
	return G_TOKEN_EQUAL_SIGN;

    token = g_scanner_get_next_token (scanner);
    if (token != G_TOKEN_INT)
	return G_TOKEN_INT;

    switch (type)
    {
    case TOKEN_OBJECT_CHANNEL_BOX:
	widget->data.chanbox->rows = scanner->value.v_int;
    }

    return G_TOKEN_NONE;
}

void layout_generate_tree()
{
    g_return_if_fail (layoutRoot != NULL);
    g_return_if_fail (rc_window_ht != NULL);
    g_return_if_fail (rc_widget_ht != NULL);

    g_hash_table_foreach (rc_window_ht, layout_hash_func, NULL);
}

void layout_hash_func (gpointer key, gpointer value, gpointer user_data)
{
    GNode *window_node;
    GNode *widget_node;
    windowLayout *rc_window;
    widgetLayout *widget;

    rc_window = value;

    window_node = g_node_append_data(layoutRoot, rc_window);

    g_return_if_fail(rc_window->child != NULL);
    widget = g_hash_table_lookup (rc_widget_ht, rc_window->child);

    if (!widget)
    {
	g_node_destroy(window_node);
	printf("gtuner: layout: unknown widget: `%s' in window `%s'\n",
		rc_window->child, rc_window->name);
	return;
    }

    widget_node = g_node_append_data(window_node, widget);
    switch (widget->type)
    {
    case TOKEN_TABLE:
    case TOKEN_HCONTAINER:
    case TOKEN_VCONTAINER:
    case TOKEN_HANDLE_BOX:
    case TOKEN_TOOLBAR:
	layout_generate_container_tree (widget_node);
	break;
    }
}

void layout_generate_container_tree (GNode *widget_node)
{
    widgetLayout *widget;
    containerLayout *container;

    g_return_if_fail(widget_node != NULL);
    g_return_if_fail(widget_node->data != NULL);

    widget = widget_node->data;
    g_return_if_fail(widget->data.container != NULL);

    container = widget->data.container;
    g_return_if_fail(container->child_list);

    g_slist_foreach (container->child_list,
			  layout_func_container, widget_node);
}

void layout_func_container (gpointer value, gpointer user_data)
{
    GNode *container_node, *widget_node;
    widgetLayout *widget, *container;
    child_info *child_info;

    g_return_if_fail (user_data != NULL);
    if (!value)
	return;

    container_node = user_data;
    container = container_node->data;
    child_info = value;
    g_return_if_fail(child_info->name != NULL);

    widget = g_hash_table_lookup (rc_widget_ht, child_info->name);

    if (!widget)
    {
	printf("gtuner: layout: unknown widget: `%s' in container `%s'\n",
			child_info->name, container->name);
	return;
    }
    if (!strcmp(widget->name ,container->name))
    {
	printf("gtuner: layout: recursive loop detected: widget `%s'\n",
			container->name);
	return;
    }

    widget_node = g_node_append_data(container_node, widget);
    switch (widget->type)
    {
    case TOKEN_TABLE:
    case TOKEN_HCONTAINER:
    case TOKEN_VCONTAINER:
    case TOKEN_HANDLE_BOX:
    case TOKEN_TOOLBAR:
	layout_generate_container_tree (widget_node);
	break;
    }
}

int layout_generate()
{
    GNode *node;

    g_return_val_if_fail (layoutRoot != NULL, 0);

    layout_generate_tree();

    node = g_node_first_child(layoutRoot);
    g_return_val_if_fail (node != NULL, 0);

    g_return_val_if_fail (layoutWidgetTree == NULL, 0);
    layoutWidgetTree = g_slist_alloc();

    while (node)
    {
	GtkWidget *window, *widget = NULL;
	windowLayout *rc_window;
	GNode *widget_node;
	widgetLayout *rc_widget;
	gtuner_window *gt_win;

#ifdef USE_GNOME
	window = gnome_app_new("gtuner", "gtuner");
#else
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
#endif
	gt_win = malloc (sizeof (gtuner_window));
	gt_win->window = window;
	gt_win->child = NULL;
	layoutWidgetTree = g_slist_append(layoutWidgetTree, gt_win);

	gtk_accel_group_attach (accel_group, GTK_OBJECT (window));

	rc_window = (windowLayout*)node->data;
	if (rc_window->name)
	    gtk_window_set_title(GTK_WINDOW(window), rc_window->name);

	widget_node = g_node_first_child(node);
	g_return_val_if_fail (widget_node != NULL, 0);

	rc_widget = (widgetLayout*)widget_node->data;
	g_return_val_if_fail (rc_widget != NULL, 0);

	switch(rc_window->close)
	{
	case TOKEN_WINDOW_CLOSE_QUIT:
	    gtk_signal_connect(GTK_OBJECT(window), "delete_event",
				(GtkSignalFunc) gtk_main_quit, NULL);
	    break;
	case TOKEN_WINDOW_CLOSE_HIDE:
	    /* Need a function to restore */
	    gtk_signal_connect(GTK_OBJECT(window), "delete_event",
				(GtkSignalFunc) gtk_widget_hide, window);
	    break;
	case TOKEN_WINDOW_CLOSE_DESTROY:
	    /* FIXME Need to do a traverse (destroy on leave) */
	    break;
	}

	switch(rc_widget->type)
	{
	case TOKEN_OBJECT_VOLUME:
	case TOKEN_OBJECT_DISPLAY:
	case TOKEN_OBJECT_FREQ_BAR:
	case TOKEN_OBJECT_QUIT:
	case TOKEN_OBJECT_MUTE:
	case TOKEN_OBJECT_STEREO:
	case TOKEN_OBJECT_CHANNEL:
	case TOKEN_OBJECT_CHANNEL_BOX:
	    widget = layout_generate_object (widget_node);
	    break;
	case TOKEN_TABLE:
	case TOKEN_HCONTAINER:
	case TOKEN_VCONTAINER:
	case TOKEN_HANDLE_BOX:
	case TOKEN_TOOLBAR:
	    widget = layout_generate_container (widget_node);
	    break;
	}
	g_return_val_if_fail(widget != NULL, 0);

	gtk_widget_show(widget);
#ifdef USE_GNOME
	gnome_app_set_contents(GNOME_APP(window), widget);
#else
	gtk_container_add(GTK_CONTAINER(window), widget);
#endif
	gtk_widget_show(window);

	gtk_widget_realize(window);
	gdk_window_set_icon_name(window->window, rc_window->name);
	if (config->pix.icon_xpm)
	    gdk_window_set_icon(window->window, NULL,
			    config->pix.icon_xpm, config->pix.icon_mask);

	node = g_node_next_sibling(node);
    }
    return TRUE;
}

GtkWidget* layout_generate_object(GNode *node)
{
    widgetLayout *rc_widget;

    g_return_val_if_fail (node != NULL, NULL);
    rc_widget = (widgetLayout*)node->data;
    g_return_val_if_fail (rc_widget != NULL, NULL);

    switch(rc_widget->type)
    {
    case TOKEN_OBJECT_VOLUME:
	return layout_generate_object_volume (node);
    case TOKEN_OBJECT_DISPLAY:
	return layout_generate_object_display (node);
    case TOKEN_OBJECT_FREQ_BAR:
	return layout_generate_object_freq_bar (node);
    case TOKEN_OBJECT_QUIT:
	return layout_generate_object_quit (node);
    case TOKEN_OBJECT_MUTE:
	return layout_generate_object_mute (node);
    case TOKEN_OBJECT_STEREO:
	return layout_generate_object_stereo (node);
    case TOKEN_OBJECT_CHANNEL:
	return layout_generate_object_channel (node);
    case TOKEN_OBJECT_CHANNEL_BOX:
	return layout_generate_object_chanbox (node);
    }
    return NULL;
}

GtkWidget* layout_generate_object_freq_bar(GNode *node)
{
    widgetLayout *rc_widget;
    struct freq_bar *freq_bar_struct;

    g_return_val_if_fail (node != NULL, NULL);
    rc_widget = (widgetLayout*)node->data;
    g_return_val_if_fail (rc_widget != NULL, NULL);
    g_return_val_if_fail (rc_widget->data.freq_bar != NULL, NULL);

    freq_bar_struct = malloc(sizeof(struct freq_bar));
    switch(rc_widget->data.freq_bar->sub_type)
    {
    case TOKEN_TYPE_FREQ_PLAIN:
	hfreq_bar_new(freq_bar_struct, 0);
	g_slist_append(freq_bar_list, freq_bar_struct);
	break;
    case TOKEN_TYPE_FREQ_RULER:
	hfreq_bar_new(freq_bar_struct, 1);
	g_slist_append(freq_bar_list, freq_bar_struct);
	break;
    case TOKEN_TYPE_FREQ_DISP:
	hfreq_bar_new(freq_bar_struct, 2);
	g_slist_append(freq_bar_list, freq_bar_struct);
	break;
    case TOKEN_TYPE_FREQ_SCROLL:
	hsfreq_bar_new(freq_bar_struct);
	g_slist_append(freq_bar_list, freq_bar_struct);
	break;
    default:
	printf("gtuner: layout: generate: freq_bar: unknown type %d\n",
		rc_widget->data.freq_bar->sub_type);
	return NULL;
    }

    return freq_bar_struct->base;
}

GtkWidget* layout_generate_object_volume(GNode *node)
{
    widgetLayout *rc_widget;
    struct volume_changer *volume_struct;

    g_return_val_if_fail (node != NULL, NULL);
    rc_widget = (widgetLayout*)node->data;
    g_return_val_if_fail (rc_widget != NULL, NULL);
    g_return_val_if_fail (rc_widget->data.volume != NULL, NULL);

    volume_struct = malloc(sizeof(struct volume_changer));
    volume_struct->vol = 0;

    switch(rc_widget->data.volume->sub_type)
    {
    case TOKEN_TYPE_VOLUME_HSCALE:
	volume_hbar_new (volume_struct);
	break;
    case TOKEN_TYPE_VOLUME_VSCALE:
	volume_vbar_new (volume_struct);
	break;
    case TOKEN_TYPE_VOLUME_HORIZ_PM:
	volume_plus_minus_new(volume_struct, TRUE);
	break;
    case TOKEN_TYPE_VOLUME_VERT_PM:
	volume_plus_minus_new(volume_struct, FALSE);
	break;
    case TOKEN_TYPE_VOLUME_DIAL:
	volume_dial_new(volume_struct);
	break;
    default:
        printf("gtuner: layout: generate: volume: unknown type %d\n",
                rc_widget->data.volume->sub_type);
        return NULL;
    }

    g_slist_append(volume_list, volume_struct);
    return volume_struct->base;
}

GtkWidget* layout_generate_object_display(GNode *node)
{
    widgetLayout *rc_widget;
    struct display *display_struct;

    g_return_val_if_fail (node != NULL, NULL);
    rc_widget = (widgetLayout*)node->data;
    g_return_val_if_fail (rc_widget != NULL, NULL);
    g_return_val_if_fail (rc_widget->data.display != NULL, NULL);

    display_struct = malloc(sizeof(struct display));

    switch(rc_widget->data.volume->sub_type)
    {
    case TOKEN_TYPE_DISPLAY_LCD:
	disp_lcd_new(display_struct);
	disp_refresh(display_struct);
	break;
    default:
	printf("gtuner: layout: generate: display: unknown type %d\n",
		rc_widget->data.display->sub_type);
	return NULL;
    }

    g_slist_append(display_list, display_struct);
    gtk_widget_show(display_struct->base);
    return display_struct->base;
}

GtkWidget* layout_generate_object_quit(GNode *node)
{
    widgetLayout *rc_widget;
    GtkWidget *quit;

    g_return_val_if_fail (node != NULL, NULL);
    rc_widget = (widgetLayout*)node->data;
    g_return_val_if_fail (rc_widget != NULL, NULL);
    g_return_val_if_fail (rc_widget->data.quit != NULL, NULL);

    switch(rc_widget->data.quit->sub_type)
    {
    case TOKEN_TYPE_BUTTON:
	quit = quit_button_new();
	break;
    default:
	printf("gtuner: layout: generate: quit: unknown type %d\n",
		rc_widget->data.quit->sub_type);
	return NULL;
    }
    return quit;
}

GtkWidget* layout_generate_object_mute(GNode *node)
{
    widgetLayout *rc_widget;
    struct mute_button *mute;

    g_return_val_if_fail (node != NULL, NULL);
    rc_widget = (widgetLayout*)node->data;
    g_return_val_if_fail (rc_widget != NULL, NULL);
    g_return_val_if_fail (rc_widget->data.mute != NULL, NULL);

    switch(rc_widget->data.mute->sub_type)
    {
    case TOKEN_TYPE_TOGGLE:
        mute = mute_button_new();
	g_slist_append(mute_list, mute);
        break;
    case TOKEN_TYPE_CHECK:
	mute = mute_checkbutton_new();
	g_slist_append(mute_list, mute);
	break;
    default:
        printf("gtuner: layout: generate: mute: unknown type %d\n",
                rc_widget->data.mute->sub_type);
        return NULL;
    }
    return mute->base;
}

GtkWidget* layout_generate_object_stereo(GNode *node)
{
    widgetLayout *rc_widget;
    struct stereo_button *stereo;

    g_return_val_if_fail (node != NULL, NULL);
    rc_widget = (widgetLayout*)node->data;
    g_return_val_if_fail (rc_widget != NULL, NULL);
    g_return_val_if_fail (rc_widget->data.stereo != NULL, NULL);

    switch(rc_widget->data.stereo->sub_type)
    {
    case TOKEN_TYPE_TOGGLE:
	stereo = stereo_button_new();
	g_slist_append(stereo_list, stereo);
	break;
    case TOKEN_TYPE_CHECK:
	stereo = stereo_checkbutton_new();
	g_slist_append(stereo_list, stereo);
	break;
    default:
	printf("gtuner: layout: generate: stereo: unknown type %d\n",
		rc_widget->data.stereo->sub_type);
	return NULL;
    }
    return stereo->base;
}

GtkWidget* layout_generate_object_channel(GNode *node)
{
    widgetLayout *rc_widget;
    GtkWidget *channel;
    struct channel_button_data *data;
    struct channel_object *obj;
    channelLayout *chan;

    g_return_val_if_fail (node != NULL, NULL);
    rc_widget = (widgetLayout*)node->data;
    g_return_val_if_fail (rc_widget != NULL, NULL);
    g_return_val_if_fail (rc_widget->data.stereo != NULL, NULL);

    chan = rc_widget->data.chan;

    switch(chan->sub_type)
    {
    case TOKEN_TYPE_BUTTON:
	obj = malloc(sizeof(struct channel_object));
	obj->base = NULL;
	obj->refresh = NULL;
	data = malloc(sizeof(struct channel_button_data));
	data->but_width = chan->but_width;
	data->but_height = chan->but_height;
	data->channel_num = chan->station;
	data->show_numbers = chan->show_number;
	data->show_text = chan->show_text;
        obj->data.channel = data;

	channel_button_new(obj, config->first_channel);
	if (obj->refresh)
	    (obj->refresh)(obj, config->first_channel);
	g_slist_append(channel_list, obj);

	channel = obj->base;
	break;
    default:
	printf("gtuner: layout: generate: channel: unknown type %d\n",
		rc_widget->data.chan->sub_type);
	return NULL;
    }
    return channel;
}

GtkWidget* layout_generate_object_chanbox(GNode *node)
{
    struct channel_object *chan_box;
    GtkWidget *chanbox;
    widgetLayout *rc_widget;

    g_return_val_if_fail (node != NULL, NULL);
    rc_widget = (widgetLayout*)node->data;
    g_return_val_if_fail (rc_widget != NULL, NULL);
    g_return_val_if_fail (rc_widget->data.chanbox != NULL, NULL);

    switch(rc_widget->data.chanbox->sub_type)
    {
    case TOKEN_TYPE_CHANNELBOX_VERT:
	chan_box = malloc(sizeof(struct channel_object));
	chan_box->base = NULL;
	chan_box->refresh = NULL;
	chan_box->data.v_box = malloc(sizeof(struct v_chan_box));
	chan_box->data.v_box->but_width = rc_widget->data.chanbox->but_width;
	chan_box->data.v_box->but_height = rc_widget->data.chanbox->but_height;
	chan_box->data.v_box->current_offset = 0;
	chan_box->data.v_box->show_numbers =
			rc_widget->data.chanbox->show_numbers;
	chan_box->data.v_box->show_text = rc_widget->data.chanbox->show_text;
	chan_box->data.v_box->cols = rc_widget->data.chanbox->cols;
	chan_box->data.v_box->rows = rc_widget->data.chanbox->rows;

	vchannel_box_new(chan_box);
	g_slist_append(channel_list, chan_box);
	if (chan_box->refresh)
	    (chan_box->refresh)(chan_box, config->first_channel);
	chanbox = chan_box->base;
	gtk_widget_show(chanbox);
	break;
    case TOKEN_TYPE_CHANNELBOX_HORIZ:
	chan_box = malloc(sizeof(struct channel_object));
	chan_box->base = NULL;
	chan_box->refresh = NULL;
	chan_box->data.h_box = malloc(sizeof(struct v_chan_box));
	chan_box->data.h_box->but_width = rc_widget->data.chanbox->but_width;
	chan_box->data.h_box->but_height = rc_widget->data.chanbox->but_height;
	chan_box->data.h_box->current_offset = 0;
	chan_box->data.h_box->show_numbers =
			rc_widget->data.chanbox->show_numbers;
	chan_box->data.h_box->show_text = rc_widget->data.chanbox->show_text;
	chan_box->data.h_box->cols = rc_widget->data.chanbox->cols;
	chan_box->data.h_box->rows = rc_widget->data.chanbox->rows;

	hchannel_box_new(chan_box);
	g_slist_append(channel_list, chan_box);
	if (chan_box->refresh)
	    (chan_box->refresh)(chan_box, config->first_channel);
	chanbox = chan_box->base;
	gtk_widget_show(chanbox);
	break;
    default:
        printf("gtuner: layout: generate: chanbox: unknown type %d\n",
                rc_widget->data.chanbox->sub_type);
        return NULL;
    }
    return chanbox;
}

GtkWidget* layout_generate_container(GNode *node)
{
    widgetLayout *rc_widget, *rc_container;
    GtkWidget *container = NULL, *widget = NULL;
    GNode *widget_node;

    g_return_val_if_fail (node != NULL, NULL);
    rc_container = (widgetLayout*)node->data;
    g_return_val_if_fail (rc_container != NULL, NULL);

    switch(rc_container->type)
    {
    case TOKEN_TABLE:
	container = NULL;
	gtk_container_border_width (GTK_CONTAINER(container),
				    rc_container->data.container->border_width);
	break;
    case TOKEN_HCONTAINER:
	container = layout_generate_h_container(rc_container);
	gtk_container_border_width (GTK_CONTAINER(container),
				    rc_container->data.container->border_width);
	break;
    case TOKEN_VCONTAINER:
	container = layout_generate_v_container(rc_container);
	gtk_container_border_width (GTK_CONTAINER(container),
				    rc_container->data.container->border_width);
	break;
    case TOKEN_HANDLE_BOX:
	container = layout_generate_handle_box(rc_container);
	gtk_container_border_width (GTK_CONTAINER(container),
				    rc_container->data.container->border_width);
	break;
    case TOKEN_TOOLBAR:
	container = NULL;
	break;
    }
    g_return_val_if_fail(container != NULL, NULL);

    widget_node = g_node_first_child(node);
    g_return_val_if_fail (widget_node != NULL, NULL);

    while (widget_node)
    {
	child_info *child_info;
	GSList *child_slist;

	rc_widget = (widgetLayout*)widget_node->data;
	g_return_val_if_fail (rc_widget != NULL, NULL);
	child_slist = g_slist_find_custom (
		rc_container->data.container->child_list,
		rc_widget->name, (GCompareFunc) layout_compare_child_info);

	g_return_val_if_fail (child_slist != NULL, NULL);
	g_return_val_if_fail (child_slist->data != NULL, NULL);
	child_info = child_slist->data;
	g_return_val_if_fail (child_info->name != NULL, NULL);

	switch(rc_widget->type)
	{
	case TOKEN_OBJECT_VOLUME:
	case TOKEN_OBJECT_DISPLAY:
	case TOKEN_OBJECT_FREQ_BAR:
	case TOKEN_OBJECT_QUIT:
	case TOKEN_OBJECT_MUTE:
	case TOKEN_OBJECT_STEREO:
	case TOKEN_OBJECT_CHANNEL:
	case TOKEN_OBJECT_CHANNEL_BOX:
	    widget = layout_generate_object (widget_node);
	    break;
	case TOKEN_TABLE:
	case TOKEN_HCONTAINER:
	case TOKEN_VCONTAINER:
	case TOKEN_HANDLE_BOX:
	case TOKEN_TOOLBAR:
	    widget = layout_generate_container (widget_node);
	    break;
	}
	g_return_val_if_fail (widget != NULL, NULL);
	gtk_widget_show (widget);
	switch (child_info->pack)
	{
	case PACK_START:
	    gtk_box_pack_start (GTK_BOX(container), widget,
				FALSE, TRUE, 0); /*padding*/
	    break;
	case PACK_NORMAL:
	    gtk_container_add (GTK_CONTAINER(container), widget);
	    break;
	case PACK_END:
	    gtk_box_pack_end (GTK_BOX(container), widget, FALSE, TRUE, 0);
	    break;
	case PACK_TABLE:
	    break;
	}
        gtk_widget_show (container);

	widget_node = widget_node->next;
    }
    return container;
}

GtkWidget *layout_generate_h_container(widgetLayout *rc_widget)
{
    GtkWidget *container;

    g_return_val_if_fail (rc_widget != NULL, NULL);
    g_return_val_if_fail (rc_widget->data.container != NULL, NULL);
    g_return_val_if_fail (rc_widget->data.container->data.box != NULL, NULL);

    container = gtk_hbox_new(rc_widget->data.container->data.box->homogeneous,
			     rc_widget->data.container->data.box->spacing);
    return container;
}

GtkWidget *layout_generate_v_container(widgetLayout *rc_widget)
{
    GtkWidget *container;

    g_return_val_if_fail (rc_widget != NULL, NULL);
    g_return_val_if_fail (rc_widget->data.container != NULL, NULL);
    g_return_val_if_fail (rc_widget->data.container->data.box != NULL, NULL);

    container = gtk_vbox_new(rc_widget->data.container->data.box->homogeneous,
			     rc_widget->data.container->data.box->spacing);
    return container;
}

GtkWidget *layout_generate_handle_box(widgetLayout *rc_widget)
{
    GtkWidget *handle;

    g_return_val_if_fail (rc_widget != NULL, NULL);
    g_return_val_if_fail (rc_widget->data.container != NULL, NULL);
    g_return_val_if_fail (rc_widget->data.container->data.handle != NULL, NULL);

    handle = gtk_handle_box_new();
    gtk_handle_box_set_shadow_type(GTK_HANDLE_BOX (handle),
			rc_widget->data.container->data.handle->type);

    return handle;
}

static gboolean windows_state = TRUE;

void layout_toggle_windows (gboolean state)
/* set windows visable or not */
{
    GSList *winlist;

    g_return_if_fail (layoutRoot != NULL);
    g_return_if_fail (layoutWidgetTree != NULL);

    winlist = layoutWidgetTree;
    while (winlist)
    {
	gtuner_window *win;
	GtkWidget *window;
	win = winlist->data;
	if (win)
	{
	    window = win->window;
	    if (window)
	    {
		if (state) gtk_widget_show(window);
		else gtk_widget_hide(window);
	    }
	}

	winlist = g_slist_next(winlist);
    }

    windows_state = state;
}

gboolean layout_windows_state ()
/* return current windows state visable = TRUE */
{
    g_return_val_if_fail (layoutRoot != NULL, 0);
    g_return_val_if_fail (layoutWidgetTree != NULL, 0);

    return windows_state;
}

