/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* layout.h */
#ifndef LAYOUT_PRIVATE_H
#define LAYOUT_PRIVATE_H

#include <gtk/gtk.h>
#include <glib.h>

GNode* layoutRoot = NULL;
GSList* layoutWidgetTree = NULL;

typedef enum {
    TOKEN_INVALID = G_TOKEN_LAST,
    TOKEN_WINDOW,
    TOKEN_WINDOW_CHILD,
    TOKEN_WINDOW_CLOSE,
    TOKEN_WINDOW_CLOSE_QUIT,
    TOKEN_OBJECT_QUIT = TOKEN_WINDOW_CLOSE_QUIT,
    TOKEN_WINDOW_CLOSE_HIDE,
    TOKEN_WINDOW_CLOSE_DESTROY,

    TOKEN_TABLE,
    TOKEN_HCONTAINER,
    TOKEN_VCONTAINER,
    TOKEN_PACK_START,
    TOKEN_PACK_END,
    TOKEN_ADD,
    TOKEN_BORDER,
    TOKEN_HANDLE_BOX,
    TOKEN_TOOLBAR,
    TOKEN_TOOLBAR_ADD_SPACE,
    TOKEN_TOOLBAR_INSERT,
    TOKEN_TOOLBAR_SPACE_WIDTH,
    TOKEN_TOOLBAR_SPACE_STYLE,
    TOKEN_TOOLBAR_SHOW_TOOLTIPS,
    TOKEN_TOOLBAR_STYLE,

    TOKEN_TYPE,
    TOKEN_TYPE_VOLUME_HSCALE,
    TOKEN_TYPE_VOLUME_VSCALE,
    TOKEN_TYPE_VOLUME_HORIZ_PM,
    TOKEN_TYPE_VOLUME_VERT_PM,
    TOKEN_TYPE_VOLUME_DIAL,
    TOKEN_TYPE_DISPLAY_LCD,
    TOKEN_TYPE_FREQ_PLAIN,
    TOKEN_TYPE_FREQ_RULER,
    TOKEN_TYPE_FREQ_DISP,
    TOKEN_TYPE_FREQ_SCROLL,
    TOKEN_TYPE_BUTTON,
    TOKEN_TYPE_TOGGLE,
    TOKEN_TYPE_CHECK,
    TOKEN_TYPE_CHANNELBOX_VERT,
    TOKEN_TYPE_CHANNELBOX_HORIZ,

    TOKEN_INFO_CHANNEL_NUMBER,
    TOKEN_INFO_SHOW_NUMBERS,
    TOKEN_INFO_SHOW_TEXT,
    TOKEN_INFO_SIZE,
    TOKEN_INFO_COLS,
    TOKEN_INFO_ROWS,

    TOKEN_OBJECT_VOLUME,
    TOKEN_OBJECT_DISPLAY,
    TOKEN_OBJECT_FREQ_BAR,
    TOKEN_OBJECT_MUTE,
    TOKEN_OBJECT_STEREO,
    TOKEN_OBJECT_CHANNEL,
    TOKEN_OBJECT_CHANNEL_BOX,

    TOKEN_TRUE,
    TOKEN_FALSE,

    TOKEN_LAST
} layoutToken;

typedef struct _windowLayout windowLayout;
typedef struct _widgetLayout widgetLayout;
typedef struct _containerLayout containerLayout;
typedef struct _freq_barLayout freq_barLayout;
typedef struct _volumeLayout volumeLayout;
typedef struct _displayLayout displayLayout;
typedef struct _quitLayout quitLayout;
typedef struct _muteLayout muteLayout;
typedef struct _stereoLayout stereoLayout;
typedef struct _chanboxLayout chanboxLayout;
typedef struct _child_info child_info;
typedef struct _box_container box_container;
typedef struct _handle_container handle_container;
typedef struct _channelLayout channelLayout;

struct _windowLayout {
    gchar *name;
    gchar *child;
    guint close;
};

struct _widgetLayout {
    char *name;
    guint type;
    union {
        containerLayout *container;
        freq_barLayout *freq_bar;
	volumeLayout *volume;
	displayLayout *display;
	quitLayout *quit;
	muteLayout *mute;
	stereoLayout *stereo;
	channelLayout *chan;
	chanboxLayout *chanbox;
    } data;
};

struct _containerLayout {
    gboolean single_child;
    GSList* child_list;
    gint border_width;
    union {
	box_container *box;
	handle_container *handle;
    } data;
};

struct _box_container {
    gboolean homogeneous;
    gint spacing;
};

struct _handle_container {
    GtkShadowType type;
};

struct _freq_barLayout {
    guint sub_type;
};

struct _volumeLayout {
    guint sub_type;
};

struct _displayLayout {
    guint sub_type;
};

struct _quitLayout {
    guint sub_type;
};

struct _muteLayout {
    guint sub_type;
};

struct _stereoLayout {
    guint sub_type;
};

struct _channelLayout {
    guint sub_type;
    guint station;
    int but_width;
    int but_height;
    gboolean show_number;
    gboolean show_text;
};

struct _chanboxLayout {
    guint sub_type;
    int but_width;
    int but_height;
    gboolean show_numbers;
    gboolean show_text;
    int cols;
    int rows;
};

typedef enum {
    PACK_START,
    PACK_NORMAL,
    PACK_END,
    PACK_TABLE
} packType;

struct _child_info {
    char *name;
    packType pack;
    int top_x;
    int top_y;
    int bot_x;
    int bot_y;
};

typedef enum {
    CONTAINER,
    VOLUME,
    FREQBAR,
    DISPLAY,
    CHANNEL,
    OTHER
} gtuner_widget_type;

typedef struct _gtuner_widget gtuner_widget;
typedef struct _gtuner_window gtuner_window;

struct _gtuner_widget {
    gtuner_widget_type type;
    void *widget;
};

struct _gtuner_window {
    GtkWidget *window;
    gtuner_widget *child;
};

#endif /* LAYOUT_H */

