/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "gui.h"
#include "popup_menu.h"
#include "prefs.h"
#include "../callbacks.h"
#include <stdio.h>
#include <gtk/gtkitemfactory.h>
#include <malloc.h>
#include <sys/utsname.h>
#ifdef USE_GNOME
#include <gnome.h>
#endif

enum {
        POPUP_PREFS=0, POPUP_ALARM, POPUP_SLEEP, POPUP_STEREO,
        POPUP_EXIT, POPUP_SAVE, POPUP_MUTE
};

void popup_about( GtkWidget *w, gpointer   data );

#ifndef USE_GNOME
static void popup_callback( GtkWidget *w, gpointer   data );

static GtkItemFactoryEntry popup_menu_items[] = {
	{ "/About gtuner",NULL, popup_about, 0, "<Item>" },
	{ "/-",         NULL,NULL,0,"<Separator>"},
        { "/Preferences","<control>P",popup_callback, POPUP_PREFS, "<Item>" },
	{ "/Save settings",	NULL,popup_callback, POPUP_SAVE, "<Item>" },
	{ "/-",         NULL,NULL,0,"<Separator>"},
	{ "/Stations",	NULL, NULL, 0, "<Item>" },
	{ "/-",		NULL,NULL,0,"<Separator>"},
	{ "/Mute","M",	popup_callback, POPUP_MUTE, "<ToggleItem>" },
	{ "/Stereo","S", popup_callback, POPUP_STEREO,"<ToggleItem>" },
	{ "/Alarm Clock","A",popup_callback,POPUP_ALARM,"<ToggleItem>" },
	{ "/Sleep Timer","T",popup_callback,POPUP_SLEEP,"<ToggleItem>" },
	{ "/-",		NULL,NULL,0,"<Separator>"},
        { "/Exit",      "<control>X",popup_callback, POPUP_EXIT, "<Item>" },
};

#define NUM_ENTRIES 13

static void about_close_cb(GtkWidget *w,gpointer data)
{
	gtk_widget_destroy((GtkWidget*)data);
}

#endif

extern struct radio_config *config;
void callback_set_stereo(GtkWidget *window, gboolean state);
void callback_set_channel(GtkWidget *, struct channel *);

void popup_about( GtkWidget *w, gpointer   data )
{
#ifdef USE_GNOME

#ifndef __DATE__
#define __DATE__ "<unknown>"
#endif

    GtkWidget *about;
    const gchar *author[] = { "Carl van Schaik <carl@seul.org>", 0 };
    char buf[512];
    struct utsname un;

    uname(&un);
   
    snprintf(buf, sizeof buf,
		"gtuner Project Started:   1 Dec 1998\n"
		"A highly configurable radio card tuner\n"
		"for X windows, written in gtk\n"
		"Author EMail:   carl@seul.org\n"
		"gtuner Web Page: http://www.gtuner.seul.org\n"
		"Author Web Page: http://www.leg.uct.ac.za/~carl/\n"
		"This software comes with NO WARRANTY at all, please\n"
		"refer to COPYING file for more details.\n");

    about = gnome_about_new("gtuner",
		VERSION, "(C) 1998-1999 Carl van Schaik",
		author, buf, NULL);
    gtk_widget_show(about);

#else
    GtkWidget *dialog, *button, *label;

    dialog = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(dialog),
		"About gtuner");

    label = gtk_label_new("gtuner\n(c) Carl van Schaik 1998-1999\n"
			    "carl@seul.org\n\n"
			    "A highly configurable radio card tuner\n"
			    "for X windows, written in gtk\n\n"
			    "http://www.gtuner.seul.org");
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),label,FALSE,FALSE,0);
    gtk_container_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), 5);

    button=gtk_button_new_with_label(" Close ");
    gtk_signal_connect(GTK_OBJECT(button),"clicked",GTK_SIGNAL_FUNC(about_close_cb),dialog);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),button,FALSE,FALSE,0);

    gtk_signal_connect(GTK_OBJECT(dialog),"destroy",
		GTK_SIGNAL_FUNC(gtk_widget_destroyed),&dialog);
    gtk_widget_show_all(dialog);
#endif
}

static void popup_station_callback(GtkWidget *w, gpointer   data )
{
    gint cmd = (gint)data-1;
    callback_set_channel(w, get_channel(config->first_channel, cmd));
}

#ifdef USE_GNOME

GtkWidget *popup_menu, *popup_stations_widget;
GtkItemFactory *popup_stations;
GtkAccelGroup *accel_group;
GtkCheckMenuItem *mute_item;
GtkCheckMenuItem *stereo_item;
GtkCheckMenuItem *alarm_item;
GtkCheckMenuItem *timer_item;
GtkMenuItem *stations_item;

static void save_settings(GtkWidget *w, gpointer data);
static void mute_cb(GtkWidget *w, gpointer   data);
static void stereo_cb(GtkWidget *w, gpointer   data);
static void timer_cb(GtkWidget *w, gpointer   data);
static void alarm_cb(GtkWidget *w, gpointer   data);
static void quit_cb(GtkWidget *w, gpointer   data);

static GnomeUIInfo stations[] =
{
    GNOMEUIINFO_END
};

static GnomeUIInfo popups[] =
    {
	{
	    GNOME_APP_UI_ITEM,
	    "About gtuner", "",
	    popup_about, 0, NULL,
	    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT,
	    0, 0, NULL
	},
	GNOMEUIINFO_SEPARATOR,
	{
	    GNOME_APP_UI_ITEM,
	    "Preferences", "",
	    show_preferences, NULL, NULL,
	    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PREF,
	    'P', GDK_CONTROL_MASK, NULL
	},
	{
	    GNOME_APP_UI_ITEM,
	    "Save settings", "",
	    save_settings, NULL, NULL,
	    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE,
	    0, 0, NULL
	},
	GNOMEUIINFO_SEPARATOR,
	{
	    GNOME_APP_UI_SUBTREE,
	    "Stations", "",
	    stations, NULL, NULL,
	    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_JUMP_TO,
	    0, 0, NULL
	},
	GNOMEUIINFO_SEPARATOR,
	
	{
	    GNOME_APP_UI_TOGGLEITEM,
	    "Mute", "",
	    mute_cb, NULL, NULL,
	    GNOME_APP_PIXMAP_NONE, NULL,
	    'M', 0, NULL
	},
	{
	    GNOME_APP_UI_TOGGLEITEM,
	    "Stereo", "",
	    stereo_cb, NULL, NULL,
	    GNOME_APP_PIXMAP_NONE, NULL,
	    'S', 0, NULL
	},
	{
	    GNOME_APP_UI_TOGGLEITEM,
	    "Alarm Clock", "",
	    alarm_cb, NULL, NULL,
	    GNOME_APP_PIXMAP_NONE, NULL,
	    'A', 0, NULL
	},
	{
	    GNOME_APP_UI_TOGGLEITEM,
	    "Sleep Timer", "",
	    timer_cb, NULL, NULL,
	    GNOME_APP_PIXMAP_NONE, NULL,
	    'T', 0, NULL
	},
	GNOMEUIINFO_SEPARATOR,
	{
	    GNOME_APP_UI_ITEM,
	    "Quit", "",
	    quit_cb, NULL, NULL,
	    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_QUIT,
	    'Q', GDK_CONTROL_MASK, NULL
	},
	GNOMEUIINFO_END
    };

static void save_settings(GtkWidget *w, gpointer   data)
{
    save_config(config);
}

static void mute_cb(GtkWidget *w, gpointer   data)
{
    if (mute_item->active)
	callback_mute(NULL, NULL);
    else
	callback_unmute(NULL, NULL);
}

static void stereo_cb(GtkWidget *w, gpointer   data)
{
    callback_set_stereo(NULL,stereo_item->active);
}

static void alarm_cb(GtkWidget *w, gpointer   data)
{
    if (alarm_item->active)
	activate_alarm(config);
    else
	deactivate_alarm(config);
}

static void timer_cb(GtkWidget *w, gpointer   data)
{
    if (timer_item->active)
	activate_timer(config);
    else
	deactivate_timer(config);
}

static void quit_cb(GtkWidget *w, gpointer   data)
{
    gtk_main_quit();
}

void initialize_popups()
{
    accel_group = gtk_accel_group_new ();
    popup_menu = gnome_popup_menu_new_with_accelgroup (popups, accel_group);

    stations_item = (GtkMenuItem*)(popups[5].widget);
    mute_item = (GtkCheckMenuItem*)(popups[7].widget);
    stereo_item = (GtkCheckMenuItem*)(popups[8].widget);
    alarm_item = (GtkCheckMenuItem*)(popups[9].widget);
    timer_item = (GtkCheckMenuItem*)(popups[10].widget);

    gtk_check_menu_item_set_active(alarm_item, config->alarm_on);
    gtk_check_menu_item_set_active(timer_item, config->timer_on);
    gtk_check_menu_item_set_active(stereo_item, TRUE);

    popups_reset_stations();
}

void popups_reset_stations()
{
    int i=1;
    GtkItemFactoryEntry *station_item;
    GtkItemFactory *temp;
    temp = popup_stations;

    popup_stations = gtk_item_factory_new (GTK_TYPE_MENU, "<main>",
							accel_group);

    popup_stations_widget = GTK_WIDGET(gtk_item_factory_get_widget(
			                        popup_stations,""));
    gtk_menu_item_set_submenu(stations_item, popup_stations_widget);

    station_item = malloc(sizeof (struct _GtkItemFactoryEntry));
    station_item->accelerator = NULL;
    station_item->item_type = NULL;
    station_item->callback = popup_station_callback;

    for (i=0; i<config->num_channels; i++)
    {
	struct channel *chan;

	chan = get_channel(config->first_channel, i);
	station_item->path = malloc (128);
	station_item->callback_action = i+1;
	snprintf(station_item->path, 127, "/%s", chan->name);
	
	gtk_item_factory_create_items(popup_stations, 1, station_item, NULL);
	free(station_item->path);
    }
    free(station_item);

    if (temp)
	gtk_object_destroy(GTK_OBJECT(temp));
}

#else

GtkItemFactory *popup_menu, *popup_stations;
GtkAccelGroup *accel_group;

void initialize_popups()
{
    accel_group = gtk_accel_group_new ();

    popup_menu = gtk_item_factory_new (GTK_TYPE_MENU, "<main>", 
                                           accel_group);
    gtk_item_factory_create_items(popup_menu,NUM_ENTRIES,popup_menu_items,NULL);

    GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(
			popup_menu,"/Stereo"))->active = TRUE;
    GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(
			popup_menu,"/Alarm Clock"))->active = config->alarm_on;
    GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(
			popup_menu,"/Sleep Timer"))->active = config->timer_on;

    popup_stations = NULL;
    popups_reset_stations();
}

void popups_reset_stations()
{
    int i=1;
    GtkItemFactoryEntry *station_item;
    GtkItemFactory *temp;
    temp = popup_stations;

    popup_stations = gtk_item_factory_new (GTK_TYPE_MENU, "<main>",
							accel_group);

    gtk_menu_item_set_submenu(GTK_MENU_ITEM(gtk_item_factory_get_widget(
		popup_menu,"/Stations")),GTK_WIDGET(gtk_item_factory_get_widget(
		popup_stations,"")));

    station_item = malloc(sizeof (struct _GtkItemFactoryEntry));
    station_item->accelerator = NULL;
    station_item->item_type = NULL;
    station_item->callback = popup_station_callback;

    for (i=0; i<config->num_channels; i++)
    {
	struct channel *chan;

	chan = get_channel(config->first_channel, i);
	station_item->path = malloc (128);
	station_item->callback_action = i+1;
	snprintf(station_item->path, 127, "/%s", chan->name);
	
	gtk_item_factory_create_items(popup_stations, 1, station_item, NULL);
	free(station_item->path);
    }
    free(station_item);

    if (temp)
	gtk_object_destroy(GTK_OBJECT(temp));
}

static void popup_callback( GtkWidget *w, gpointer   data )
{
    gint cmd = (gint)data;

    switch(cmd)
    {
    case POPUP_PREFS:
	show_preferences ();
	break;
    case POPUP_ALARM:
	if (GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(
			popup_menu,"/Alarm Clock"))->active)
	    activate_alarm(config);
	else
	    deactivate_alarm(config);
	break;
    case POPUP_SLEEP:
	if (GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(
			popup_menu,"/Sleep Timer"))->active)
	    activate_timer(config);
	else
	    deactivate_timer(config);
	break;
    case POPUP_STEREO:
	callback_set_stereo(NULL,GTK_CHECK_MENU_ITEM(
		gtk_item_factory_get_widget(popup_menu,"/Stereo"))->active);
	break;
    case POPUP_MUTE:
	if (GTK_CHECK_MENU_ITEM(
		    gtk_item_factory_get_widget(popup_menu,"/Mute"))->active)
	    callback_mute(NULL, NULL);
	else
	    callback_unmute(NULL, NULL);
	break;
    case POPUP_EXIT:
	gtk_main_quit(); break;
    case POPUP_SAVE:
	save_config(config);
	break;
    default:
	printf("gtuner: Invalid command from popups recieved\n");
	break;
    }
}

#endif
