/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "gui.h"
#include "gtkdial.h"
#include <malloc.h>

extern void callback_set_volume (GtkWidget *window, struct volume_changer *vc);
extern struct radio_config *config;

static
void volume_vbar_callback (GtkWidget *window, struct volume_changer *vc)
{
    if (vc->ignore)
    {
        vc->ignore = FALSE;
        return;
    }
    vc->vol = 31-(int)GTK_ADJUSTMENT(vc->data.vvb->scale)->value;
    callback_set_volume(window, vc);
}

static
void volume_vbar_set(struct volume_changer *vc, int vol)
{
    if (vc->vol != vol) vc->ignore = TRUE;
    if (vc->data.vvb)
	gtk_adjustment_set_value(GTK_ADJUSTMENT(vc->data.vvb->scale), 31-vol);
}

static
void volume_hbar_set(struct volume_changer *vc, int vol)
{
    if (vc->vol != vol) vc->ignore = TRUE;
    if (vc->data.vhb)
	gtk_adjustment_set_value(GTK_ADJUSTMENT(vc->data.vhb->scale), vol);
}

void volume_vbar_new(struct volume_changer *vc)
{
    vc->data.vvb = malloc (sizeof(struct volume_bar_v));
    vc->data.vvb->scale = gtk_adjustment_new(31-vc->vol, 0,32,1,8,1);
    vc->set_vol = volume_vbar_set;

    vc->base = gtk_vscale_new(GTK_ADJUSTMENT (vc->data.vvb->scale));
    gtk_scale_set_draw_value(GTK_SCALE (vc->base), FALSE);

    gtk_widget_show(vc->base);

    gtk_signal_connect(GTK_OBJECT(vc->data.vvb->scale), "value_changed",
			(GtkSignalFunc)volume_vbar_callback ,vc);
}

static
void volume_hbar_callback (GtkWidget *window, struct volume_changer *vc)
{
    if (vc->ignore)
    {    
        vc->ignore = FALSE;
        return;    
    }    
    vc->vol = (int)GTK_ADJUSTMENT(vc->data.vvb->scale)->value;
    callback_set_volume(window, vc);
}

void volume_hbar_new(struct volume_changer *vc)
{
    vc->data.vhb = malloc (sizeof(struct volume_bar_h));
    vc->data.vhb->scale = gtk_adjustment_new(vc->vol, 0,32,1,8,1);
    vc->set_vol = volume_hbar_set;

    vc->base = gtk_hscale_new(GTK_ADJUSTMENT (vc->data.vvb->scale));
    gtk_scale_set_draw_value(GTK_SCALE (vc->base), FALSE);

    gtk_widget_show(vc->base);

    gtk_signal_connect(GTK_OBJECT(vc->data.vhb->scale), "value_changed",
                        (GtkSignalFunc)volume_hbar_callback ,vc);
}

static
void volume_pm_set(struct volume_changer *vc, int vol)
{
    vc->vol = vol;
}

static
void volume_pm_up(GtkWidget *win, struct volume_changer *vc)
{
    if (vc->vol >= 31) return;
    vc->vol++;
    callback_set_volume(win, vc);
}

static
void volume_pm_down(GtkWidget *win, struct volume_changer *vc)
{
    if (vc->vol <= 0) return;
    vc->vol--;
    callback_set_volume(win, vc);
}

void volume_plus_minus_new(struct volume_changer *vc, int horizontal)
{
    vc->data.vpm = malloc (sizeof(struct volume_plus_minus));
    vc->set_vol = volume_pm_set;
    vc->data.vpm->horizontal = horizontal;

    if (!config->pix.plus_xpm)
	vc->data.vpm->plus = gtk_button_new_with_label("+");
    else
	{
	    vc->data.vpm->plus = gtk_button_new();
	    vc->data.vpm->plus_xpm = gtk_pixmap_new( config->pix.plus_xpm,
					config->pix.plus_mask);
	    gtk_widget_show(vc->data.vpm->plus_xpm);
	    gtk_container_add(GTK_CONTAINER(vc->data.vpm->plus),
					vc->data.vpm->plus_xpm);
        }
    if (!config->pix.minus_xpm)
	vc->data.vpm->minus = gtk_button_new_with_label("-");
    else    
	{
	    vc->data.vpm->minus = gtk_button_new();
	    vc->data.vpm->minus_xpm = gtk_pixmap_new( config->pix.minus_xpm,
					config->pix.minus_mask);
	    gtk_widget_show(vc->data.vpm->minus_xpm);
	    gtk_container_add(GTK_CONTAINER(vc->data.vpm->minus),
					vc->data.vpm->minus_xpm);
	}

    gtk_widget_show(vc->data.vpm->plus);
    gtk_widget_show(vc->data.vpm->minus);

    if (horizontal)
    {
	vc->base = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(vc->base), vc->data.vpm->minus);
	gtk_container_add(GTK_CONTAINER(vc->base), vc->data.vpm->plus);
    }
    else
    {
	vc->base = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(vc->base), vc->data.vpm->plus);
	gtk_container_add(GTK_CONTAINER(vc->base), vc->data.vpm->minus);
    }
    gtk_widget_show(vc->base);

    gtk_signal_connect(GTK_OBJECT(vc->data.vpm->plus), "clicked",
			(GtkSignalFunc) volume_pm_up, vc);
    gtk_signal_connect(GTK_OBJECT(vc->data.vpm->minus), "clicked",
			(GtkSignalFunc) volume_pm_down, vc);
}

static
void volume_dial_set(struct volume_changer *vc, int vol)
{
    if (vc->vol != vol) vc->ignore = TRUE;
    if (vc->data.dial)
        gtk_adjustment_set_value(GTK_ADJUSTMENT(vc->data.dial->scale), vol);
}

static
void volume_dial_callback (GtkWidget *window, struct volume_changer *vc)
{
    if (vc->ignore)
    {
        vc->ignore = FALSE;
        return;
    }
    vc->vol = (int)GTK_ADJUSTMENT(vc->data.dial->scale)->value;
    callback_set_volume(window, vc);
}

void volume_dial_new(struct volume_changer *vc)
{
    vc->data.dial = malloc (sizeof(struct volume_dial));
    vc->data.dial->scale = gtk_adjustment_new(vc->vol, 0,32,1,8,1);
    vc->set_vol = volume_dial_set;

    vc->base = gtk_dial_new(GTK_ADJUSTMENT (vc->data.dial->scale));

    gtk_widget_show(vc->base);

    gtk_signal_connect(GTK_OBJECT(vc->data.dial->scale), "value_changed",
                        (GtkSignalFunc)volume_dial_callback ,vc);
}
