/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "hardware.h"
#include "callbacks.h"
#include <malloc.h>
#include <stdio.h>

struct CardPluginData *card_data;

GList *get_card_list()
{
    return card_data->card_list;
}

CardPlugin *get_current_card()
{
    return card_data->current_card_plugin;
}

void set_current_card(CardPlugin *new_card, int port)
{
    int result;
    if (card_data->current_card_plugin)
	card_data->current_card_plugin->cleanup();
    card_data->current_card_plugin = new_card;
    result = card_data->current_card_plugin->init(port);
    if (result != NO_ERROR)
    {
	printf("gtuner: plugin: %s: ",
		card_data->current_card_plugin->description);
	switch (result)
	{
	case PORT_ACCESS_DENIED:
		printf("access to port 0x%x denied", port); break;
	case CARD_NOT_SUPPORTED:
		printf("card not supported"); break;
	case DEVICE_NOT_PRESENT:
		printf("device not present"); break;
	case DEVICE_INITIALIZATION_FAIL:
		printf("device initialization error"); break;
	default:
		printf("unknown error");
	}
	printf("\n");
	card_data->current_card_plugin = NULL;
	card_data->card_on = FALSE;
    }
    else
	card_on();
    callback_set_freq (NULL, card_get_frequency());
}

void card_on  ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->card_on)
	    card_data->current_card_plugin->card_on();
    card_data->card_on = TRUE;
}

void card_off ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->card_off)
	    card_data->current_card_plugin->card_off();
    card_data->card_on = FALSE;
}

void card_mute ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->card_mute)
	    card_data->current_card_plugin->card_mute();
    card_data->muted = TRUE;
}

void card_unmute ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->card_unmute)
	    card_data->current_card_plugin->card_unmute();
    card_data->muted = FALSE;
}

void card_inc_volume ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->inc_volume)
	    card_data->current_card_plugin->inc_volume();
}

void card_dec_volume ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->dec_volume)
	    card_data->current_card_plugin->dec_volume();
}

void card_set_volume (int vol)
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->set_volume)
	    card_data->current_card_plugin->set_volume(vol);
}

int  card_get_volume ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->get_volume)
	    return card_data->current_card_plugin->get_volume();

    return 0;
}

void card_set_frequency  (double freq)
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->set_frequency)
	    card_data->current_card_plugin->set_frequency(freq);
    card_data->channel = -1;
}

void card_set_channel(struct channel *chan)
{
    if (chan)
    if (card_data->current_card_plugin)
        if (card_data->current_card_plugin->set_frequency)
	{
            card_data->current_card_plugin->set_frequency(chan->frequency);
	    card_data->channel = chan->number;
	}
}

double card_get_frequency ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->get_frequency)
	    return card_data->current_card_plugin->get_frequency();

    return 0;
}

void card_tune_up ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->tune_up)
	    card_data->current_card_plugin->tune_up();
}

void card_tune_down ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->tune_down)
	    card_data->current_card_plugin->tune_down();
}

void card_set_stereo (gboolean state)
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->set_stereo)
	    card_data->current_card_plugin->set_stereo(state);
    card_data->stereo = state;
}

gboolean  card_is_tuned ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->is_tuned)
	    return card_data->current_card_plugin->is_tuned();

    return FALSE;
}

gboolean  card_is_stereo ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->is_stereo)
	    return card_data->current_card_plugin->is_stereo();

    return FALSE;
}

double card_get_frequency_step ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->get_frequency_step)
	    return card_data->current_card_plugin->get_frequency_step();

    return 0;
}

double card_get_max_frequency ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->get_max_frequency)
	    return card_data->current_card_plugin->get_max_frequency();

    return 0;
}

double card_get_min_frequency ()
{
    if (card_data->current_card_plugin)
	if (card_data->current_card_plugin->get_min_frequency)
	    return card_data->current_card_plugin->get_min_frequency();

    return 0;
}

void card_about(int card)
{
    CardPlugin *cp;

    cp = g_list_nth(card_data->card_list, card)->data;
    if (cp)
	if (cp->about)
	    cp->about();
}

void card_configure(int card)
{
    CardPlugin *cp;

    cp = g_list_nth(card_data->card_list, card)->data;
    if (cp)
	if (cp->configure)
	    cp->configure();
}

void card_cleanup()
{
    if (card_data->current_card_plugin)
    {
	if (card_data->current_card_plugin->cleanup)
	    card_data->current_card_plugin->cleanup();
	card_data->current_card_plugin = NULL;
    }
}

void initialize_cards()
{
    card_data = malloc(sizeof(struct CardPluginData));
    card_data->card_list = NULL;
    card_data->current_card_plugin = NULL;
    card_data->channel = 0;
    card_data->muted = 0;
    card_data->card_on = 0;
    card_data->stereo = 0;
}
