/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "plugins.h"
#include "plugin_handler.h"
#include "../hardware.h"
#include "../conf.h"
#include <glib.h>
#include <gtk/gtk.h>
#include <malloc.h>
#include <dlfcn.h>
#include <stdio.h>
#include <dirent.h>
#include <sys/stat.h>

extern struct CardPluginData *card_data;
extern struct radio_config *config;
GList *plugin_list=0;
GList *enabled_list=0;

void add_plugin(gchar *filename);

void scan_plugins(char *dirname)
{
	gchar *filename,*ext;
	DIR *dir;
	struct dirent *ent;
	struct stat statbuf;

	dir=opendir(dirname);
	if(dir)
	{
		while(ent=readdir(dir))
		{
			filename=(gchar *)g_malloc(strlen(dirname)+strlen(ent->d_name)+2);
			sprintf(filename,"%s/%s",dirname,ent->d_name);
			if(!stat(filename,&statbuf))
			{
				if(S_ISREG(statbuf.st_mode))
				{
					if(ext=(char*)strrchr(ent->d_name,'.'))
					{
						if(!strcmp(ext,".so"))
							add_plugin(filename);
						else
							g_free(filename);
					}
					else
						g_free(filename);
				}
				else
					g_free(filename);
			}
			else
				g_free(filename);
		}
	}
/*      else
	{
		perror ("opendir");
	} */ /* prevent error printout if a plugin dir can't be opened */
}

gboolean plugin_enabled(GenericPlugin *p)
{
    return (g_list_find (enabled_list, p) != 0);
}

void enable_plugin(int id, gboolean enable)
{
    GenericPlugin *p;
    gboolean found;
    if (id > g_list_length(plugin_list))
    {
	printf("gtuner: plugins: invalid plugin: %i\n",id);
	return;
    }
    p = g_list_nth(plugin_list,id)->data;
    found = plugin_enabled(p);
    if (enable)
    {
	if (found) {printf("gtuner: plugins: plugin already enabled\n");return;}
	if (p->init)
	{
	    p->init();
	    enabled_list = g_list_append(enabled_list, p);
	} else
	    printf("gtuner: plugins: plugin does not have a init method\n");
    } else
    {
	if (!found) {printf("gtuner: plugins: plugin not enabled\n");return;}
	if (p->cleanup)
	{
	    p->cleanup();
	    enabled_list = g_list_remove(enabled_list, p);
	} else
	    printf("gtuner: plugins: plugin does not have a cleanup method\n");
    }
}

void initialize_plugins()
{
    GList *node, *node2;
    plugin_list = 0;

    printf("gtuner: initializing plugins...\n");
    scan_plugins(PLUGINDIR);

    node = plugin_list;
    if (config->load_plugins)
    while (node)
    {
	node2 = config->plugin_load_list;
	while (node2)
	{
	    if (strcmp((char*)node2->data,
			((GenericPlugin*)node->data)->filename)==0)
	    {
		if (((GenericPlugin*)node->data)->init)
		{
		    ((GenericPlugin*)node->data)->init();
		    enabled_list = g_list_append(enabled_list, node->data);
		}
	    }
	    node2 = node2->next;
	}
	node = node->next;
    }
}

void add_plugin(gchar *filename)
{
    void *plugin;
    void *(*gpi)(void);

    if ((plugin=dlopen(filename, RTLD_NOW)))
    {
	if ((gpi=dlsym(plugin,"get_gplugin_info")))
	{
	    GenericPlugin *p;
	    p = (GenericPlugin*)gpi();
	    p->handle = plugin;
	    p->filename = filename;
	    plugin_list = g_list_prepend(plugin_list, p);
	}
	else
	if ((gpi=dlsym(plugin,"get_cplugin_info")))
	{
	    CardPlugin *p;
	    p = (CardPlugin*)gpi();
	    p->handle = plugin;
	    p->filename = filename;
	    card_data->card_list = g_list_prepend(card_data->card_list, p);
	}
	else
	{
	    fprintf(stderr, "Unknown plugin in register loop (%s)\n", filename);
	    dlclose(plugin);
	}
    }
    else
	fprintf(stderr,"%s\n",dlerror());

    /* These are so that the plugin commands don't get ignored by the linker */
	if (plugin_set_channel)
	if (plugin_channel_up)
	if (plugin_channel_down)
	if (plugin_scan)
	if (plugin_toggle_mute)
	if (plugin_get_volume)
	if (plugin_set_volume)
	if (plugin_get_alarm_state)
	if (plugin_set_alarm_state)
	if (plugin_quit_program)
	if (plugin_set_timer)
	if (plugin_get_timer_state)
	if (plugin_set_stereo)
	if (plugin_is_stereo)
	    printf("");

}
