/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "plugins.h"
#include "../conf.h"
#include "../channel.h"
#include "../callbacks.h"
#include "../gui/gui.h"
#include "../alarm.h"
#include "../hardware.h"

extern struct radio_config *config;
extern struct CardPluginData *card_data;

void    plugin_set_channel(int chan)
{
    struct channel *channel;

    if ((chan >= 0) && (chan < config->num_channels))
	{
	    channel = get_channel(config->first_channel, chan);
	    callback_set_channel(NULL, channel);
	}
}

void	plugin_channel_up()
{
    if (card_data->channel == -1)
	plugin_set_channel(0);
    else
    {
	if (card_data->channel >= config->num_channels)
	    card_data->channel = 0;
	else
	    card_data->channel++;
        plugin_set_channel(card_data->channel);
    }
}

void	plugin_channel_down()
{
    if (card_data->channel == -1)
	plugin_set_channel(0);
    else
    {   
	if (card_data->channel == 0)
	    card_data->channel = config->num_channels-1;
	else
	    card_data->channel--;
	plugin_set_channel(card_data->channel);
    }
}

void    plugin_scan(int direction)
{
    if ((direction * direction) == 1) /* 1 or -1 */
	callback_scan(card_get_frequency_step ()*(double)direction);
}

void    plugin_toggle_mute()
{
    if (!card_data->muted)
	callback_mute(NULL, NULL);
    else
	callback_unmute(NULL, NULL);
}

int     plugin_get_volume()
{
    return card_get_volume ();
}

void    plugin_set_volume(int vol)
{
    struct volume_changer vc;

    if ((vol < 0) || (vol > GLOBAL_MAX_VOL))
	return;

    vc.vol = vol;
    callback_set_volume(NULL, &vc);
}

int     plugin_get_alarm_state()
{
    return config->alarm.active;
}

void    plugin_set_alarm_state(int state)
{
    if (state)
	activate_alarm(config);
    else
	deactivate_alarm(config);
}

void    plugin_quit_program()
{
    gtk_main_quit();
}

void    plugin_set_timer(int state)
{
    if (state)
	activate_timer(config);
    else
	deactivate_timer(config);
}

gboolean plugin_get_timer_state()
{
    return config->sleep.active;
}

void	plugin_set_stereo(gboolean state)
{
    callback_set_stereo(NULL, state);
}

gboolean plugin_is_stereo()
{
    return card_data->stereo;
}
