/* gtuner
	Copyright (c) 1998-1999 Carl van Schaik (carl@leg.uct.ac.za)
	cRadio, kTuner, gtuner (c) 1998-1999 Carl van Schaik

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef PLUGINS_H
#define PLUGINS_H

#include <glib.h>
#include "../configfile.h"

#define GLOBAL_MAX_VOL 32
extern GList *plugin_list, *enabled_list;

typedef struct
{
    void	*handle;        	/* Filled in by gtuner */
    char	*filename;      	/* Filled in by gtuner */
    char	*description;   	/* The description of the plugin */
    void	(*init)(void);		/* Called when plugin is initialized */
    void	(*about)(void);		/* Display the about box */
    void	(*configure)(void);	/* Show the configuration box */
    void	(*cleanup)(void);	/* Called when plugin is removed */
} GenericPlugin;

/* Objects that a control plugin can get access to and control */
void	plugin_set_channel(int chan);
void	plugin_channel_up();
void	plugin_channel_down();
void	plugin_scan(int direction);
void	plugin_toggle_mute();
int	plugin_get_volume();
void	plugin_set_volume(int vol);
int	plugin_get_alarm_state();
void	plugin_set_alarm_state(int state);
void	plugin_quit_program();
void	plugin_set_timer(int state);
gboolean plugin_get_timer_state();
void	plugin_set_stereo(gboolean state);
gboolean plugin_is_stereo();

typedef enum {
    NO_ERROR,
    PORT_ACCESS_DENIED,
    CARD_NOT_SUPPORTED,
    DEVICE_NOT_PRESENT,
    DEVICE_INITIALIZATION_FAIL
} radioError;

typedef struct
{
    void	*handle;        	/* Filled in by gtuner */
    char	*filename;      	/* Filled in by gtuner */
    char	*description;   	/* The description of the plugin */
    radioError	(*init)(int port);	/* Called when plugin is initialized */
    void	(*about)(void);		/* Display the about box */
    void	(*configure)(void);	/* Show the configuration box */
    void	(*cleanup)(void);	/* Called when plugin is removed */

    void	(*card_on) ();
    void	(*card_off) ();
    void	(*card_mute) ();
    void	(*card_unmute) ();
    void	(*inc_volume) ();
    void	(*dec_volume) ();
    void	(*set_volume) (int);
    int 	(*get_volume) ();
    void	(*set_frequency)  (double);
    double	(*get_frequency) ();
    void	(*tune_up) ();
    void	(*tune_down) ();
    void	(*set_stereo) (gboolean);
    gboolean	(*is_tuned) ();
    gboolean	(*is_stereo) ();
    double	(*get_frequency_step) ();
    double	(*get_max_frequency) ();
    double	(*get_min_frequency) ();
} CardPlugin;

extern gboolean plugin_enabled(GenericPlugin*);
extern void enable_plugin(int id, gboolean enable);

#endif
